@@ -2,7 +2,7 @@
 """
 fitpack (dierckx in netlib) --- A Python-C wrapper to FITPACK (by P. Dierckx).
         FITPACK is a collection of FORTRAN programs for CURVE and SURFACE
-        FITTING with SPLINES and TENSOR PRODUCT SPLINES. 
+        FITTING with SPLINES and TENSOR PRODUCT SPLINES.
 
 See
  http://www.cs.kuleuven.ac.be/cwis/research/nalag/research/topics/fitpack.html
@@ -10,8 +10,8 @@ or
  http://www.netlib.org/dierckx/index.html
 
 Copyright 2002 Pearu Peterson all rights reserved,
-Pearu Peterson <pearu@cens.ioc.ee>          
-Permission to use, modify, and distribute this software is given under the 
+Pearu Peterson <pearu@cens.ioc.ee>
+Permission to use, modify, and distribute this software is given under the
 terms of the SciPy (BSD style) license.  See LICENSE.txt that came with
 this distribution for specifics.
 
@@ -30,9 +30,15 @@ TODO: Make interfaces to the following fitpack functions:
 
 __all__ = ['splrep', 'splprep', 'splev', 'splint', 'sproot',
                          'spalde', 'bisplrep', 'bisplev']
-__version__ = "$Revision: 1.7 $"[10:-1]
+__version__ = "$Revision: 2186 $"[10:-1]
 import _fitpack
-from common_routines import *
+from numpy import atleast_1d, array, ones, zeros, sqrt, ravel, transpose, \
+     dot, sin, cos, pi, arange, empty, int32
+myasarray = atleast_1d
+
+# Try to replace _fitpack interface with
+#  f2py-generated version
+import dfitpack
 
 _iermess = {0:["""\
     The spline has a residual sum of squares fp such that abs(fp-s)/s<=0.001""",None],
@@ -43,7 +49,7 @@ _iermess = {0:["""\
     fp gives the upper bound fp0 for the smoothing factor s""",None],
                1:["""\
     The required storage space exceeds the available strorage space.
-    Probably causes: nest to small or s is too small. (fp>s)""",ValueError],
+    Probably causes: data (x,y) size is too small or smoothing parameter s is too small (fp>s).""",ValueError],
                2:["""\
     A theoretically impossible results when finding a smoothin spline
     with fp = s. Probably causes: s too small. (abs(fp-s)/s>0.001)""",ValueError],
@@ -55,6 +61,7 @@ _iermess = {0:["""\
     Error on input data""",ValueError],
                'unknown':["""\
     An error occured""",TypeError]}
+
 _iermess2 = {0:["""\
     The spline has a residual sum of squares fp such that abs(fp-s)/s<=0.001""",None],
             -1:["""\
@@ -93,8 +100,8 @@ _iermess2 = {0:["""\
             'unknown':["""\
     An error occured""",TypeError]}
 
-_parcur_cache = {'t': array([],'d'), 'wrk': array([],'d'), 'iwrk':array([],'i'),
-                 'u': array([],'d'),'ub':0,'ue':1}
+_parcur_cache = {'t': array([],float), 'wrk': array([],float),
+                 'iwrk':array([],int32), 'u': array([],float),'ub':0,'ue':1}
 
 def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
             full_output=0,nest=None,per=0,quiet=1):
@@ -108,7 +115,7 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
 
     Inputs:
 
-      x -- A list of sample vector arrays representing the curve. 
+      x -- A list of sample vector arrays representing the curve.
       u -- An array of parameter values.  If not given, these values are
            calculated automatically as (M = len(x[0])):
            v[0] = 0
@@ -126,7 +133,7 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
               If task=-1 find the weighted least square spline for a given set of
                 knots, t.
       s -- A smoothing condition.  The amount of smoothness is determined by
-           satisfying the conditions: sum((w * (y - g))**2) <= s where
+           satisfying the conditions: sum((w * (y - g))**2,axis=0) <= s where
            g(x) is the smoothed interpolation of (x,y).  The user can use s to
            control the tradeoff between closeness and smoothness of fit.  Larger
            s means more smoothing while smaller values of s indicate less
@@ -149,20 +156,21 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
       tck -- (t,c,k) a tuple containing the vector of knots, the B-spline
              coefficients, and the degree of the spline.
       u -- An array of the values of the parameter.
-             
+
       fp -- The weighted sum of squared residuals of the spline approximation.
       ier -- An integer flag about splrep success.  Success is indicated
              if ier<=0. If ier in [1,2,3] an error occurred but was not raised.
              Otherwise an error is raised.
-      msg -- A message corresponding to the integer flag, ier.          
- 
+      msg -- A message corresponding to the integer flag, ier.
+
     Remarks:
 
-      SEE splev for evaluation of the spline and its derivatives.      
+      SEE splev for evaluation of the spline and its derivatives.
     """
     if task<=0:
-        _parcur_cache = {'t': array([],'d'), 'wrk': array([],'d'),
-                         'iwrk':array([],'i'),'u': array([],'d'),'ub':0,'ue':1}
+        _parcur_cache = {'t': array([],float), 'wrk': array([],float),
+                         'iwrk':array([],int32),'u': array([],float),
+                         'ub':0,'ue':1}
     x=myasarray(x)
     idim,m=x.shape
     if per:
@@ -171,7 +179,7 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
                 if quiet<2:print 'Warning: Setting x[%d][%d]=x[%d][0]'%(i,m,i)
                 x[i][-1]=x[i][0]
     if not 0<idim<11: raise TypeError,'0<idim<11 must hold'
-    if w is None: w=ones(m,'d')
+    if w is None: w=ones(m,float)
     else: w=myasarray(w)
     ipar=(u is not None)
     if ipar:
@@ -180,7 +188,7 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
         else: _parcur_cache['ub']=ub
         if ue is None: _parcur_cache['ue']=u[-1]
         else: _parcur_cache['ue']=ue
-    else: _parcur_cache['u']=zeros(m,'d')
+    else: _parcur_cache['u']=zeros(m,float)
     if not (1<=k<=5): raise TypeError, '1<=k=%d<=5 must hold'%(k)
     if not (-1<=task<=1): raise TypeError, 'task must be either -1,0, or 1'
     if (not len(w)==m) or (ipar==1 and (not len(u)==m)):
@@ -193,7 +201,7 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
     if task==-1 and n<2*k+2:
         raise TypeError, 'There must be at least 2*k+2 knots for task=-1'
     if m<=k: raise TypeError, 'm>k must hold'
-    if nest is None: nest=m/2
+    if nest is None: nest=m+2*k
 
     if (task>=0 and s==0) or (nest<0):
         if per: nest=m+2*k
@@ -236,9 +244,10 @@ def splprep(x,w=None,u=None,ub=None,ue=None,k=3,task=0,s=None,t=None,
     else:
         return tcku
 
-_curfit_cache = {'t': array([],'d'), 'wrk': array([],'d'), 'iwrk':array([],'i')}
-def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
-           full_output=0,nest=None,per=0,quiet=1):
+_curfit_cache = {'t': array([],float), 'wrk': array([],float),
+                 'iwrk':array([],int32)}
+def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=1e-3,t=None,
+           full_output=0,per=0,quiet=1):
     """Find the B-spline representation of 1-D curve.
 
     Description:
@@ -261,13 +270,15 @@ def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
            Even order splines should be avoided especially with small s values.
            1 <= k <= 5
       task -- If task==0 find t and c for a given smoothing factor, s.
-              If task==1 find t and c for another value of the smoothing factor,
-                s. There must have been a previous call with task=0 or task=1
-                for the same set of data.
-              If task=-1 find the weighted least square spline for a given set of
-                knots, t.
+              If task==1 find t and c for another value of the
+                smoothing factor, s. There must have been a previous
+                call with task=0 or task=1 for the same set of data
+                (t will be stored an used internally)
+              If task=-1 find the weighted least square spline for
+                a given set of knots, t.  These should be interior knots
+                as knots on the ends will be added automatically.
       s -- A smoothing condition.  The amount of smoothness is determined by
-           satisfying the conditions: sum((w * (y - g))**2) <= s where
+           satisfying the conditions: sum((w * (y - g))**2,axis=0) <= s where
            g(x) is the smoothed interpolation of (x,y).  The user can use s to
            control the tradeoff between closeness and smoothness of fit.  Larger
            s means more smoothing while smaller values of s indicate less
@@ -275,10 +286,10 @@ def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
            weights represent the inverse of the standard-deviation of y, then a
            good s value should be found in the range (m-sqrt(2*m),m+sqrt(2*m))
            where m is the number of datapoints in x, y, and w.
-      t -- The knots needed for task=-1.
+           default : s=m-sqrt(2*m)
+      t -- The knots needed for task=-1.  If given then task is automatically
+           set to -1.
       full_output -- If non-zero, then return optional outputs.
-      nest -- An over-estimate of the total number of knots of the spline to
-              help in determining the storage space.  By default nest=m/2.
       per -- If non-zero, data points are considered periodic with period
              x[m-1] - x[0] and a smooth periodic spline approximation is returned.
              Values of y[m-1] and w[m-1] are not used.
@@ -288,7 +299,7 @@ def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
 
       tck -- (t,c,k) a tuple containing the vector of knots, the B-spline
              coefficients, and the degree of the spline.
-             
+
       fp -- The weighted sum of squared residuals of the spline approximation.
       ier -- An integer flag about splrep success.  Success is indicated if
              ier<=0. If ier in [1,2,3] an error occurred but was not raised.
@@ -297,51 +308,64 @@ def splrep(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
 
     Remarks:
 
-      SEE splev for evaluation of the spline and its derivatives.      
+      See splev for evaluation of the spline and its derivatives.
+      
+    Example:
+        
+      x = linspace(0, 10, 10)
+      y = sin(x)
+      tck = splrep(x, y)
+      x2 = linspace(0, 10, 200)
+      y2 = splev(x2, tck)
+      plot(x, y, 'o', x2, y2)
+      
     """
     if task<=0:
-        _curfit_cache = {'t': array([],'d'), 'wrk': array([],'d'),
-                         'iwrk':array([],'i')}
+        _curfit_cache = {}
     x,y=map(myasarray,[x,y])
     m=len(x)
-    if w is None: w=ones(m,'d')
+    if w is None: w=ones(m,float)
     else: w=myasarray(w)
     if not len(w) == m: raise TypeError,' len(w)=%d is not equal to m=%d'%(len(w),m)
-    if xb is None: xb=x[0]
-    if xe is None: xe=x[-1]
-    if not (-1<=task<=1): raise TypeError, 'task must be either -1,0, or 1'
-    if s is None: s=m-sqrt(2*m)
-    if t is None and task==-1: raise TypeError, 'Knots must be given for task=-1'
-    if t is not None: _curfit_cache['t']=myasarray(t)
-    n=len(_curfit_cache['t'])
-    if task==-1 and n<2*k+2:
-        raise TypeError, 'There must be at least 2*k+2 knots for task=-1'
     if (m != len(y)) or (m != len(w)):
         raise TypeError, 'Lengths of the first three arguments (x,y,w) must be equal'
     if not (1<=k<=5):
         raise TypeError, 'Given degree of the spline (k=%d) is not supported. (1<=k<=5)'%(k)
-    if m<=k: raise TypeError, 'm>k must hold'
-    if nest is None: nest=m/2
-    if nest<0:
-        if per: nest=m+2*k
-        else: nest=m+k+1
-    nest=max(nest,2*k+3)
-    if task>=0 and s==0:
-        if per: nest=m+2*k
-        else: nest=m+k+1
-    if task==-1:
-        _curfit_cache['t']=myasarray(t)
-        if not (2*k+2<=len(t)<=min(nest,m+k+1)):
-            raise TypeError, 'Number of knots n is not acceptable (2*k+2<=n<=min(nest,m+l+1))'
-    t=_curfit_cache['t']
-    wrk=_curfit_cache['wrk']
-    iwrk=_curfit_cache['iwrk']
-    t,c,o = _fitpack._curfit(x,y,w,xb,xe,k,task,s,t,nest,wrk,iwrk,per)
-    _curfit_cache['t']=t
-    _curfit_cache['wrk']=o['wrk']
-    _curfit_cache['iwrk']=o['iwrk']
-    ier,fp=o['ier'],o['fp']
-    tck = [t,c,k]
+    if m<=k: raise TypeError, 'm>k must hold'     
+    if xb is None: xb=x[0]
+    if xe is None: xe=x[-1]
+    if not (-1<=task<=1): raise TypeError, 'task must be either -1,0, or 1'
+    if s is None: s = m-sqrt(2*m)
+    if t is not None:
+        task = -1
+    if task == -1:
+        if t is None: raise TypeError, 'Knots must be given for task=-1'
+        numknots = len(t)
+        _curfit_cache['t'] = empty((numknots + 2*k+2,),float)
+        _curfit_cache['t'][k+1:-k-1] = t
+        nest = len(_curfit_cache['t'])
+    elif task == 0:
+        if per:
+            nest = max(m+2*k,2*k+3)
+        else:
+            nest = max(m+k+1,2*k+3)
+        t = empty((nest,),float)
+        _curfit_cache['t'] = t
+    if task <= 0:
+        _curfit_cache['wrk'] = empty((m*(k+1)+nest*(7+3*k),),float)
+        _curfit_cache['iwrk'] = empty((nest,),int32)
+    try:
+        t=_curfit_cache['t']
+        wrk=_curfit_cache['wrk']
+        iwrk=_curfit_cache['iwrk']
+    except KeyError:
+        raise TypeError, "must call with task=1 only after"\
+              " call with task=0,-1"
+    if not per:
+        n,c,fp,ier = dfitpack.curfit(task, x, y, w, t, wrk, iwrk, xb, xe, k, s)
+    else:
+        n,c,fp,ier = dfitpack.percur(task, x, y, w, t, wrk, iwrk, k, s)
+    tck = [t[:n],c[:n-k-1],k]
     if ier<=0 and not quiet:
         print _iermess[ier][0]
         print "\tk=%d n=%d m=%d fp=%f s=%f"%(k,len(t),m,fp,s)
@@ -365,7 +389,7 @@ def _ntlist(l): # return non-trivial list
     return l
     #if len(l)>1: return l
     #return l[0]
-    
+
 def splev(x,tck,der=0):
     """Evaulate a B-spline and its derivatives.
 
@@ -387,9 +411,9 @@ def splev(x,tck,der=0):
 
     Outputs: (y, )
 
-      y -- an array of values representing the spline function or curve.  If tck
-           was returned from splrep, then this is a list of arrays representing
-           the curve in N-dimensional space.
+      y -- an array of values representing the spline function or curve.
+           If tck was returned from splrep, then this is a list of arrays
+           representing the curve in N-dimensional space.
     """
     t,c,k=tck
     try:
@@ -425,7 +449,7 @@ def splint(a,b,tck,full_output=0):
       wrk -- An array containing the integrals of the normalized B-splines defined
              on the set of knots.
 
-    """      
+    """
     t,c,k=tck
     try: c[0][0];return _ntlist(map(lambda c,a=a,b=b,t=t,k=k:splint(a,b,[t,c,k]),c))
     except: pass
@@ -486,7 +510,7 @@ def spalde(x,tck):
 
       results -- An array (or a list of arrays) containing all derivatives
                  up to order k inclusive for each point x.
-    """    
+    """
     t,c,k=tck
     try:
         c[0][0]
@@ -507,10 +531,11 @@ def spalde(x,tck):
 #def _curfit(x,y,w=None,xb=None,xe=None,k=3,task=0,s=None,t=None,
 #           full_output=0,nest=None,per=0,quiet=1):
 
-_surfit_cache = {'tx': array([],'d'),'ty': array([],'d'),
-                 'wrk': array([],'d'), 'iwrk':array([],'i')}
-def bisplrep(x,y,z,w=None,xb=None,xe=None,yb=None,ye=None,kx=3,ky=3,task=0,s=None,
-             eps=1e-16,tx=None,ty=None,full_output=0,nxest=None,nyest=None,quiet=1):
+_surfit_cache = {'tx': array([],float),'ty': array([],float),
+                 'wrk': array([],float), 'iwrk':array([],int32)}
+def bisplrep(x,y,z,w=None,xb=None,xe=None,yb=None,ye=None,kx=3,ky=3,task=0,
+             s=None,eps=1e-16,tx=None,ty=None,full_output=0,
+             nxest=None,nyest=None,quiet=1):
     """Find a bivariate B-spline representation of a surface.
 
     Description:
@@ -522,9 +547,9 @@ def bisplrep(x,y,z,w=None,xb=None,xe=None,yb=None,ye=None,kx=3,ky=3,task=0,s=Non
 
       x, y, z -- Rank-1 arrays of data points.
       w -- Rank-1 array of weights. By default w=ones(len(x)).
-      xb, xe -- End points of approximation interval in x. 
+      xb, xe -- End points of approximation interval in x.
       yb, ye -- End points of approximation interval in y.
-                By default xb, xe, yb, ye = x[0], x[-1], y[0], y[-1]
+                By default xb, xe, yb, ye = x.min(), x.max(), y.min(), y.max()
       kx, ky -- The degrees of the spline (1 <= kx, ky <= 5).  Third order
                 (kx=ky=3) is recommended.
       task -- If task=0, find knots in x and y and coefficients for a given
@@ -533,17 +558,18 @@ def bisplrep(x,y,z,w=None,xb=None,xe=None,yb=None,ye=None,kx=3,ky=3,task=0,s=Non
                 smoothing factor, s.  bisplrep must have been previously called
                 with task=0 or task=1.
               If task=-1, find coefficients for a given set of knots tx, ty.
-      s -- A non-negative smoothing factor.  If weights correspond to the inverse
-           of the standard-deviation of the errors in z, then a good s-value
-           should be found in the range (m-sqrt(2*m),m+sqrt(2*m)) where m=len(x)
-      eps -- A threshold for determining the effective rank of an over-determined
-             linear system of equations (0 < eps < 1) --- not likely to need
-             changing.
+      s -- A non-negative smoothing factor.  If weights correspond
+           to the inverse of the standard-deviation of the errors in z,
+           then a good s-value should be found in the range
+           (m-sqrt(2*m),m+sqrt(2*m)) where m=len(x)
+      eps -- A threshold for determining the effective rank of an
+             over-determined linear system of equations (0 < eps < 1)
+             --- not likely to need changing.
       tx, ty -- Rank-1 arrays of the knots of the spline for task=-1
       full_output -- Non-zero to return optional outputs.
-      nxest, nyest -- Over-estimates of the total number of knots.  If None then
-                      nxest = max(kx+sqrt(m/2),2*kx+3),
-                      nyest = max(ky+sqrt(m/2),2*ky+3)
+      nxest, nyest -- Over-estimates of the total number of knots.
+                      If None then nxest = max(kx+sqrt(m/2),2*kx+3),
+                                   nyest = max(ky+sqrt(m/2),2*ky+3)
       quiet -- Non-zero to suppress printing of messages.
 
     Outputs: (tck, {fp, ier, msg})
@@ -561,32 +587,32 @@ def bisplrep(x,y,z,w=None,xb=None,xe=None,yb=None,ye=None,kx=3,ky=3,task=0,s=Non
     Remarks:
 
       SEE bisplev to evaluate the value of the B-spline given its tck
-      representation.           
+      representation.
     """
     x,y,z=map(myasarray,[x,y,z])
     x,y,z=map(ravel,[x,y,z])  # ensure 1-d arrays.
     m=len(x)
-    if not (m==len(y)==len(z)): raise TypeError, 'len(x)==len(y)==len(z) must hold.'  
-    if w is None: w=ones(m,'d')
+    if not (m==len(y)==len(z)): raise TypeError, 'len(x)==len(y)==len(z) must hold.'
+    if w is None: w=ones(m,float)
     else: w=myasarray(w)
     if not len(w) == m: raise TypeError,' len(w)=%d is not equal to m=%d'%(len(w),m)
-    if xb is None: xb=x[0]
-    if xe is None: xe=x[-1]
-    if yb is None: yb=y[0]
-    if ye is None: ye=y[-1]
+    if xb is None: xb=x.min()
+    if xe is None: xe=x.max()
+    if yb is None: yb=y.min()
+    if ye is None: ye=y.max()
     if not (-1<=task<=1): raise TypeError, 'task must be either -1,0, or 1'
     if s is None: s=m-sqrt(2*m)
     if tx is None and task==-1: raise TypeError, 'Knots_x must be given for task=-1'
-    if tx is not None: _curfit_cache['tx']=myasarray(tx)
+    if tx is not None: _surfit_cache['tx']=myasarray(tx)
     nx=len(_surfit_cache['tx'])
     if ty is None and task==-1: raise TypeError, 'Knots_y must be given for task=-1'
-    if ty is not None: _curfit_cache['ty']=myasarray(ty)
+    if ty is not None: _surfit_cache['ty']=myasarray(ty)
     ny=len(_surfit_cache['ty'])
     if task==-1 and nx<2*kx+2:
         raise TypeError, 'There must be at least 2*kx+2 knots_x for task=-1'
     if task==-1 and ny<2*ky+2:
         raise TypeError, 'There must be at least 2*ky+2 knots_x for task=-1'
-    if not ((1<=kx<=5) and (1<=ky<=5)): 
+    if not ((1<=kx<=5) and (1<=ky<=5)):
         raise TypeError, 'Given degree of the spline (kx,ky=%d,%d) is not supported. (1<=k<=5)'%(kx,ky)
     if m<(kx+1)*(ky+1): raise TypeError, 'm>=(kx+1)(ky+1) must hold'
     if nxest is None: nxest=kx+sqrt(m/2)
@@ -683,7 +709,7 @@ if __name__ == "__main__":
         runtest=map(string.atoi,sys.argv[1:])
     put=sys.stdout.write
     def norm2(x):
-        return matrixmultiply(transpose(x),x)
+        return dot(transpose(x),x)
     def f1(x,d=0):
         if d is None: return "sin"
         if x is None: return "sin(x)"
@@ -701,8 +727,8 @@ if __name__ == "__main__":
     def test1(f=f1,per=0,s=0,a=0,b=2*pi,N=20,at=0,xb=None,xe=None):
         if xb is None: xb=a
         if xe is None: xe=b
-        x=a+(b-a)*arange(N+1,typecode='d')/float(N)    # nodes
-        x1=a+(b-a)*arange(1,N,typecode='d')/float(N-1) # middle points of the nodes
+        x=a+(b-a)*arange(N+1,dtype=float)/float(N)    # nodes
+        x1=a+(b-a)*arange(1,N,dtype=float)/float(N-1) # middle points of the nodes
         v,v1=f(x),f(x1)
         nk=[]
         for k in range(1,6):
@@ -731,7 +757,7 @@ if __name__ == "__main__":
               ia=0,ib=2*pi,dx=0.2*pi):
         if xb is None: xb=a
         if xe is None: xe=b
-        x=a+(b-a)*arange(N+1,typecode='d')/float(N)    # nodes
+        x=a+(b-a)*arange(N+1,dtype=float)/float(N)    # nodes
         v=f(x)
         nk=[]
         for k in range(1,6):
@@ -758,7 +784,7 @@ if __name__ == "__main__":
               ia=0,ib=2*pi,dx=0.2*pi):
         if xb is None: xb=a
         if xe is None: xe=b
-        x=a+(b-a)*arange(N+1,typecode='d')/float(N)    # nodes
+        x=a+(b-a)*arange(N+1,dtype=float)/float(N)    # nodes
         v=f(x)
         nk=[]
         print "  k  :     Roots of s(x) approx %s  x in [%s,%s]:"%\
@@ -770,8 +796,8 @@ if __name__ == "__main__":
               ia=0,ib=2*pi,dx=0.2*pi):
         if xb is None: xb=a
         if xe is None: xe=b
-        x=a+(b-a)*arange(N+1,typecode='d')/float(N)    # nodes
-        x1=a+(b-a)*arange(1,N,typecode='d')/float(N-1) # middle points of the nodes
+        x=a+(b-a)*arange(N+1,dtype=float)/float(N)    # nodes
+        x1=a+(b-a)*arange(1,N,dtype=float)/float(N-1) # middle points of the nodes
         v,v1=f(x),f(x1)
         nk=[]
         print " u = %s   N = %d"%(`round(dx,3)`,N)
@@ -798,8 +824,8 @@ if __name__ == "__main__":
         xy.shape=sh[0]*sh[1],sh[2]
         return transpose(xy)
     def test5(f=f2,kx=3,ky=3,xb=0,xe=2*pi,yb=0,ye=2*pi,Nx=20,Ny=20,s=0):
-        x=xb+(xe-xb)*arange(Nx+1,typecode='d')/float(Nx)
-        y=yb+(ye-yb)*arange(Ny+1,typecode='d')/float(Ny)
+        x=xb+(xe-xb)*arange(Nx+1,dtype=float)/float(Nx)
+        y=yb+(ye-yb)*arange(Ny+1,dtype=float)/float(Ny)
         xy=makepairs(x,y)
         tck=bisplrep(xy[0],xy[1],f(xy[0],xy[1]),s=s,kx=kx,ky=ky)
         tt=[tck[0][kx:-kx],tck[1][ky:-ky]]
@@ -849,10 +875,3 @@ if __name__ == "__main__":
 \tTests of bisplrep, bisplev
 ******************************************"""
         test5()
-
-
-
-
-
-
-
