@@ -3,7 +3,7 @@
 # io - Data input and output
 #
 
-from info_io import __doc__
+from info import __doc__
 
 from numpyio import packbits, unpackbits, bswap, fread, fwrite, \
      convert_objectarray
@@ -13,3 +13,7 @@ from data_store import *
 from pickler import *
 
 from mmio import mminfo,mmread,mmwrite
+
+__all__ = filter(lambda s:not s.startswith('_'),dir())
+from numpy.testing import ScipyTest
+test = ScipyTest().test
