@@ -1,40 +1,46 @@
+## Automatically adapted for scipy Oct 05, 2005 by convertcode.py
+
 # Author: Travis Oliphant
 
-from MLab import squeeze
-from scipy_base import *
-from scipy_base.fastumath import *
-import numpyio
 import struct, os, sys
 import types
+from tempfile import mkstemp
+import zlib
+
+from numpy import array, asarray, empty, obj2sctype, product, reshape, \
+    squeeze, transpose, zeros, vstack, ndarray, shape, diff, where, uint8, \
+    atleast_1d
+import numpyio
+
 try:
     import scipy.sparse
     have_sparse = 1
 except ImportError:
     have_sparse = 0
 
-if sys.version_info[0] < 2 or sys.version_info[1] < 3:
-    False = 0
-    True = 1
+LittleEndian = (sys.byteorder == 'little')
+
+_unit_imag = {'f': array(1j,'F'), 'd': 1j}
 
 __all__ = ['fopen','loadmat','savemat']
 
 def getsize_type(mtype):
-    if mtype in ['b','uchar','byte','unsigned char','integer*1', 'int8']:
+    if mtype in ['B','uchar','byte','unsigned char','integer*1', 'int8']:
+        mtype = 'B'
+    elif mtype in ['S1', 'char', 'char*1']:
+        mtype = 'B'
+    elif mtype in ['b', 'schar', 'signed char']:
         mtype = 'b'
-    elif mtype in ['c', 'char','char*1']:
-        mtype = 'c'
-    elif mtype in ['1','schar', 'signed char']:
-        mtype = '1'
-    elif mtype in ['s','short','int16','integer*2']:
-        mtype = 's'
-    elif mtype in ['w','ushort','uint16','unsigned short']:
-        mtype = 'w'
+    elif mtype in ['h','short','int16','integer*2']:
+        mtype = 'h'
+    elif mtype in ['H','ushort','uint16','unsigned short']:
+        mtype = 'H'
     elif mtype in ['i','int']:
         mtype = 'i'
-    elif mtype in ['u','uint','uint32','unsigned int']:
-        mtype = 'u'
-    elif mtype in ['l','long','int32','integer*4']:
-        mtype = 'l'
+    elif mtype in ['I','uint','uint32','unsigned int']:
+        mtype = 'I'
+    elif mtype in ['u4','int32','integer*4']:
+        mtype = 'u4'
     elif mtype in ['f','float','float32','real*4', 'real']:
         mtype = 'f'
     elif mtype in ['d','double','float64','real*8', 'double precision']:
@@ -44,113 +50,112 @@ def getsize_type(mtype):
     elif mtype in ['D','complex*16','complex128','complex','complex double']:
         mtype = 'D'
     else:
-        raise TypeError, 'Bad datatype -- ' + mtype
+        mtype = obj2sctype(mtype)
 
-    argout = (array(0,mtype).itemsize(),mtype)
-    return argout
+    newarr = empty((1,),mtype)
+    return newarr.itemsize, newarr.dtype.char
 
-class fopen:
-    """Class for reading and writing binary files into Numeric arrays.
+class fopen(object):
+    """Class for reading and writing binary files into numpy arrays.
 
     Inputs:
 
       file_name -- The complete path name to the file to open.
-      permission -- Open the file with given permissions: ('r', 'w', 'a')
+      permission -- Open the file with given permissions: ('r', 'H', 'a')
                     for reading, writing, or appending.  This is the same
                     as the mode argument in the builtin open command.
       format -- The byte-ordering of the file:
-                (['native', 'n'], ['ieee-le', 'l'], ['ieee-be', 'b']) for
+                (['native', 'n'], ['ieee-le', 'l'], ['ieee-be', 'B']) for
                 native, little-endian, or big-endian respectively.
 
-    Methods:
-
-      read -- read data from file and return Numeric array
-      write -- write to file from Numeric array
-      fort_read -- read Fortran-formatted binary data from the file.
-      fort_write -- write Fortran-formatted binary data to the file.
-      rewind -- rewind to beginning of file
-      size -- get size of file
-      seek -- seek to some position in the file
-      tell -- return current position in file
-      close -- close the file
-
     Attributes (Read only):
 
       bs -- non-zero if byte-swapping is performed on read and write.
       format -- 'native', 'ieee-le', or 'ieee-be'
-      fid -- the file object
       closed -- non-zero if the file is closed.
       mode -- permissions with which this file was opened
       name -- name of the file
-      
     """
-    
+
+#    Methods:
+#
+#      read -- read data from file and return numpy array
+#      write -- write to file from numpy array
+#      fort_read -- read Fortran-formatted binary data from the file.
+#      fort_write -- write Fortran-formatted binary data to the file.
+#      rewind -- rewind to beginning of file
+#      size -- get size of file
+#      seek -- seek to some position in the file
+#      tell -- return current position in file
+#      close -- close the file
+
     def __init__(self,file_name,permission='rb',format='n'):
         if 'b' not in permission: permission += 'b'
-        if type(file_name) in (types.StringType, types.UnicodeType):
-            self.__dict__['fid'] = open(file_name,permission)
-        elif 'fileno' in file_name.__methods__:  # first argument is an open file
-            self.__dict__['fid'] = file_name 
-        if format in ['native','n','default']:
-            self.__dict__['bs'] = 0
-            self.__dict__['format'] = 'native'
-        elif format in ['ieee-le','l','little-endian','le']:
-            self.__dict__['bs'] = not LittleEndian
-            self.__dict__['format'] = 'ieee-le'
-        elif format in ['ieee-be','b','big-endian','be']:
-            self.__dict__['bs'] = LittleEndian
-            self.__dict__['format'] = 'ieee-be'
+        if isinstance(file_name, basestring):
+            self.file = file(file_name, permission)
+        elif isinstance(file_name, file) and not file_name.closed:
+            # first argument is an open file
+            self.file = file_name
         else:
-            raise ValueError, "Unrecognized format: " + format
-
-        self.__dict__['seek'] = self.fid.seek
-        self.__dict__['tell']= self.fid.tell
-        self.__dict__['close'] = self.fid.close
-        self.__dict__['fileno'] = self.fid.fileno
-        self.__dict__['mode'] = self.fid.mode
-        self.__dict__['closed'] = self.fid.closed
-        self.__dict__['name'] = self.fid.name
-
-    def __setattr__(self, attribute):
-        raise SyntaxError, "There are no user-settable attributes."            
-
+            raise TypeError, 'Need filename or open file as input'
+        self.setformat(format)
+        self.zbuffer = None
+        
     def __del__(self):
         try:
-            self.fid.close()
+            self.file.close()
         except:
             pass
 
+    def close(self):
+        self.file.close()
+
+    def seek(self, *args):
+        self.file.seek(*args)
+
+    def tell(self):
+        return self.file.tell()
+        
+    def raw_read(self, size=-1):
+        """Read raw bytes from file as string."""
+        return self.file.read(size)
+
+    def raw_write(self, str):
+        """Write string to file as raw bytes."""
+        return self.file.write(str)
+
     def setformat(self, format):
+        """Set the byte-order of the file."""
         if format in ['native','n','default']:
-            self.__dict__['bs'] = False
-            self.__dict__['format'] = 'native'
+            self.bs = False
+            self.format = 'native'
         elif format in ['ieee-le','l','little-endian','le']:
-            self.__dict__['bs'] = not LittleEndian
-            self.__dict__['format'] = 'ieee-le'
-        elif format in ['ieee-be','b','big-endian','be']:
-            self.__dict__['bs'] = LittleEndian
-            self.__dict__['format'] = 'ieee-be'
+            self.bs = not LittleEndian
+            self.format = 'ieee-le'
+        elif format in ['ieee-be','B','big-endian','be']:
+            self.bs = LittleEndian
+            self.format = 'ieee-be'
         else:
             raise ValueError, "Unrecognized format: " + format
         return
-        
+
     def write(self,data,mtype=None,bs=None):
-        """Write to open file object the flattened Numeric array data.
+        """Write to open file object the flattened numpy array data.
 
         Inputs:
 
-          data -- the Numeric array to write.
+          data -- the numpy array to write.
           mtype -- a string indicating the binary type to write.
                    The default is the type of data. If necessary a cast is made.
-                   unsigned byte  : 'b', 'uchar', 'byte' 'unsigned char', 'int8',
+                   unsigned byte  : 'B', 'uchar', 'byte' 'unsigned char', 'int8',
                                     'integer*1'
-                   character      : 'c', 'char', 'char*1'
-                   signed char    : '1', 'schar', 'signed char'
-                   short          : 's', 'short', 'int16', 'integer*2'
-                   unsigned short : 'w', 'ushort','uint16','unsigned short'
+                   character      : 'S1', 'char', 'char*1'
+                   signed char    : 'b', 'schar', 'signed char'
+                   short          : 'h', 'short', 'int16', 'integer*2'
+                   unsigned short : 'H', 'ushort','uint16','unsigned short'
                    int            : 'i', 'int'
-                   unsigned int   : 'u', 'uint32','uint','unsigned int'
-                   long           : 'l', 'long', 'int32', 'integer*4'
+                   unsigned int   : 'I', 'uint32','uint','unsigned int'
+                   int32           : 'u4', 'int32', 'integer*4'
                    float          : 'f', 'float', 'float32', 'real*4'
                    double         : 'd', 'double', 'float64', 'real*8'
                    complex float  : 'F', 'complex float', 'complex*8', 'complex64'
@@ -161,18 +166,22 @@ class fopen:
             bs = self.bs
         else:
             bs = (bs == 1)
-        data = asarray(data)
+        if isinstance(data, str):
+            N, buf = len(data), buffer(data)
+            data = ndarray(shape=(N,),dtype='B',buffer=buf)
+        else:
+            data = asarray(data)
         if mtype is None:
-            mtype = data.typecode()
+            mtype = data.dtype.char
         howmany,mtype = getsize_type(mtype)
-        count = product(data.shape)
-        numpyio.fwrite(self.fid,count,data,mtype,bs)
-        return 
+        count = product(data.shape,axis=0)
+        numpyio.fwrite(self.file,count,data,mtype,bs)
+        return
 
     fwrite = write
 
     def read(self,count,stype,rtype=None,bs=None,c_is_b=0):
-        """Read data from file and return it in a Numeric array.
+        """Read data from file and return it in a numpy array.
 
         Inputs:
 
@@ -188,14 +197,14 @@ class fopen:
 
         Outputs: (output,)
 
-          output -- a Numeric array of type rtype.
+          output -- a numpy array of type rtype.
         """
         if bs is None:
             bs = self.bs
         else:
             bs = (bs == 1)
         howmany,stype = getsize_type(stype)
-        shape = None        
+        shape = None
         if c_is_b:
             if count % howmany != 0:
                 raise ValueError, "When c_is_b is non-zero then " \
@@ -206,20 +215,20 @@ class fopen:
             # allow -1 to specify unknown dimension size as in reshape
             minus_ones = shape.count(-1)
             if minus_ones == 0:
-                count = product(shape)
+                count = product(shape,axis=0)
             elif minus_ones == 1:
-                now = self.fid.tell()
-                self.fid.seek(0,2)
-                end = self.fid.tell()
-                self.fid.seek(now)
+                now = self.tell()
+                self.seek(0,2)
+                end = self.tell()
+                self.seek(now)
                 remaining_bytes = end - now
-                know_dimensions_size = -product(count) * getsize_type(stype)[0]
+                know_dimensions_size = -product(count,axis=0) * getsize_type(stype)[0]
                 unknown_dimension_size, illegal = divmod(remaining_bytes,
                                                          know_dimensions_size)
                 if illegal:
                     raise ValueError("unknown dimension doesn't match filesize")
                 shape[shape.index(-1)] = unknown_dimension_size
-                count = product(shape)
+                count = product(shape,axis=0)
             else:
                 raise ValueError(
                     "illegal count; can only specify one unknown dimension")
@@ -230,9 +239,7 @@ class fopen:
             howmany,rtype = getsize_type(rtype)
         if count == 0:
             return zeros(0,rtype)
-        retval = numpyio.fread(self.fid, count, stype, rtype, bs)
-        if len(retval) == 1:
-            retval = retval[0]
+        retval = numpyio.fread(self.file, count, stype, rtype, bs)
         if shape is not None:
             retval = resize(retval, shape)
         return retval
@@ -240,7 +247,7 @@ class fopen:
     fread = read
 
     def rewind(self,howmany=None):
-        """Rewind a file to it's beginning or by a specified amount.
+        """Rewind a file to its beginning or by a specified amount.
         """
         if howmany is None:
             self.seek(0)
@@ -252,12 +259,12 @@ class fopen:
         """
         try:
             sz = self.thesize
-        except AttributeError:            
+        except AttributeError:
             curpos = self.tell()
-            self.fid.seek(0,2)
-            sz = self.fid.tell()
-            self.fid.seek(curpos)
-            self.__dict__['thesize'] = sz
+            self.seek(0,2)
+            sz = self.tell()
+            self.seek(curpos)
+            self.thesize = sz
         return sz
 
     def fort_write(self,fmt,*args):
@@ -268,10 +275,10 @@ class fopen:
           fmt -- If a string then it represents the same format string as
                  used by struct.pack.  The remaining arguments are passed
                  to struct.pack.
-                 
+
                  If fmt is an array, then this array will be written as
                  a Fortran record using the output type args[0].
-                 
+
           *args -- Arguments representing data to write.
         """
         if self.format == 'ieee-le':
@@ -280,26 +287,26 @@ class fopen:
             nfmt = ">i"
         else:
             nfmt = "i"
-        if type(fmt) in (types.StringType, types.UnicodeType):
+        if isinstance(fmt, basestring):
             if self.format == 'ieee-le':
                 fmt = "<"+fmt
             elif self.format == 'ieee-be':
                 fmt = ">"+fmt
             str = apply(struct.pack,(fmt,)+args)
             strlen = struct.pack(nfmt,len(str))
-            self.fid.write(strlen)
-            self.fid.write(str)
-            self.fid.write(strlen)
+            self.write(strlen)
+            self.write(str)
+            self.write(strlen)
         elif type(fmt) == type(array([0])):
             if len(args) > 0:
                 sz,mtype = getsize_type(args[0])
             else:
-                sz,mtype = getsize_type(fmt.typecode())
-            count = product(fmt.shape)
+                sz,mtype = getsize_type(fmt.dtype.char)
+            count = product(fmt.shape,axis=0)
             strlen = struct.pack(nfmt,count*sz)
-            self.fid.write(strlen)
-            numpyio.fwrite(self.fid,count,fmt,mtype,self.bs)
-            self.fid.write(strlen)
+            self.write(strlen)
+            numpyio.fwrite(self.file,count,fmt,mtype,self.bs)
+            self.write(strlen)
         else:
             raise TypeError, "Unknown type in first argument"
 
@@ -326,19 +333,19 @@ class fopen:
             fmt = lookup_dict[self.format] + fmt
             numbytes = struct.calcsize(fmt)
             nn = struct.calcsize("i");
-            if (self.fid.read(nn) == ''):
+            if (self.raw_read(nn) == ''):
                 raise ValueError, "Unexpected end of file..."
-            strdata = self.fid.read(numbytes)
+            strdata = self.raw_read(numbytes)
             if strdata == '':
                 raise ValueError, "Unexpected end of file..."
             data = struct.unpack(fmt,strdata)
-            if (self.fid.read(nn) == ''):
+            if (self.raw_read(nn) == ''):
                 raise ValueError, "Unexpected end of file..."
             return data
         else:  # Ignore format string and read in next record as an array.
             fmt = lookup_dict[self.format] + "i"
             nn = struct.calcsize(fmt)
-            nbytestr = self.fid.read(nn)
+            nbytestr = self.raw_read(nn)
             if nbytestr == '':
                 raise ValueError, "Unexpected end of file..."
             nbytes = struct.unpack(fmt,nbytestr)[0]
@@ -351,14 +358,42 @@ class fopen:
                 raise ValueError, "Negative number of bytes to read:\n    file is probably not opened with correct endian-ness."
             if ncount == 0:
                 raise ValueError, "End of file?  Zero-bytes to read."
-            retval = numpyio.fread(self.fid, ncount, dtype, dtype, self.bs)
+            retval = numpyio.fread(self.file, ncount, dtype, dtype, self.bs)
             if len(retval) == 1:
                 retval = retval[0]
-            if (self.fid.read(nn) == ''):
+            if (self.raw_read(nn) == ''):
                 raise ValueError, "Unexpected end of file..."
             return retval
+        
 
+class CompressedFopen(fopen):
+    """ File container for temporary buffer to decompress data """
+    def __init__(self, *args, **kwargs):
+        fd, fname = mkstemp()
+        super(CompressedFopen, self).__init__(
+            os.fdopen(fd, 'w+b'), *args, **kwargs)
+        self.file_name = fname
+        
+    def fill(self, bytes):
+        """ Uncompress buffer in @bytes and write to file """
+        self.rewind()
+        self.raw_write(zlib.decompress(bytes))
+        self.rewind()
 
+    def __del__(self):
+        try:
+            self.file.truncate(0)
+        except:
+            pass
+        try:
+            self.close()
+        except:
+            pass
+        try:
+            os.remove(self.file_name)
+        except:
+            pass
+        
 #### MATLAB Version 5 Support ###########
 
 # Portions of code borrowed and (heavily) adapted
@@ -404,6 +439,10 @@ miDOUBLE = 9
 miINT64 =12
 miUINT64 = 13
 miMATRIX = 14
+miCOMPRESSED = 15
+miUTF8 = 16
+miUTF16 = 17
+miUTF32 = 18
 
 miNumbers = (
     miINT8,
@@ -419,19 +458,34 @@ miNumbers = (
     )
 
 miDataTypes = {
-    miINT8 : ('miINT8', 1,'1'),
-    miUINT8 : ('miUINT8', 1,'b'),
-    miINT16 : ('miINT16', 2,'s'),
-    miUINT16 :('miUINT16',2,'w'),
-    miINT32 : ('miINT32',4,'l'),
-    miUINT32 : ('miUINT32',4,'u'),
+    miINT8 : ('miINT8', 1,'b'),
+    miUINT8 : ('miUINT8', 1,'B'),
+    miINT16 : ('miINT16', 2,'h'),
+    miUINT16 :('miUINT16',2,'H'),
+    miINT32 : ('miINT32',4,'u4'),
+    miUINT32 : ('miUINT32',4,'I'),
     miSINGLE : ('miSINGLE',4,'f'),
     miDOUBLE : ('miDOUBLE',8,'d'),
-    miINT64 : ('miINT64',8,'c'),   # Handle 64 bit integers as string data.
-    miUINT64 : ('miUINT64',8,'c'),
+    miINT64 : ('miINT64',8,'q'),
+    miUINT64 : ('miUINT64',8,'Q'),
     miMATRIX : ('miMATRIX',0,None),
+    miUTF8 : ('miUTF8',1,'b'),
+    miUTF16 : ('miUTF16',2,'h'),
+    miUTF32 : ('miUTF32',4,'u4'),
     }
-    
+
+''' Before release v7.1 (release 14) matlab used the system default
+character encoding scheme padded out to 16-bits. Release 14 and later
+use Unicode. When saving character data, matlab R14 checks if it can
+be encoded in 7-bit ascii, and saves in that format if so.'''
+miCodecs = {
+    miUINT8: 'ascii',
+    miUINT16: sys.getdefaultencoding(),
+    miUTF8: 'utf8',
+    miUTF16: 'utf16',
+    miUTF32: 'utf32',
+    } 
+
 mxCELL_CLASS = 1
 mxSTRUCT_CLASS = 2
 mxOBJECT_CLASS = 3
@@ -457,35 +511,42 @@ mxArrays = (
     mxINT32_CLASS,
     mxUINT32_CLASS,
     )
-
-def _parse_header(fid, dict):
+    
+def _parse_header(fid, hdict):
     correct_endian = (ord('M')<<8) + ord('I')
-                 # if this number is read no BS    
+                 # if this number is read no BS
     fid.seek(126)  # skip to endian detector
     endian_test = fid.read(1,'int16')
     if (endian_test == correct_endian): openstr = 'n'
     else:  # must byteswap
         if LittleEndian:
-            openstr = 'b'
+            openstr = 'B'
         else: openstr = 'l'
     fid.setformat(openstr)  # change byte-order if necessary
-    fid.rewind()    
-    dict['__header__'] = fid.fid.read(124).strip(' \t\n\000')
+    fid.rewind()
+    hdict['__header__'] = fid.raw_read(124).strip(' \t\n\000')
     vers = fid.read(1,'int16')
-    dict['__version__'] = '%d.%d' % (vers >> 8, vers & 255)
+    hdict['__version__'] = '%d.%d' % (vers >> 8, vers & 0xFF)
     fid.seek(2,1)  # move to start of data
     return
 
+def _skip_padding(fid, numbytes, rowsize):
+    """ Skip to next row or @rowsize after previous read of @numbytes """
+    mod = numbytes % rowsize
+    if mod:
+        skip = rowsize-mod
+        fid.seek(skip,1)
+
 def _parse_array_flags(fid):
     # first 8 bytes are always miUINT32 and 8 --- just a check
-    dtype, nbytes = fid.read(2,'u')
+    dtype, nbytes = fid.read(2,'I')
     if (dtype != miUINT32) or (nbytes != 8):
         raise IOError, "Invalid MAT file. Perhaps a byte-order problem."
 
     # read array flags.
-    rawflags = fid.read(2,'u')
-    class_ = rawflags[0] & 255
-    flags = (rawflags[0] & 65535) >> 8
+    rawflags = fid.read(2,'I')
+    class_ = rawflags[0] & 0xFF
+    flags = (rawflags[0] & 0xFFFF) >> 8
     # Global and logical fields are currently ignored
     if (flags & 8): cmplx = 1
     else: cmplx = 0
@@ -495,157 +556,207 @@ def _parse_array_flags(fid):
         nzmax = None
     return class_, cmplx, nzmax
 
-def _parse_mimatrix(fid,bytes): 
+def _parse_mimatrix(fid,bytes):
     dclass, cmplx, nzmax =_parse_array_flags(fid)
-    dims = _get_element(fid)[0]
-    name = ''.join(asarray(_get_element(fid)[0]).astype('c'))
+    dims = _get_element(fid)
+    name = _get_element(fid).tostring()
+    tupdims = tuple(dims[::-1])
     if dclass in mxArrays:
-        result, unused =_get_element(fid)
-        if type == mxCHAR_CLASS:
-            result = ''.join(asarray(result).astype('c'))
+        result, unused, dtype =_get_element(fid, return_name_dtype=True)
+        if dclass == mxCHAR_CLASS:
+            en = miCodecs[dtype]
+            try:
+                " ".encode(en)
+            except LookupError:
+                raise TypeError, 'Character encoding %s not supported' % en
+            if dtype == miUINT16:
+                char_len = len("  ".encode(en)) - len(" ".encode(en))
+                if char_len == 1: # Need to downsample from 16 bit
+                    result = result.astype(uint8)
+                elif char_len != 2:
+                    raise TypeError, 'miUNIT16 type cannot use >2 bytes encoding'
+            result = squeeze(transpose(reshape(result,tupdims)))
+            dims = result.shape
+            if len(dims) >= 2: # return array of strings
+                n_dims = dims[:-1]
+                string_arr = reshape(result, (product(n_dims,axis=0), dims[-1]))
+                result = empty(n_dims, dtype=object)
+                for i in range(0, n_dims[-1]):
+                    result[...,i] = string_arr[i].tostring().decode(en)
+            else: # return string
+                result = result.tostring().decode(en)
         else:
             if cmplx:
-                imag, unused =_get_element(fid)
-                result = result + cast[imag.typecode()](1j) * imag
-            result = squeeze(transpose(reshape(result,dims[::-1])))
-
+                imag  =_get_element(fid)
+                try:
+                    result = result + _unit_imag[imag.dtype.char] * imag
+                except KeyError:
+                    result = result + 1j*imag
+            result = squeeze(transpose(reshape(result,tupdims)))
+            
     elif dclass == mxCELL_CLASS:
-        length = product(dims)
-        result = zeros(length, PyObject)
+        length = product(dims,axis=0)
+        result = empty(length, dtype=object)
         for i in range(length):
-            sa, unused = _get_element(fid)
-            result[i]= sa
-        result = squeeze(transpose(reshape(result,dims[::-1])))
-        if rank(result)==0: result = result.toscalar()
-        
+            result[i] = _get_element(fid)
+        result = squeeze(transpose(reshape(result,tupdims)))
+        if not result.shape:
+            result = result.item()
+
     elif dclass == mxSTRUCT_CLASS:
-        length = product(dims)
-        result = zeros(length, PyObject)
-        namelength = _get_element(fid)[0]
+        length = product(dims,axis=0)
+        result = zeros(length, object)
+        namelength = _get_element(fid)
         # get field names
-        names = _get_element(fid)[0]
+        names = _get_element(fid)
         splitnames = [names[i:i+namelength] for i in \
                       xrange(0,len(names),namelength)]
-        fieldnames = [''.join(asarray(x).astype('c')).strip('\x00')
+        fieldnames = [x.tostring().strip('\x00')
                               for x in splitnames]
         for i in range(length):
             result[i] = mat_struct()
             for element in fieldnames:
-                val,unused = _get_element(fid)
-                result[i].__dict__[element] = val
-        result = squeeze(transpose(reshape(result,dims[::-1])))
-        if rank(result)==0: result = result.toscalar()        
+                result[i].__dict__[element]  = _get_element(fid)
+        result = squeeze(transpose(reshape(result,tupdims)))
+        if not result.shape:
+            result = result.item()
 
         # object is like a structure with but with a class name
     elif dclass == mxOBJECT_CLASS:
-        class_name = ''.join(asarray(_get_element(fid)[0]).astype('c'))
-        length = product(dims)
-        result = zeros(length, PyObject)
-        namelength = _get_element(fid)[0]
+        class_name = _get_element(fid).tostring()
+        length = product(dims,axis=0)
+        result = zeros(length, object)
+        namelength = _get_element(fid)
         # get field names
-        names = _get_element(fid)[0]
+        names = _get_element(fid)
         splitnames = [names[i:i+namelength] for i in \
                       xrange(0,len(names),namelength)]
-        fieldnames = [''.join(asarray(x).astype('c')).strip('\x00')
+        fieldnames = [x.tostring().strip('\x00')
                               for x in splitnames]
         for i in range(length):
             result[i] = mat_obj()
             result[i]._classname = class_name
             for element in fieldnames:
-                val,unused = _get_element(fid)
-                result[i].__dict__[element] = val
-        result = squeeze(transpose(reshape(result,dims[::-1])))
-        if rank(result)==0: result = result.toscalar()        
-         
+                result[i].__dict__[element] = _get_element(fid)
+        result = squeeze(transpose(reshape(result,tupdims)))
+        if not result.shape:
+            result = result.item()
+
     elif dclass == mxSPARSE_CLASS:
-        rowind, unused = _get_element(fid)
-        colind, unused = _get_element(fid)
-        res, unused = _get_element(fid)
+        rowind  = _get_element(fid)
+        colind = _get_element(fid)
+        res = _get_element(fid)
         if cmplx:
-            imag, unused = _get_element(fid)
-            res = res + cast[imag.typecode()](1j)*imag
+            imag = _get_element(fid)
+            try:
+                res = res + _unit_imag[imag.dtype.char] * imag
+            except (KeyError,AttributeError):
+                res = res + 1j*imag
+        ''' From the matlab API documentation, last found here:
+        http://www.mathworks.com/access/helpdesk/help/techdoc/matlab_external/
+        @rowind are simply the row indices for all the (@res) non-zero
+        entries in the sparse array.  @rowind has nzmax entries, so
+        may well have more entries than len(@res), the actual number
+        of non-zero entries, but @rowind[len(res):] can be discarded
+        and should be 0. @colind has length (number of columns + 1),
+        and is such that, if D = diff(@colind), D[j] gives the number
+        of non-zero entries in column j. Because @rowind values are
+        stored in column order, this gives the column corresponding to
+        each @rowind
+        '''
+        cols = empty((len(res)), dtype=rowind.dtype)
+        col_counts = diff(colind)
+        start_row = 0
+        for i in where(col_counts)[0]:
+            end_row = start_row + col_counts[i]
+            cols[start_row:end_row] = i
+            start_row = end_row
+        ij = vstack((rowind[:len(res)], cols))
         if have_sparse:
-            spmat = scipy.sparse.spmatrix(dims[1],dims[0],typecode=res.typecode())
-            spmat.data = res
-            # stored with zero-based indices --- spmatrix needs 1 based
-            #  indices internally
-            spmat.index = [rowind+1, colind+1]
-            spmat.nzmax = len(res)
-            spmat.lastel = len(res)-1
-            result = spmat.transp(inplace=1)
-            result = spmat
+            result = scipy.sparse.csc_matrix((res,ij), [dims[0],dims[1]])
         else:
-            result = (rowind, colind, res)
+            result = (dims, ij, res)
 
     return result, name
-    
-# Return a Python object for the element
-def _get_element(fid):
 
-    test = fid.fid.read(1)
+# Return a Python object for the element
+def _get_element(fid, return_name_dtype=False):
+    """ Return a python object from next element in @fid
+
+    @fid    - fopen object for matfile
+    @return_name_dtype - if True, return tuple of (element, name, dtype)
+                         if False, return element only
+    """
+    name = None
+    test = fid.raw_read(1)
     if len(test) == 0:  # nothing left
         raise EOFError
     else:
         fid.rewind(1)
     # get the data tag
-    raw_tag = fid.read(1,'u')
+    raw_tag = int(fid.read(1,'I'))
     
-    # check for compressed
+    # check for small data element format
     numbytes = raw_tag >> 16
-    if numbytes > 0:  # compressed format
+    if numbytes > 0:  # small data element format
         if numbytes > 4:
             raise IOError, "Problem with MAT file: " \
-                  "too many bytes in compressed format."
-        dtype = raw_tag & 65535
+                  "too many bytes in small data element format."
+        dtype = int(raw_tag & 0xFFFF)
         el = fid.read(numbytes,miDataTypes[dtype][2],c_is_b=1)
         fid.seek(4-numbytes,1)  # skip padding
-        return el, None
+    else:
+        # otherwise parse tag
+        dtype = raw_tag
+        numbytes = fid.read(1,'I')
+        
+        if dtype == miCOMPRESSED: # compressed data type
+            if not fid.zbuffer:
+                fid.zbuffer = CompressedFopen(format=fid.format)
+            fid.zbuffer.fill(fid.raw_read(numbytes))
+            _skip_padding(fid, numbytes, 8)
+            return _get_element(fid.zbuffer, return_name_dtype)
+        if dtype != miMATRIX:  # therefore basic data type
+            try:
+                el = fid.read(numbytes,miDataTypes[dtype][2],c_is_b=1)
+            except KeyError:
+                raise ValueError, "Unknown data type"
+            _skip_padding(fid, numbytes, 8)
+        else:
+            # handle miMatrix type
+            el, name = _parse_mimatrix(fid,numbytes)
 
-    # otherwise parse tag
-    dtype = raw_tag
-    numbytes = fid.read(1,'u')
-    if dtype != miMATRIX:  # basic data type 
-        try:
-            outarr = fid.read(numbytes,miDataTypes[dtype][2],c_is_b=1)
-        except KeyError:
-            raise ValueError, "Unknown data type"
-        mod8 = numbytes%8
-        if mod8:       # skip past padding
-            skip = 8-mod8
-            fid.seek(skip,1)
-        return outarr, None
-
-    # handle miMatrix type
-    el, name = _parse_mimatrix(fid,numbytes)
-    return el, name
+    if return_name_dtype:
+        return el, name, dtype
+    return el
 
 def _loadv5(fid,basename):
-    # return a dictionary from a Matlab version 5 file
+    # return a dictionary from a Matlab version 5-7.1 file
     # always contains the variable __header__
-    dict = {}
-    _parse_header(fid,dict)
+    mdict = {}
+    _parse_header(fid,mdict)
     var = 0
     while 1:  # file pointer to start of next data
         try:
             var = var + 1
-            el, varname = _get_element(fid)
+            el, varname, unused = _get_element(fid, return_name_dtype=True)
             if varname is None:
                 varname = '%s_%04d' % (basename,var)
-            dict[varname] = el
+            mdict[varname] = el
         except EOFError:
             break
-    return dict
+    return mdict
 
 ### END MATLAB v5 support #############
 
-def loadmat(name, dict=None, appendmat=1, basename='raw'):
+def loadmat(name, mdict=None, appendmat=1, basename='raw'):
     """Load the MATLAB(tm) mat file.
 
     If name is a full path name load it in.  Otherwise search for the file
     on the sys.path list and load the first one found (the current directory
     is searched first).
 
-    Both v4 (Level 1.0) and v5 matfiles are supported.
+    v4 (Level 1.0), v6 and v7.1 matfiles are supported.  
 
     Inputs:
 
@@ -679,6 +790,7 @@ def loadmat(name, dict=None, appendmat=1, basename='raw'):
                 fid = open(test_name,'rb')
                 fid.close()
                 full_name = test_name
+                break
             except IOError:
                 pass
         if full_name is None:
@@ -690,13 +802,13 @@ def loadmat(name, dict=None, appendmat=1, basename='raw'):
     if not (0 in test_vals):       # MATLAB version 5 format
         fid.rewind()
         thisdict = _loadv5(fid,basename)
-        if dict is not None:
-            dict.update(thisdict)
+        if mdict is not None:
+            mdict.update(thisdict)
             return
         else:
             return thisdict
         
-
+    # The remainder of this function is the v4 codepath
     testtype = struct.unpack('i',test_vals.tostring())
     # Check to see if the number is positive and less than 5000.
     if testtype[0] < 0 or testtype[0] > 4999:
@@ -720,14 +832,13 @@ def loadmat(name, dict=None, appendmat=1, basename='raw'):
     thisdict = {}
     while 1:
         if (fid.tell() == length):
-            break        
+            break
         header = fid.fread(5,'int')
         if len(header) != 5:
             fid.close()
             print "Warning: Read error in file."
             break
-
-        M,rest = divmod(header[0],1000)
+        M,rest = divmod(int(header[0]),1000) # int is for workaround numpy 0.9.9 bug
         O,rest = divmod(rest,100)
         P,rest = divmod(rest,10)
         T = rest
@@ -743,7 +854,7 @@ def loadmat(name, dict=None, appendmat=1, basename='raw'):
             fid.close()
             raise ValueError, "Cannot handle sparse matrices, yet."
 
-        storage = {0:'d',1:'f',2:'i',3:'s',4:'w',5:'b'}[P]
+        storage = {0:'d',1:'f',2:'i',3:'h',4:'H',5:'B'}[P]
 
         varname = fid.fread(header[-1],'char')[:-1]
         varname = varname.tostring()
@@ -753,67 +864,67 @@ def loadmat(name, dict=None, appendmat=1, basename='raw'):
             data = atleast_1d(fid.fread(numels,storage))
             if header[3]:  # imaginary data
                 data2 = fid.fread(numels,storage)
-                if data.typecode() == 'f' and data2.typecode() == 'f':
-                    new = zeros(data.shape,'F')
+                if data.dtype.char == 'f' and data2.dtype.char == 'f':
+                    new = empty(data.shape,'F')
                     new.real = data
                     new.imag = data2
                     data = new
                     del(new)
                     del(data2)
             if len(data) > 1:
-                data.shape = (header[2], header[1])                
+                data=data.reshape((header[2], header[1])                )
                 thisdict[varname] = transpose(squeeze(data))
             else:
                 thisdict[varname] = data
         else:
             data = atleast_1d(fid.fread(numels,storage,'char'))
             if len(data) > 1:
-                data.shape = (header[2], header[1])
+                data=data.reshape((header[2], header[1]))
                 thisdict[varname] = transpose(squeeze(data))
             else:
                 thisdict[varname] = data
 
     fid.close()
-    if dict is not None:
+    if mdict is not None:
         print "Names defined = ", defnames
-        dict.update(thisdict)
+        mdict.update(thisdict)
     else:
         return thisdict
 
 
-def savemat(filename, dict):
+def savemat(filename, mdict):
     """Save a dictionary of names and arrays into the MATLAB-style .mat file.
 
     This saves the arrayobjects in the given dictionary to a matlab Version 4
     style .mat file.
     """
-    storage = {'D':0,'d':0,'F':1,'f':1,'l':2,'i':2,'s':3,'b':5}
+    storage = {'D':0,'d':0,'F':1,'f':1,'u4':2,'i':2,'h':3,'B':5}
     if filename[-4:] != ".mat":
         filename = filename + ".mat"
     fid = fopen(filename,'wb')
     M = not LittleEndian
     O = 0
-    for variable in dict.keys():
-        var = dict[variable]
-        if type(var) is not ArrayType:
+    for variable in mdict.keys():
+        var = mdict[variable]
+        if not isinstance(var, ndarray):
             continue
-        if var.typecode() == 'c':
+        if var.dtype.char == 'S1':
             T = 1
         else:
             T = 0
-        if var.typecode() == '1':
-            var = var.astype('s')
-        P = storage[var.typecode()]
+        if var.dtype.char == 'b':
+            var = var.astype('h')
+        P = storage[var.dtype.char]
         fid.fwrite([M*1000+O*100+P*10+T],'int')
 
         if len(var.shape) == 1:
-            var.shape = (len(var), 1)
+            var=var.reshape((len(var), 1))
         var = transpose(var)
 
         if len(var.shape) > 2:
-            var.shape = (product(var.shape[:-1]), var.shape[-1])
+            var=var.reshape((product(var.shape[:-1],axis=0), var.shape[-1]))
 
-        imagf = var.typecode() in ['F', 'D']
+        imagf = var.dtype.char in ['F', 'D']
         fid.fwrite([var.shape[1], var.shape[0], imagf, len(variable)+1],'int')
         fid.fwrite(variable+'\x00','char')
         if imagf:
@@ -823,10 +934,3 @@ def savemat(filename, dict):
             fid.fwrite(var)
     fid.close()
     return
-        
-        
-            
-
-
-
-
