@@ -1,3 +1,7 @@
+## Automatically adapted for scipy Oct 18, 2005 by
+
+## Automatically adapted for scipy Oct 18, 2005 by
+
 #
 # Author: Pearu Peterson, March 2002
 #
@@ -9,14 +13,14 @@ __all__ = ['solve','inv','det','lstsq','norm','pinv','pinv2',
            'all_mat']
 
 #from blas import get_blas_funcs
-from lapack import get_lapack_funcs
 from flinalg import get_flinalg_funcs
-from scipy_base import asarray,zeros,sum,NewAxis,greater_equal,subtract,arange,\
-     conjugate,ravel,r_,mgrid,take,ones,dot,transpose,diag,sqrt,add,real
-import Matrix
-import scipy_base
-from scipy_base import asarray_chkfinite
-import calc_lwork
+from lapack import get_lapack_funcs
+from numpy import asarray,zeros,sum,newaxis,greater_equal,subtract,arange,\
+     conjugate,ravel,r_,mgrid,take,ones,dot,transpose,sqrt,add,real
+import numpy
+from numpy import asarray_chkfinite, outer, concatenate, reshape, single
+from numpy import matrix as Matrix
+from scipy.linalg import calc_lwork
 
 class LinAlgError(Exception):
     pass
@@ -32,7 +36,7 @@ def lu_solve((lu, piv), b, trans=0, overwrite_b=0):
       b        -- a set of right-hand sides
       trans    -- type of system to solve:
                   0 : a   * x = b   (no transpose)
-                  1 : a^T * x = b   (transpose) 
+                  1 : a^T * x = b   (transpose)
                   2   a^H * x = b   (conjugate transpose)
 
     Outputs:
@@ -42,7 +46,7 @@ def lu_solve((lu, piv), b, trans=0, overwrite_b=0):
     b1 = asarray_chkfinite(b)
     overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
     if lu.shape[0] != b1.shape[0]:
-        raise ValuError, "incompatible dimensions."
+        raise ValueError, "incompatible dimensions."
     getrs, = get_lapack_funcs(('getrs',),(lu,b1))
     x,info = getrs(lu,piv,b1,trans=trans,overwrite_b=overwrite_b)
     if info==0:
@@ -67,7 +71,7 @@ def cho_solve((c, lower), b, overwrite_b=0):
     b1 = asarray_chkfinite(b)
     overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
     if c.shape[0] != b1.shape[0]:
-        raise ValuError, "incompatible dimensions."
+        raise ValueError, "incompatible dimensions."
     potrs, = get_lapack_funcs(('potrs',),(c,b1))
     x,info = potrs(c,b1,lower=lower,overwrite_b=overwrite_b)
     if info==0:
@@ -106,7 +110,7 @@ def solve(a, b, sym_pos=0, lower=0, overwrite_a=0, overwrite_b=0,
         print 'solve:overwrite_a=',overwrite_a
         print 'solve:overwrite_b=',overwrite_b
     if sym_pos:
-        posv, = get_lapack_funcs(('posv',),(a1,b1),debug=debug)
+        posv, = get_lapack_funcs(('posv',),(a1,b1))
         c,x,info = posv(a1,b1,
                         lower = lower,
                         overwrite_a=overwrite_a,
@@ -116,7 +120,7 @@ def solve(a, b, sym_pos=0, lower=0, overwrite_a=0, overwrite_b=0,
         lu,piv,x,info = gesv(a1,b1,
                              overwrite_a=overwrite_a,
                              overwrite_b=overwrite_b)
-        
+
     if info==0:
         return x
     if info>0:
@@ -132,7 +136,7 @@ def solve_banded((l,u), ab, b, overwrite_ab=0, overwrite_b=0,
     a is a banded matrix stored in diagonal orded form
 
      *   *     a1u
-     
+
      *  a12 a23 ...
     a11 a22 a33 ...
     a21 a32 a43 ...
@@ -154,8 +158,8 @@ def solve_banded((l,u), ab, b, overwrite_ab=0, overwrite_b=0,
     overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
 
     gbsv, = get_lapack_funcs(('gbsv',),(a1,b1))
-    a2 = zeros((2*l+u+1,a1.shape[1]),gbsv.typecode)
-    a2[l:,:] = a1 
+    a2 = zeros((2*l+u+1,a1.shape[1]), dtype=gbsv.dtype)
+    a2[l:,:] = a1
     lu,piv,x,info = gbsv(l,u,a2,b1,
                          overwrite_ab=1,
                          overwrite_b=overwrite_b)
@@ -221,58 +225,69 @@ def inv(a, overwrite_a=0):
 
 ## matrix and Vector norm
 import decomp
-def norm(x, ord=2):
-    """ norm(x, ord=2) -> n
+def norm(x, ord=None):
+    """ norm(x, ord=None) -> n
 
-    Matrix and vector norm.
+    Matrix or vector norm.
 
     Inputs:
 
       x -- a rank-1 (vector) or rank-2 (matrix) array
-      ord -- the order of norm.
+      ord -- the order of the norm.
 
      Comments:
+       For arrays of any rank, if ord is None:
+         calculate the square norm (Euclidean norm for vectors, Frobenius norm for matrices)
 
        For vectors ord can be any real number including Inf or -Inf.
          ord = Inf, computes the maximum of the magnitudes
          ord = -Inf, computes minimum of the magnitudes
-         ord is finite, computes sum(abs(x)**ord)**(1.0/ord)
+         ord is finite, computes sum(abs(x)**ord,axis=0)**(1.0/ord)
 
-       For matrices ord can only be + or - 1, 2, Inf.
+       For matrices ord can only be one of the following values:
          ord = 2 computes the largest singular value
          ord = -2 computes the smallest singular value
          ord = 1 computes the largest column sum of absolute values
          ord = -1 computes the smallest column sum of absolute values
          ord = Inf computes the largest row sum of absolute values
          ord = -Inf computes the smallest row sum of absolute values
-         ord = 'fro' computes the frobenius norm sqrt(sum(diag(X.H * X)))
+         ord = 'fro' computes the frobenius norm sqrt(sum(diag(X.H * X),axis=0))
+
+       For values ord < 0, the result is, strictly speaking, not a
+       mathematical 'norm', but it may still be useful for numerical purposes.
     """
     x = asarray_chkfinite(x)
+    if ord is None: # check the default case first and handle it immediately
+        return sqrt(add.reduce(real((conjugate(x)*x).ravel())))
+
     nd = len(x.shape)
-    Inf = scipy_base.Inf
+    Inf = numpy.Inf
     if nd == 1:
         if ord == Inf:
-            return scipy_base.amax(abs(x))
+            return numpy.amax(abs(x))
         elif ord == -Inf:
-            return scipy_base.amin(abs(x))
+            return numpy.amin(abs(x))
+        elif ord == 1:
+            return numpy.sum(abs(x),axis=0) # special case for speedup
+        elif ord == 2:
+            return sqrt(numpy.sum(real((conjugate(x)*x)),axis=0)) # special case for speedup
         else:
-            return scipy_base.sum(abs(x)**ord)**(1.0/ord)
+            return numpy.sum(abs(x)**ord,axis=0)**(1.0/ord)
     elif nd == 2:
         if ord == 2:
-            return scipy_base.amax(decomp.svd(x,compute_uv=0))
+            return numpy.amax(decomp.svd(x,compute_uv=0))
         elif ord == -2:
-            return scipy_base.amin(decomp.svd(x,compute_uv=0))
+            return numpy.amin(decomp.svd(x,compute_uv=0))
         elif ord == 1:
-            return scipy_base.amax(scipy_base.sum(abs(x)))
+            return numpy.amax(numpy.sum(abs(x),axis=0))
         elif ord == Inf:
-            return scipy_base.amax(scipy_base.sum(abs(x),axis=1))
+            return numpy.amax(numpy.sum(abs(x),axis=1))
         elif ord == -1:
-            return scipy_base.amin(scipy_base.sum(abs(x)))
+            return numpy.amin(numpy.sum(abs(x),axis=0))
         elif ord == -Inf:
-            return scipy_base.amin(scipy_base.sum(abs(x),axis=1))
+            return numpy.amin(numpy.sum(abs(x),axis=1))
         elif ord in ['fro','f']:
-            val = real((conjugate(x)*x).flat)
-            return sqrt(add.reduce(val))
+            return sqrt(add.reduce(real((conjugate(x)*x).ravel())))
         else:
             raise ValueError, "Invalid norm order for matrices."
     else:
@@ -330,7 +345,7 @@ def lstsq(a, b, cond=None, overwrite_a=0, overwrite_b=0):
     if n>m:
         # need to extend b matrix as it will be filled with
         # a larger solution matrix
-        b2 = zeros((n,nrhs),gelss.typecode)
+        b2 = zeros((n,nrhs), dtype=gelss.dtype)
         if len(b1.shape)==2: b2[:m,:] = b1
         else: b2[:m,0] = b1
         b1 = b2
@@ -347,10 +362,10 @@ def lstsq(a, b, cond=None, overwrite_a=0, overwrite_b=0):
     if info>0: raise LinAlgError, "SVD did not converge in Linear Least Squares"
     if info<0: raise ValueError,\
        'illegal value in %-th argument of internal gelss'%(-info)
-    resids = asarray([],x.typecode())
+    resids = asarray([], dtype=x.dtype)
     if n<m:
         x1 = x[:n]
-        if rank==n: resids = sum(x[n:]**2)
+        if rank==n: resids = sum(x[n:]**2,axis=0)
         x = x1
     return x,resids,rank,s
 
@@ -361,13 +376,13 @@ def pinv(a, cond=None):
     Compute generalized inverse of A using least-squares solver.
     """
     a = asarray_chkfinite(a)
-    t = a.typecode()
-    b = scipy_base.identity(a.shape[0],t)
+    b = numpy.identity(a.shape[0], dtype=a.dtype)
     return lstsq(a, b, cond=cond)[0]
 
 
-eps = scipy_base.limits.double_epsilon
-feps = scipy_base.limits.float_epsilon
+eps = numpy.finfo(float).eps.item()
+feps = numpy.finfo(single).eps.item()
+
 _array_precision = {'f': 0, 'd': 1, 'F': 0, 'D': 1}
 def pinv2(a, cond=None):
     """ pinv2(a, cond=None) -> a_pinv
@@ -376,11 +391,11 @@ def pinv2(a, cond=None):
     """
     a = asarray_chkfinite(a)
     u, s, vh = decomp.svd(a)
-    t = u.typecode()
+    t = u.dtype.char
     if cond in [None,-1]:
         cond = {0: feps*1e3, 1: eps*1e6}[_array_precision[t]]
     m,n = a.shape
-    cutoff = cond*scipy_base.maximum.reduce(s)
+    cutoff = cond*numpy.maximum.reduce(s)
     psigma = zeros((m,n),t)
     for i in range(len(s)):
         if s[i] > cutoff:
@@ -392,55 +407,57 @@ def pinv2(a, cond=None):
 # matrix construction functions
 #-----------------------------------------------------------------------------
 
-def tri(N, M=None, k=0, typecode=None):
-    """ returns a N-by-M matrix where all the diagonals starting from 
+def tri(N, M=None, k=0, dtype=None):
+    """ returns a N-by-M matrix where all the diagonals starting from
         lower left corner up to the k-th are all ones.
     """
     if M is None: M = N
-    if type(M) == type('d'): 
-        typecode = M
+    if type(M) == type('d'):
+        #pearu: any objections to remove this feature?
+        #       As tri(N,'d') is equivalent to tri(N,dtype='d')
+        dtype = M
         M = N
     m = greater_equal(subtract.outer(arange(N), arange(M)),-k)
-    if typecode is None:
+    if dtype is None:
         return m
     else:
-        return m.astype(typecode)
+        return m.astype(dtype)
 
 def tril(m, k=0):
     """ returns the elements on and below the k-th diagonal of m.  k=0 is the
         main diagonal, k > 0 is above and k < 0 is below the main diagonal.
     """
-    svsp = m.spacesaver()
-    m = asarray(m,savespace=1)
-    out = tri(m.shape[0], m.shape[1], k=k, typecode=m.typecode())*m
-    out.savespace(svsp)
+    svsp = getattr(m,'spacesaver',lambda:0)()
+    m = asarray(m)
+    out = tri(m.shape[0], m.shape[1], k=k, dtype=m.dtype.char)*m
+    pass  ## pass  ## out.savespace(svsp)
     return out
 
 def triu(m, k=0):
     """ returns the elements on and above the k-th diagonal of m.  k=0 is the
         main diagonal, k > 0 is above and k < 0 is below the main diagonal.
     """
-    svsp = m.spacesaver()
-    m = asarray(m,savespace=1)
-    out = (1-tri(m.shape[0], m.shape[1], k-1, m.typecode()))*m
-    out.savespace(svsp)
+    svsp = getattr(m,'spacesaver',lambda:0)()
+    m = asarray(m)
+    out = (1-tri(m.shape[0], m.shape[1], k-1, m.dtype.char))*m
+    pass  ## pass  ## out.savespace(svsp)
     return out
 
 def toeplitz(c,r=None):
     """ Construct a toeplitz matrix (i.e. a matrix with constant diagonals).
 
         Description:
-    
+
            toeplitz(c,r) is a non-symmetric Toeplitz matrix with c as its first
            column and r as its first row.
-    
-           toeplitz(c) is a symmetric (Hermitian) Toeplitz matrix (r=c). 
-    
+
+           toeplitz(c) is a symmetric (Hermitian) Toeplitz matrix (r=c).
+
         See also: hankel
     """
-    isscalar = scipy_base.isscalar
+    isscalar = numpy.isscalar
     if isscalar(c) or isscalar(r):
-        return c   
+        return c
     if r is None:
         r = c
         r[0] = conjugate(r[0])
@@ -453,27 +470,27 @@ def toeplitz(c,r=None):
     vals = r_[r[rN-1:0:-1], c]
     cols = mgrid[0:cN]
     rows = mgrid[rN:0:-1]
-    indx = cols[:,NewAxis]*ones((1,rN)) + \
-           rows[NewAxis,:]*ones((cN,1)) - 1
-    return take(vals, indx)
+    indx = cols[:,newaxis]*ones((1,rN),dtype=int) + \
+           rows[newaxis,:]*ones((cN,1),dtype=int) - 1
+    return take(vals, indx, 0)
 
 
 def hankel(c,r=None):
     """ Construct a hankel matrix (i.e. matrix with constant anti-diagonals).
-    
+
         Description:
-    
+
           hankel(c,r) is a Hankel matrix whose first column is c and whose
           last row is r.
-    
+
           hankel(c) is a square Hankel matrix whose first column is C.
           Elements below the first anti-diagonal are zero.
-    
+
         See also:  toeplitz
     """
-    isscalar = scipy_base.isscalar
+    isscalar = numpy.isscalar
     if isscalar(c) or isscalar(r):
-        return c   
+        return c
     if r is None:
         r = zeros(len(c))
     elif r[0] != c[-1]:
@@ -484,12 +501,12 @@ def hankel(c,r=None):
     vals = r_[c, r[1:rN]]
     cols = mgrid[1:cN+1]
     rows = mgrid[0:rN]
-    indx = cols[:,NewAxis]*ones((1,rN)) + \
-           rows[NewAxis,:]*ones((cN,1)) - 1
-    return take(vals, indx)
+    indx = cols[:,newaxis]*ones((1,rN),dtype=int) + \
+           rows[newaxis,:]*ones((cN,1),dtype=int) - 1
+    return take(vals, indx, 0)
 
 def all_mat(*args):
-    return map(Matrix.Matrix,args)
+    return map(Matrix,args)
 
 def kron(a,b):
     """kronecker product of a and b
@@ -499,10 +516,10 @@ def kron(a,b):
      [ ...                                   ...   ],
      [ a[m-1,0]*b, a[m-1,1]*b, ... , a[m-1,n-1]*b  ]]
     """
-    if not a.iscontiguous():
+    if not a.flags['CONTIGUOUS']:
         a = reshape(a, a.shape)
-    if not b.iscontiguous():
+    if not b.flags['CONTIGUOUS']:
         b = reshape(b, b.shape)
-    o = outerproduct(a,b)
-    o.shape = a.shape + b.shape
+    o = outer(a,b)
+    o=o.reshape(a.shape + b.shape)
     return concatenate(concatenate(o, axis=1), axis=1)
