@@ -1,16 +1,18 @@
+## Automatically adapted for scipy Oct 21, 2005 by convertcode.py
+
 """Filter design.
 """
 
-import scipy_base
-from scipy_base.fastumath import *
-from scipy_base import atleast_1d, poly, polyval, roots, imag, real, asarray,\
-     allclose, Float, resize, pi, concatenate, absolute, logspace, c_
-from scipy import comb, special, optimize, linalg
+import numpy
+from numpy.core.umath import *
+from numpy import atleast_1d, poly, polyval, roots, imag, real, asarray,\
+     allclose, resize, pi, concatenate, absolute, logspace, c_
+from numpy import mintypecode, select
+from scipy import special, optimize, linalg
+from scipy.misc import comb
 import string, types
 
 
-MLab = scipy_base
-Num = MLab
 abs = absolute
 
 def findfreqs(num, den, N):
@@ -20,11 +22,11 @@ def findfreqs(num, den, N):
     if len(ep) == 0:
         ep = atleast_1d(-1000)+0j
 
-    ez = c_[Num.compress(ep.imag >=0, ep), Num.compress((abs(tz) < 1e5) & (tz.imag >=0),tz)]
+    ez = c_[numpy.compress(ep.imag >=0, ep,axis=-1), numpy.compress((abs(tz) < 1e5) & (tz.imag >=0),tz,axis=-1)]
 
     integ = abs(ez) < 1e-10
-    hfreq = Num.around(Num.log10(Num.max(3*abs(ez.real + integ)+1.5*ez.imag))+0.5)
-    lfreq = Num.around(Num.log10(0.1*Num.min(abs(real(ez+integ))+2*ez.imag))-0.5)
+    hfreq = numpy.around(numpy.log10(numpy.max(3*abs(ez.real + integ)+1.5*ez.imag))+0.5)
+    lfreq = numpy.around(numpy.log10(0.1*numpy.min(abs(real(ez+integ))+2*ez.imag))-0.5)
 
     w = logspace(lfreq, hfreq, N)
     return w
@@ -79,7 +81,7 @@ def freqz(b, a, worN=None, whole=0, plot=None):
            jw  B(e)    b[0] + b[1]e + .... + b[m]e
         H(e) = ---- = ------------------------------------
                   jw               -jw            -jnw
-               A(e)    a[0] + a[2]e + .... + a[n]e             
+               A(e)    a[0] + a[2]e + .... + a[n]e
 
     Inputs:
 
@@ -103,10 +105,10 @@ def freqz(b, a, worN=None, whole=0, plot=None):
         lastpoint = pi
     if worN is None:
         N = 512
-        w = Num.arange(0,lastpoint,lastpoint/N)
+        w = numpy.arange(0,lastpoint,lastpoint/N)
     elif isinstance(worN, types.IntType):
         N = worN
-        w = Num.arange(0,lastpoint,lastpoint/N)
+        w = numpy.arange(0,lastpoint,lastpoint/N)
     else:
         w = worN
     w = atleast_1d(w)
@@ -115,7 +117,7 @@ def freqz(b, a, worN=None, whole=0, plot=None):
     if not plot is None:
         plot(w, h)
     return w, h
-    
+
 def tf2zpk(b,a):
     """Return zero, pole, gain (z,p,k) representation from a numerator,
     denominator representation of a linear filter.
@@ -128,7 +130,7 @@ def tf2zpk(b,a):
     z = roots(b)
     p = roots(a)
     return z, p, k
-    
+
 def zpk2tf(z,p,k):
     """Return polynomial transfer function representation from zeros
     and poles
@@ -146,7 +148,7 @@ def zpk2tf(z,p,k):
     k = atleast_1d(k)
     if len(z.shape) > 1:
         temp = poly(z[0])
-        b = zeros((z.shape[0], z.shape[1]+1), temp.typecode())
+        b = zeros((z.shape[0], z.shape[1]+1), temp.dtype.char)
         if len(k) == 1:
             k = [k[0]]*z.shape[0]
         for i in range(z.shape[0]):
@@ -165,13 +167,13 @@ def normalize(b,a):
     if len(b.shape) > 2:
         raise ValueError, "Numerator polynomial must be rank-1 or rank-2 array."
     if len(b.shape) == 1:
-        b = asarray([b],b.typecode())
+        b = asarray([b],b.dtype.char)
     while a[0] == 0.0 and len(a) > 1:
         a = a[1:]
     while allclose(b[:,0], 0, rtol=1e-14) and (b.shape[-1] > 1):
         b = b[:,1:]
     if b.shape[0] == 1:
-        b = b[0]        
+        b = b[0]
     outb = b * (1.0) / a[0]
     outa = a * (1.0) / a[0]
     return outb, outa
@@ -188,7 +190,7 @@ def lp2lp(b,a,wo=1.0):
     d = len(a)
     n = len(b)
     M = max((d,n))
-    pwo = pow(wo,Num.arange(M-1,-1,-1))
+    pwo = pow(wo,numpy.arange(M-1,-1,-1))
     start1 = max((n-d,0))
     start2 = max((d-n,0))
     b = b * pwo[start1]/pwo[start2:]
@@ -201,18 +203,18 @@ def lp2hp(b,a,wo=1.0):
     """
     a,b = map(atleast_1d,(a,b))
     if type(wo) is type(a):
-        wo = wo[0]    
+        wo = wo[0]
     d = len(a)
     n = len(b)
     if wo != 1:
-        pwo = pow(wo,Num.arange(max((d,n))))
+        pwo = pow(wo,numpy.arange(max((d,n))))
     else:
-        pwo = Num.ones(max((d,n)),b.typecode())
+        pwo = numpy.ones(max((d,n)),b.dtype.char)
     if d >= n:
         outa = a[::-1] * pwo
         outb = resize(b,(d,))
         outb[n:] = 0.0
-        outb[:n] = b[::-1] * pwo[:n]        
+        outb[:n] = b[::-1] * pwo[:n]
     else:
         outb = b[::-1] * pwo
         outa = resize(a,(n,))
@@ -228,14 +230,12 @@ def lp2bp(b,a,wo=1.0, bw=1.0):
     a,b = map(atleast_1d,(a,b))
     D = len(a) - 1
     N = len(b) - 1
-    artype = b.typecode()
-    if artype not in ['F','D','f','d']:
-        artype = Num.Float
+    artype = mintypecode((a,b))
     ma = max([N,D])
     Np = N + ma
     Dp = D + ma
-    bprime = Num.zeros(Np+1,artype)
-    aprime = Num.zeros(Dp+1,artype)
+    bprime = numpy.zeros(Np+1,artype)
+    aprime = numpy.zeros(Dp+1,artype)
     wosq = wo*wo
     for j in range(Np+1):
         val = 0.0
@@ -251,7 +251,7 @@ def lp2bp(b,a,wo=1.0, bw=1.0):
                 if ma-i+2*k == j:
                     val += comb(i,k)*a[D-i]*(wosq)**(i-k) / bw**i
         aprime[Dp-j] = val
-        
+
     return normalize(bprime, aprime)
 
 def lp2bs(b,a,wo=1,bw=1):
@@ -261,14 +261,12 @@ def lp2bs(b,a,wo=1,bw=1):
     a,b = map(atleast_1d,(a,b))
     D = len(a) - 1
     N = len(b) - 1
-    artype = b.typecode()
-    if artype not in ['F','D','f','d']:
-        artype = Num.Float
+    artype = mintypecode((a,b))
     M = max([N,D])
     Np = M + M
     Dp = M + M
-    bprime = Num.zeros(Np+1,artype)
-    aprime = Num.zeros(Dp+1,artype)
+    bprime = numpy.zeros(Np+1,artype)
+    aprime = numpy.zeros(Dp+1,artype)
     wosq = wo*wo
     for j in range(Np+1):
         val = 0.0
@@ -284,7 +282,7 @@ def lp2bs(b,a,wo=1,bw=1):
                 if i+2*k == j:
                     val += comb(M-i,k)*a[D-i]*(wosq)**(M-i-k) * bw**i
         aprime[Dp-j] = val
-        
+
     return normalize(bprime, aprime)
 
 def bilinear(b,a,fs=1.0):
@@ -296,20 +294,20 @@ def bilinear(b,a,fs=1.0):
     a,b = map(atleast_1d,(a,b))
     D = len(a) - 1
     N = len(b) - 1
-    artype = Num.Float
+    artype = float
     M = max([N,D])
     Np = M
     Dp = M
-    bprime = Num.zeros(Np+1,artype)
-    aprime = Num.zeros(Dp+1,artype)
+    bprime = numpy.zeros(Np+1,artype)
+    aprime = numpy.zeros(Dp+1,artype)
     for j in range(Np+1):
         val = 0.0
         for i in range(N+1):
-             for k in range(i+1):
+            for k in range(i+1):
                 for l in range(M-i+1):
                     if k+l == j:
                         val += comb(i,k)*comb(M-i,l)*b[N-i]*pow(2*fs,i)*(-1)**k
-	bprime[j] = real(val)
+        bprime[j] = real(val)
     for j in range(Dp+1):
         val = 0.0
         for i in range(D+1):
@@ -317,8 +315,8 @@ def bilinear(b,a,fs=1.0):
                 for l in range(M-i+1):
                     if k+l == j:
                         val += comb(i,k)*comb(M-i,l)*a[D-i]*pow(2*fs,i)*(-1)**k
-	aprime[j] = real(val)
-        
+        aprime[j] = real(val)
+
     return normalize(bprime, aprime)
 
 def iirdesign(wp, ws, gpass, gstop, analog=0, ftype='ellip', output='ba'):
@@ -353,7 +351,7 @@ def iirdesign(wp, ws, gpass, gstop, analog=0, ftype='ellip', output='ba'):
     Outputs: (b,a) or (z,p,k)
 
       b,a -- Numerator and denominator of the iir filter.
-      z,p,k -- Zeros, poles, and gain of the iir filter.      
+      z,p,k -- Zeros, poles, and gain of the iir filter.
     """
 
     try:
@@ -366,12 +364,12 @@ def iirdesign(wp, ws, gpass, gstop, analog=0, ftype='ellip', output='ba'):
     wp = atleast_1d(wp)
     ws = atleast_1d(ws)
     band_type = 2*(len(wp)-1)
-    band_type +=1 
+    band_type +=1
     if wp[0] >= ws[0]:
         band_type += 1
 
     btype = {1:'lowpass', 2:'highpass', 3:'bandstop', 4:'bandpass'}[band_type]
-       
+
     N, Wn = ordfunc(wp, ws, gpass, gstop, analog=analog)
     return iirfilter(N, Wn, rp=gpass, rs=gstop, analog=analog, btype=btype, ftype=ftype, output=output)
 
@@ -382,7 +380,7 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
     Description:
 
       Design an Nth order lowpass digital or analog filter and return the filter
-      coefficients in (B,A) (numerator, denominator) or (Z,P,K) form.          
+      coefficients in (B,A) (numerator, denominator) or (Z,P,K) form.
 
     Inputs:
 
@@ -414,21 +412,21 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
 
     if output not in ['ba', 'zpk']:
         raise ValueError, "%s is not a valid output form." % output
-    
+
     #pre-warp frequencies for digital filter design
     if not analog:
         fs = 2.0
         warped = 2*fs*tan(pi*Wn/fs)
     else:
         warped = Wn
-    
+
     # convert to low-pass prototype
     if btype in ['lowpass', 'highpass']:
         wo = warped
     else:
         bw = warped[1] - warped[0]
-        wo = sqrt(warped[0]*warped[1])    
-    
+        wo = sqrt(warped[0]*warped[1])
+
     # Get analog lowpass prototype
     if typefunc in [buttap, besselap]:
         z, p, k = typefunc(N)
@@ -442,11 +440,11 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
         z, p, k = typefunc(N, rs)
     else:  # Elliptic filters
         if rs is None or rp is None:
-            raise ValueErrro, "Both rp and rs must be provided to design an elliptic filter."
+            raise ValueError, "Both rp and rs must be provided to design an elliptic filter."
         z, p, k = typefunc(N, rp, rs)
-    
+
     b, a = zpk2tf(z,p,k)
-    
+
     # transform to lowpass, bandpass, highpass, or bandstop
     if btype == 'lowpass':
         b, a = lp2lp(b,a,wo=wo)
@@ -456,18 +454,18 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
         b, a = lp2bp(b,a,wo=wo,bw=bw)
     else: # 'bandstop'
         b, a = lp2bs(b,a,wo=wo,bw=bw)
-               
+
 
     # Find discrete equivalent if necessary
     if not analog:
         b, a = bilinear(b, a, fs=fs)
-    
-    # Transform to proper out type (pole-zero, state-space, numer-denom)    
+
+    # Transform to proper out type (pole-zero, state-space, numer-denom)
     if output == 'zpk':
         return tf2zpk(b,a)
     else:
         return b,a
-      
+
 
 def butter(N, Wn, btype='low', analog=0, output='ba'):
     """Butterworth digital and analog filter design.
@@ -489,7 +487,7 @@ def cheby1(N, rp, Wn, btype='low', analog=0, output='ba'):
       Design an Nth order lowpass digital or analog Chebyshev type I filter
       and return the filter coefficients in (B,A) or (Z,P,K) form.
 
-    See also cheb1ord.      
+    See also cheb1ord.
     """
     return iirfilter(N, Wn, rp=rp, btype=btype, analog=analog, output=output, ftype='cheby1')
 
@@ -528,7 +526,7 @@ def bessel(N, Wn, btype='low', analog=0, output='ba'):
     """
     return iirfilter(N, Wn, btype=btype, analog=analog, output=output, ftype='bessel')
 
-    
+
 def maxflat():
     pass
 
@@ -608,7 +606,7 @@ def buttord(wp, ws, gpass, gstop, analog=0):
     Outputs: (ord, Wn)
 
       ord -- The lowest order for a Butterworth filter which meets specs.
-      Wn -- The Butterworth natural frequency (i.e. the "3dB frequency"). 
+      Wn -- The Butterworth natural frequency (i.e. the "3dB frequency").
             Should be used with scipy.signal.butter to give filter results.
 
     """
@@ -616,7 +614,7 @@ def buttord(wp, ws, gpass, gstop, analog=0):
     wp = atleast_1d(wp)
     ws = atleast_1d(ws)
     filter_type = 2*(len(wp)-1)
-    filter_type +=1 
+    filter_type +=1
     if wp[0] >= ws[0]:
         filter_type += 1
 
@@ -667,18 +665,18 @@ def buttord(wp, ws, gpass, gstop, analog=0):
     elif filter_type == 2: # high
         WN = passb / W0
     elif filter_type == 3:  # stop
-        WN = Num.zeros(2,Float)
+        WN = numpy.zeros(2,float)
         WN[0] = ((passb[1] - passb[0]) + sqrt((passb[1] - passb[0])**2 + \
                                         4*W0**2 * passb[0] * passb[1])) / (2*W0)
         WN[1] = ((passb[1] - passb[0]) - sqrt((passb[1] - passb[0])**2 + \
                                         4*W0**2 * passb[0] * passb[1])) / (2*W0)
-        WN = Num.sort(abs(WN))
+        WN = numpy.sort(abs(WN))
     elif filter_type == 4: # pass
-        W0 = Num.array([-W0, W0],Float)
+        W0 = numpy.array([-W0, W0],float)
         WN = -W0 * (passb[1]-passb[0]) / 2.0 + sqrt(W0**2 / 4.0 * \
                                               (passb[1]-passb[0])**2 + \
                                               passb[0]*passb[1])
-        WN = Num.sort(abs(WN))
+        WN = numpy.sort(abs(WN))
     else:
         raise ValueError, "Bad type."
 
@@ -757,7 +755,7 @@ def cheb1ord(wp, ws, gpass, gstop, analog=0):
     GPASS = 10**(0.1*abs(gpass))
     ord = int(ceil(arccosh(sqrt((GSTOP-1.0) / (GPASS-1.0))) / arccosh(nat)))
 
-    # Natural frequencies are just the passband edges 
+    # Natural frequencies are just the passband edges
     if not analog:
         wn = (2.0/pi)*arctan(passb)
     else:
@@ -766,7 +764,7 @@ def cheb1ord(wp, ws, gpass, gstop, analog=0):
     if len(wn) == 1:
         wn = wn[0]
     return ord, wn
-    
+
 
 def cheb2ord(wp, ws, gpass, gstop, analog=0):
     """Chebyshev type II filter order selection.
@@ -841,23 +839,23 @@ def cheb2ord(wp, ws, gpass, gstop, analog=0):
 
     new_freq = cosh(1.0/ord * arccosh(sqrt((GSTOP-1.0)/(GPASS-1.0))))
     new_freq = 1.0 / new_freq
-    
+
     if filter_type == 1:
         nat = passb / new_freq
     elif filter_type == 2:
         nat = passb * new_freq
     elif filter_type == 3:
-        nat = Num.zeros(2,Num.Float)
+        nat = numpy.zeros(2,float)
         nat[0] = new_freq / 2.0 * (passb[0]-passb[1]) + \
                  sqrt(new_freq**2 * (passb[1]-passb[0])**2 / 4.0 + \
                       passb[1] * passb[0])
         nat[1] = passb[1] * passb[0] / nat[0]
     elif filter_type == 4:
-        nat = Num.zeros(2,Num.Float)
+        nat = numpy.zeros(2,float)
         nat[0] = 1.0/(2.0*new_freq) * (passb[0] - passb[1]) + \
                  sqrt((passb[1]-passb[0])**2 / (4.0*new_freq**2) + \
                       passb[1] * passb[0])
-        nat[1] = passb[0] * passb[1] / nat[0]        
+        nat[1] = passb[0] * passb[1] / nat[0]
 
     if not analog:
         wn = (2.0/pi)*arctan(nat)
@@ -948,13 +946,13 @@ def ellipord(wp, ws, gpass, gstop, analog=0):
     if len(wn) == 1:
         wn = wn[0]
     return ord, wn
-    
+
 def buttap(N):
     """Return (z,p,k) zero, pole, gain for analog prototype of an Nth
     order Butterworth filter."""
     z = []
-    n = Num.arange(1,N+1)
-    p = Num.exp(1j*(2*n-1)/(2.0*N)*pi)*1j
+    n = numpy.arange(1,N+1)
+    p = numpy.exp(1j*(2*n-1)/(2.0*N)*pi)*1j
     k = 1
     return z, p, k
 
@@ -964,12 +962,12 @@ def cheb1ap(N,rp):
     in the passband.
     """
     z = []
-    eps = Num.sqrt(10**(0.1*rp)-1.0)
-    n = Num.arange(1,N+1)
-    mu = 1.0/N * Num.log((1.0+Num.sqrt(1+eps*eps)) / eps)
+    eps = numpy.sqrt(10**(0.1*rp)-1.0)
+    n = numpy.arange(1,N+1)
+    mu = 1.0/N * numpy.log((1.0+numpy.sqrt(1+eps*eps)) / eps)
     theta = pi/2.0 * (2*n-1.0)/N
-    p = -Num.sinh(mu)*Num.sin(theta) + 1j*Num.cosh(mu)*Num.cos(theta)
-    k = MLab.prod(-p).real
+    p = -numpy.sinh(mu)*numpy.sin(theta) + 1j*numpy.cosh(mu)*numpy.cos(theta)
+    k = numpy.prod(-p,axis=0).real
     if N % 2 == 0:
         k = k / sqrt((1+eps*eps))
     return z, p, k
@@ -985,18 +983,18 @@ def cheb2ap(N,rs):
 
     if N % 2:
         m = N - 1
-        n = Num.concatenate((Num.arange(1,N-1,2),Num.arange(N+2,2*N,2)))
+        n = numpy.concatenate((numpy.arange(1,N-1,2),numpy.arange(N+2,2*N,2)))
     else:
         m = N
-        n = Num.arange(1,2*N,2)
-        
+        n = numpy.arange(1,2*N,2)
+
     z = conjugate(1j / cos(n*pi/(2.0*N)))
-    p = exp(1j*(pi*Num.arange(1,2*N,2)/(2.0*N) + pi/2.0))
+    p = exp(1j*(pi*numpy.arange(1,2*N,2)/(2.0*N) + pi/2.0))
     p = sinh(mu) * p.real + 1j*cosh(mu)*p.imag
     p = 1.0 / p
-    k = (MLab.prod(-p)/MLab.prod(-z)).real
+    k = (numpy.prod(-p,axis=0)/numpy.prod(-z,axis=0)).real
     return z, p, k
-    
+
 
 EPSILON = 2e-16
 
@@ -1026,7 +1024,7 @@ def ellipap(N,rp,rs):
     in the passband and a stopband rs decibels down.
 
     See Chapter 12 and Chapter 5 of "Filter Design for Signal Processing",
-    by Lutova, Tosic, and Evans.  This is 
+    by Lutova, Tosic, and Evans.  This is
     """
     if N == 1:
         p = -sqrt(1.0/(10**(0.1*rp)-1.0))
@@ -1034,9 +1032,9 @@ def ellipap(N,rp,rs):
         z = []
         return z, p, k
 
-    eps = Num.sqrt(10**(0.1*rp)-1)
-    ck1 = eps / Num.sqrt(10**(0.1*rs)-1)
-    ck1p = Num.sqrt(1-ck1*ck1)
+    eps = numpy.sqrt(10**(0.1*rp)-1)
+    ck1 = eps / numpy.sqrt(10**(0.1*rs)-1)
+    ck1p = numpy.sqrt(1-ck1*ck1)
     if ck1p == 1:
         raise ValueError, "Cannot design a filter with given rp and rs specifications."
 
@@ -1052,19 +1050,19 @@ def ellipap(N,rp,rs):
     if m < 0 or m > 1:
         m = optimize.fminbound(kratio, 0, 1, args=(krat,), maxfun=250,
                                maxiter=250, disp=0)
-    
+
     capk = special.ellipk(m)
     ws = wp / sqrt(m)
     m1 = 1-m
 
-    j = Num.arange(1-N%2,N,2)
+    j = numpy.arange(1-N%2,N,2)
     jj = len(j)
 
-    [s,c,d,phi] = special.ellipj(j*capk/N,m*Num.ones(jj))
-    snew = Num.compress(abs(s) > EPSILON, s)
+    [s,c,d,phi] = special.ellipj(j*capk/N,m*numpy.ones(jj))
+    snew = numpy.compress(abs(s) > EPSILON, s,axis=-1)
     z = 1.0 / (sqrt(m)*snew)
     z = 1j*z
-    z = Num.concatenate((z,conjugate(z)))
+    z = numpy.concatenate((z,conjugate(z)))
 
     r = optimize.fmin(vratio, special.ellipk(m), args=(1./eps, ck1p*ck1p),
                       maxfun=250, maxiter=250, disp=0)
@@ -1074,14 +1072,14 @@ def ellipap(N,rp,rs):
     p = -(c*d*sv*cv + 1j*s*dv) / (1-(d*sv)**2.0)
 
     if N % 2:
-        newp = Num.compress(abs(p.imag) > EPSILON*Num.sqrt(MLab.sum(p*Num.conjugate(p)).real), p)
-        p = Num.concatenate((p,conjugate(newp)))
+        newp = numpy.compress(abs(p.imag) > EPSILON*numpy.sqrt(numpy.sum(p*numpy.conjugate(p),axis=0).real), p,axis=-1)
+        p = numpy.concatenate((p,conjugate(newp)))
     else:
-        p = Num.concatenate((p,conjugate(p)))
+        p = numpy.concatenate((p,conjugate(p)))
 
-    k = (MLab.prod(-p) / MLab.prod(-z)).real
+    k = (numpy.prod(-p,axis=0) / numpy.prod(-z,axis=0)).real
     if N % 2 == 0:
-        k = k / Num.sqrt((1+eps*eps))
+        k = k / numpy.sqrt((1+eps*eps))
 
     return z, p, k
 
@@ -1477,7 +1475,7 @@ band_dict = {'band':'bandpass',
              'highpass' : 'highpass',
              'h' : 'highpass'
              }
-             
+
 def kaiserord(ripple, width):
     """Design a Kaiser window to limit ripple and width of transition region.
 
@@ -1501,17 +1499,20 @@ def kaiserord(ripple, width):
     Oppenheim, Schafer, "Discrete-Time Signal Processing,", p.475-476.
     """
     A = abs(ripple)  # in case somebody is confused as to what's meant
-    beta = select([A>50, A>21],
-                  [0.1102*(A-8.7), 0.5842*(A-21)**(0.4) + 0.07866*(A-21)],
-                  0.0)
+    if (A>50):
+        beta = 0.1102*(A-8.7)
+    elif (A>21):
+        beta = 0.5842*(A-21)**0.4 + 0.07886*(A-21)
+    else:
+        beta = 0.0
     N = (A-8)/2.285/(pi*width)
-    return ceil(N), beta 
+    return ceil(N), beta
 
 def firwin(N, cutoff, width=None, window='hamming'):
     """FIR Filter Design using windowed ideal filter method.
 
     Inputs:
-    
+
       N      -- order of filter (number of taps)
       cutoff -- cutoff frequency of filter (normalized so that 1 corresponds to
                   Nyquist or pi radians / sample)
@@ -1523,7 +1524,7 @@ def firwin(N, cutoff, width=None, window='hamming'):
 
     Outputs:
 
-      h      -- coefficients of length N fir filter. 
+      h      -- coefficients of length N fir filter.
     """
 
     from signaltools import get_window
@@ -1536,9 +1537,6 @@ def firwin(N, cutoff, width=None, window='hamming'):
 
     win = get_window(window,N,fftbins=1)
     alpha = N//2
-    m = Num.arange(0,N)
+    m = numpy.arange(0,N)
     h = win*special.sinc(cutoff*(m-alpha))
-    return h / sum(h)
-    
-    
-    
+    return h / sum(h,axis=0)
