@@ -1,17 +1,19 @@
+## Automatically adapted for scipy Oct 21, 2005 by convertcode.py
+
 #
 # Author: Travis Oliphant 2001
 #
 
 from filter_design import tf2zpk, zpk2tf, normalize
-from Numeric import product, zeros, concatenate, \
-     array, dot, transpose, arange, ones, Float
-import Numeric
+import numpy
+from numpy import product, zeros, \
+     array, dot, transpose, arange, ones
+from numpy.oldnumeric import Float
 import scipy.interpolate as interpolate
 import scipy.integrate as integrate
 import scipy.linalg as linalg
-from scipy_base import r_, c_, eye, real, atleast_1d, atleast_2d, poly, \
+from numpy import r_, c_, eye, real, atleast_1d, atleast_2d, poly, \
      squeeze, diag, asarray
-from Matrix import Matrix as Mat
 
 def tf2ss(num, den):
     """Transfer function to state-space representation.
@@ -25,18 +27,18 @@ def tf2ss(num, den):
       A, B, C, D -- state space representation of the system.
     """
     # Controller canonical state-space representation.
-    #  if M+1 = len(num) and K+1 = len(den) then we must have M <= K 
+    #  if M+1 = len(num) and K+1 = len(den) then we must have M <= K
     #  states are found by asserting that X(s) = U(s) / D(s)
-    #  then Y(s) = N(s) * X(s) 
+    #  then Y(s) = N(s) * X(s)
     #
     #   A, B, C, and D follow quite naturally.
     #
     num, den = normalize(num, den)   # Strips zeros, checks arrays
     nn = len(num.shape)
     if nn == 1:
-        num = asarray([num],num.typecode())
-    M = num.shape[1] 
-    K = len(den) 
+        num = asarray([num],num.dtype.char)
+    M = num.shape[1]
+    K = len(den)
     if (M > K):
         raise ValueError, "Improper transfer function."
     if (M == 0 or K == 0):  # Null system
@@ -44,7 +46,7 @@ def tf2ss(num, den):
                array([], Float)
 
     # pad numerator to have same number of columns has denominator
-    num = c_[zeros((num.shape[0],K-M),num.typecode()), num]
+    num = c_[zeros((num.shape[0],K-M),num.dtype.char), num]
 
     if num.shape[-1] > 0:
         D = num[:,0]
@@ -53,7 +55,7 @@ def tf2ss(num, den):
 
     if K == 1:
         return array([], Float), array([], Float), array([], Float), D
-    
+
     frow = -array([den[1:]])
     A = r_[frow, eye(K-2, K-1)]
     B = eye(K-1, 1)
@@ -103,10 +105,10 @@ def abcd_normalize(A=None, B=None, C=None, D=None):
     if MD != MC:
         raise ValueError, "C and D must have the same number of rows."
     if ND != NB:
-        raise ValueErrro, "B and D must have the same number of columns."
+        raise ValueError, "B and D must have the same number of columns."
 
     return A, B, C, D
-    
+
 def ss2tf(A, B, C, D, input=0):
     """State-space to transfer function.
 
@@ -122,7 +124,7 @@ def ss2tf(A, B, C, D, input=0):
     """
     # transfer function is C (sI - A)**(-1) B + D
     A, B, C, D = map(asarray, (A, B, C, D))
-    # Check consistency and 
+    # Check consistency and
     #     make them all rank-2 arrays
     A, B, C, D = abcd_normalize(A, B, C, D)
 
@@ -139,16 +141,16 @@ def ss2tf(A, B, C, D, input=0):
 
     den = poly(A)
 
-    if (product(B.shape) == 0) and (product(C.shape) == 0):
-        num = Numeric.ravel(D)
-        if (product(D.shape) == 0) and (product(A.shape) == 0):
+    if (product(B.shape,axis=0) == 0) and (product(C.shape,axis=0) == 0):
+        num = numpy.ravel(D)
+        if (product(D.shape,axis=0) == 0) and (product(A.shape,axis=0) == 0):
             den = []
         end
         return num, den
 
-    num_states = A.shape[0] 
+    num_states = A.shape[0]
     type_test = A[:,0] + B[:,0] + C[0,:] + D
-    num = Numeric.zeros((nout, num_states+1),type_test.typecode())
+    num = numpy.zeros((nout, num_states+1),type_test.dtype.char)
     for k in range(nout):
         Ck = atleast_2d(C[k,:])
         num[k] = poly(A - dot(B,Ck)) + (D[k]-1)*den
@@ -179,7 +181,7 @@ def ss2zpk(A,B,C,D,input=0):
     Outputs:
 
       z, p, k -- zeros and poles in sequences and gain constant.
-    """    
+    """
     return tf2zpk(*ss2tf(A,B,C,D,input=input))
 
 class lti:
@@ -223,8 +225,8 @@ class lti:
             self.__dict__['zeros'], self.__dict__['poles'], \
                                     self.__dict__['gain'] = ss2zpk(*args)
             self.__dict__['num'], self.__dict__['den'] = ss2tf(*args)
-            self.inputs = B.shape[-1]
-            self.outputs = C.shape[0]
+            self.inputs = self.B.shape[-1]
+            self.outputs = self.C.shape[0]
         else:
             raise ValueError, "Needs 2, 3, or 4 arguments."
 
@@ -256,7 +258,7 @@ class lti:
                                            self.C, self.D)
             self.__dict__['num'], self.__dict__['den'] = \
                                   ss2tf(self.A, self.B,
-                                        self.C, self.D)            
+                                        self.C, self.D)
         else:
             self.__dict__[attr] = val
 
@@ -283,7 +285,7 @@ def lsim2(system, U, T, X0=None):
                   4 (A, B, C, D)
       U -- an input array describing the input at each time T
            (linear interpolation is assumed between given times).
-           If there are multiple inputs, then each column of the 
+           If there are multiple inputs, then each column of the
            rank-2 array represents an input.
       T -- the time steps at which the input is defined and at which
            the output is desired.
@@ -306,7 +308,7 @@ def lsim2(system, U, T, X0=None):
     #   being the number of inputs
 
     # rather than use lsim, use direct integration and matrix-exponential.
-    if isinstance(system, lti):        
+    if isinstance(system, lti):
         sys = system
     else:
         sys = lti(*system)
@@ -314,7 +316,7 @@ def lsim2(system, U, T, X0=None):
     T = atleast_1d(T)
     if len(U.shape) == 1:
         U.shape = (U.shape[0],1)
-    sU = U.shape        
+    sU = U.shape
     if len(T.shape) != 1:
         raise ValueError, "T must be a rank-1 array."
     if sU[0] != len(T):
@@ -323,7 +325,7 @@ def lsim2(system, U, T, X0=None):
         raise ValueError, "System does not define that many inputs."
 
     if X0 is None:
-        X0 = zeros(sys.B.shape[0],sys.A.typecode())
+        X0 = zeros(sys.B.shape[0],sys.A.dtype.char)
 
     # for each output point directly integrate assume zero-order hold
     #   or linear interpolation.
@@ -351,7 +353,7 @@ def lsim(system, U, T, X0=None, interp=1):
                   4 (A, B, C, D)
       U -- an input array describing the input at each time T
            (interpolation is assumed between given times).
-           If there are multiple inputs, then each column of the 
+           If there are multiple inputs, then each column of the
            rank-2 array represents an input.
       T -- the time steps at which the input is defined and at which
            the output is desired.
@@ -381,7 +383,7 @@ def lsim(system, U, T, X0=None, interp=1):
     T = atleast_1d(T)
     if len(U.shape) == 1:
         U.shape = (U.shape[0],1)
-    sU = U.shape        
+    sU = U.shape
     if len(T.shape) != 1:
         raise ValueError, "T must be a rank-1 array."
     if sU[0] != len(T):
@@ -390,9 +392,9 @@ def lsim(system, U, T, X0=None, interp=1):
         raise ValueError, "System does not define that many inputs."
 
     if X0 is None:
-        X0 = zeros(sys.B.shape[0],sys.A.typecode())
+        X0 = zeros(sys.B.shape[0],sys.A.dtype.char)
 
-    xout = zeros((len(T),sys.B.shape[0]),sys.A.typecode())
+    xout = zeros((len(T),sys.B.shape[0]),sys.A.dtype.char)
     xout[0] = X0
     A = sys.A
     AT, BT = transpose(sys.A), transpose(sys.B)
@@ -400,10 +402,10 @@ def lsim(system, U, T, X0=None, interp=1):
     lam, v = linalg.eig(A)
     vt = transpose(v)
     vti = linalg.inv(vt)
-    GT = dot(dot(vti,diag(Numeric.exp(dt*lam))),vt).astype(xout.typecode())
+    GT = dot(dot(vti,diag(numpy.exp(dt*lam))),vt).astype(xout.dtype.char)
     ATm1 = linalg.inv(AT)
     ATm2 = dot(ATm1,ATm1)
-    I = eye(A.shape[0],typecode=A.typecode())
+    I = eye(A.shape[0],dtype=A.dtype.char)
     GTmI = GT-I
     F1T = dot(dot(BT,GTmI),ATm1)
     if interp:
@@ -413,7 +415,7 @@ def lsim(system, U, T, X0=None, interp=1):
         dt1 = T[k] - T[k-1]
         if dt1 != dt:
             dt = dt1
-            GT = dot(dot(vti,diag(Numeric.exp(dt*lam))),vt).astype(xout.typecode())
+            GT = dot(dot(vti,diag(numpy.exp(dt*lam))),vt).astype(xout.dtype.char)
             GTmI = GT-I
             F1T = dot(dot(BT,GTmI),ATm1)
             if interp:
@@ -458,13 +460,13 @@ def impulse(system, X0=None, T=None, N=None):
         vals = linalg.eigvals(sys.A)
         tc = 1.0/min(abs(real(vals)))
         T = arange(0,7*tc,7*tc / float(N))
-    h = zeros(T.shape, sys.A.typecode())
+    h = zeros(T.shape, sys.A.dtype.char)
     s,v = linalg.eig(sys.A)
     vi = linalg.inv(v)
     C = sys.C
     for k in range(len(h)):
-        es = diag(Numeric.exp(s*T[k]))
-        eA = (dot(dot(v,es),vi)).astype(h.typecode())
+        es = diag(numpy.exp(s*T[k]))
+        eA = (dot(dot(v,es),vi)).astype(h.dtype.char)
         h[k] = squeeze(dot(dot(C,eA),B))
     return T, h
 
@@ -495,7 +497,6 @@ def step(system, X0=None, T=None, N=None):
         vals = linalg.eigvals(sys.A)
         tc = 1.0/min(abs(real(vals)))
         T = arange(0,7*tc,7*tc / float(N))
-    U = ones(T.shape, sys.A.typecode())
+    U = ones(T.shape, sys.A.dtype.char)
     vals = lsim(sys, U, T, X0=X0)
     return vals[0], vals[1]
-    
