@@ -1,22 +1,24 @@
 # Author: Travis Oliphant
 # 1999 -- 2002
 
-import sigtools
-import scipy.special as special
-import scipy.linalg as linalg
-from scipy.fftpack import fft, ifft, ifftshift, fft2, ifft2
-from scipy_base import polyadd, polymul, polydiv, polysub, \
-     roots, poly, polyval, polyder, cast, asarray, isscalar
 import types
-import scipy
-from scipy.stats import mean
-import Numeric
-from Numeric import array, arange, where, sqrt, rank, zeros, NewAxis, argmax
-from scipy_base.fastumath import *
+import sigtools
+from scipy import special, linalg
+from scipy.fftpack import fft, ifft, ifftshift, fft2, ifft2
+from numpy import polyadd, polymul, polydiv, polysub, \
+     roots, poly, polyval, polyder, cast, asarray, isscalar, atleast_1d, \
+     ones, sin, linspace, real, extract, real_if_close, zeros, array, arange, \
+     where, sqrt, rank, newaxis, argmax, product, cos, pi, exp, \
+     ravel, size, less_equal, sum, r_, iscomplexobj, take, \
+     argsort, allclose, expand_dims, unique, prod, sort, reshape, c_, \
+     transpose, dot, any, minimum, maximum, mean
+import numpy
+from scipy.fftpack import fftn, ifftn
+from scipy.misc import factorial
 
 _modedict = {'valid':0, 'same':1, 'full':2}
 _boundarydict = {'fill':0, 'pad':0, 'wrap':2, 'circular':2, 'symm':1, 'symmetric':1, 'reflect':4}
-                                                                            
+
 def _valfrommode(mode):
     try:
         val = _modedict[mode]
@@ -59,14 +61,14 @@ def correlate(in1, in2, mode='full'):
 
     out -- an N-dimensional array containing a subset of the discrete linear
            cross-correlation of in1 with in2.
- 
+
     """
     # Code is faster if kernel is smallest array.
     volume = asarray(in1)
     kernel = asarray(in2)
     if rank(volume) == rank(kernel) == 0:
         return volume*kernel
-    if (Numeric.product(kernel.shape) > Numeric.product(volume.shape)):
+    if (product(kernel.shape,axis=0) > product(volume.shape,axis=0)):
         temp = kernel
         kernel = volume
         volume = temp
@@ -76,6 +78,42 @@ def correlate(in1, in2, mode='full'):
 
     return sigtools._correlateND(volume, kernel, val)
 
+def _centered(arr, newsize):
+    # Return the center newsize portion of the array.
+    newsize = asarray(newsize)
+    currsize = array(arr.shape)
+    startind = (currsize - newsize) / 2
+    endind = startind + newsize
+    myslice = [slice(startind[k], endind[k]) for k in range(len(endind))]
+    return arr[tuple(myslice)]
+
+def fftconvolve(in1, in2, mode="full"):
+    """Convolve two N-dimensional arrays using FFT. See convolve.
+    """
+    s1 = array(in1.shape)
+    s2 = array(in2.shape)
+    if (s1.dtype.char in ['D','F']) or (s2.dtype.char in ['D', 'F']):
+        cmplx=1
+    else: cmplx=0
+    size = s1+s2-1
+    IN1 = fftn(in1,size)
+    IN1 *= fftn(in2,size)
+    ret = ifftn(IN1)
+    del IN1
+    if not cmplx:
+        ret = real(ret)
+    if mode == "full":
+        return ret
+    elif mode == "same":
+        if product(s1,axis=0) > product(s2,axis=0):
+            osize = s1
+        else:
+            osize = s2
+        return _centered(ret,osize)
+    elif mode == "valid":
+        return _centered(ret,abs(s2-s1)+1)
+
+
 def convolve(in1, in2, mode='full'):
     """Convolve two N-dimensional arrays.
 
@@ -106,7 +144,7 @@ def convolve(in1, in2, mode='full'):
     kernel = asarray(in2)
     if rank(volume) == rank(kernel) == 0:
         return volume*kernel
-    if (Numeric.product(kernel.shape) > Numeric.product(volume.shape)):
+    if (product(kernel.shape,axis=0) > product(volume.shape,axis=0)):
         temp = kernel
         kernel = volume
         volume = temp
@@ -114,12 +152,12 @@ def convolve(in1, in2, mode='full'):
 
     slice_obj = [slice(None,None,-1)]*len(kernel.shape)
     val = _valfrommode(mode)
-    
+
     return sigtools._correlateND(volume,kernel[slice_obj],val)
 
 def order_filter(a, domain, order):
     """Perform an order filter on an N-dimensional array.
-    
+
   Description:
 
     Perform an order filter on the array in.  The domain argument acts as a
@@ -127,7 +165,7 @@ def order_filter(a, domain, order):
     used to select elements surrounding each input pixel which are placed
     in a list.   The list is sorted, and the output for that pixel is the
     element corresponding to rank in the sorted list.
-    
+
   Inputs:
 
     in -- an N-dimensional input array.
@@ -141,7 +179,7 @@ def order_filter(a, domain, order):
 
     out -- the results of the order filter in an array with the same
            shape as in.
-          
+
     """
     domain = asarray(domain)
     size = domain.shape
@@ -149,7 +187,7 @@ def order_filter(a, domain, order):
         if (size[k] % 2) != 1:
             raise ValueError, "Each dimension of domain argument should have an odd number of elements."
     return sigtools._orderfilterND(a, domain, rank)
-   
+
 
 def medfilt(volume,kernel_size=None):
     """Perform a median filter on an N-dimensional array.
@@ -171,33 +209,33 @@ def medfilt(volume,kernel_size=None):
 
     out -- An array the same size as input containing the median filtered
            result.
-  
+
     """
     volume = asarray(volume)
     if kernel_size is None:
         kernel_size = [3] * len(volume.shape)
     kernel_size = asarray(kernel_size)
     if len(kernel_size.shape) == 0:
-        kernel_size = [kernel_size.toscalar()] * len(volume.shape)
+        kernel_size = [kernel_size.item()] * len(volume.shape)
     kernel_size = asarray(kernel_size)
 
     for k in range(len(volume.shape)):
         if (kernel_size[k] % 2) != 1:
-            raise ValueError, "Each element of kernel_size should be odd." 
+            raise ValueError, "Each element of kernel_size should be odd."
 
-    domain = Numeric.ones(kernel_size)
+    domain = ones(kernel_size)
 
-    numels = Numeric.product(kernel_size)
+    numels = product(kernel_size,axis=0)
     order = int(numels/2)
     return sigtools._order_filterND(volume,domain,order)
 
 
 def wiener(im,mysize=None,noise=None):
-    """Perform a wiener filter on an N-dimensional array.
+    """Perform a Wiener filter on an N-dimensional array.
 
   Description:
 
-    Apply a wiener filter to the N-dimensional array in.
+    Apply a Wiener filter to the N-dimensional array in.
 
   Inputs:
 
@@ -220,14 +258,14 @@ def wiener(im,mysize=None,noise=None):
     mysize = asarray(mysize);
 
     # Estimate the local mean
-    lMean = correlate(im,Numeric.ones(mysize),1) / Numeric.product(mysize)
+    lMean = correlate(im,ones(mysize),1) / product(mysize,axis=0)
 
     # Estimate the local variance
-    lVar = correlate(im**2,Numeric.ones(mysize),1) / Numeric.product(mysize) - lMean**2
+    lVar = correlate(im**2,ones(mysize),1) / product(mysize,axis=0) - lMean**2
 
     # Estimate the noise power if needed.
     if noise==None:
-        noise = mean(Numeric.ravel(lVar))
+        noise = mean(ravel(lVar),axis=0)
 
     res = (im - lMean)
     res *= (1-noise / lVar)
@@ -235,7 +273,7 @@ def wiener(im,mysize=None,noise=None):
     out = where(lVar < noise, lMean, res)
 
     return out
-    
+
 
 def convolve2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     """Conolve two 2-dimensional arrays.
@@ -270,7 +308,7 @@ def convolve2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     """
     val = _valfrommode(mode)
     bval = _bvalfromboundary(boundary)
-        
+
     return sigtools._convolve2d(in1,in2,1,val,bval,fillvalue)
 
 def correlate2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
@@ -306,7 +344,7 @@ def correlate2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     """
     val = _valfrommode(mode)
     bval = _bvalfromboundary(boundary)
-        
+
     return sigtools._convolve2d(in1, in2, 0,val,bval,fillvalue)
 
 def medfilt2d(input, kernel_size=3):
@@ -335,19 +373,19 @@ def medfilt2d(input, kernel_size=3):
         kernel_size = [3] * 2
     kernel_size = asarray(kernel_size)
     if len(kernel_size.shape) == 0:
-        kernel_size = [kernel_size.toscalar()] * 2
+        kernel_size = [kernel_size.item()] * 2
     kernel_size = asarray(kernel_size)
 
     for size in kernel_size:
         if (size % 2) != 1:
-            raise ValueError, "Each element of kernel_size should be odd." 
+            raise ValueError, "Each element of kernel_size should be odd."
 
     return sigtools._medfilt2d(image, kernel_size)
 
 def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
           maxiter=25, grid_density=16):
     """Calculate the minimax optimal filter using Remez exchange algorithm.
-    
+
   Description:
 
     Calculate the filter-coefficients for the finite impulse response
@@ -372,7 +410,7 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
 
     out -- A rank-1 array containing the coefficients of the optimal
            (in a minimax sense) filter.
-    
+
     """
     # Convert type
     try:
@@ -384,7 +422,7 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
     if weight is None:
         weight = [1] * len(desired)
 
-    bands = bands.copy()
+    bands = asarray(bands).copy()
     return sigtools._remez(numtaps, bands, desired, weight, tnum, Hz,
                            maxiter, grid_density)
 
@@ -396,7 +434,7 @@ def lfilter(b, a, x, axis=-1, zi=None):
     Filter a data sequence, x, using a digital filter.  This works for many
     fundamental data types (including Object type).  The filter is a direct
     form II transposed implementation of the standard difference equation
-    (see "Algorithm"). 
+    (see "Algorithm").
 
   Inputs:
 
@@ -444,10 +482,10 @@ def lfilter(b, a, x, axis=-1, zi=None):
             Y(z) = ---------------------------------- X(z)
                                 -1               -na
                     a[0] + a[1]z  + ... + a[na] z
-                    
+
     """
     if isscalar(a):
-	a = [a]
+        a = [a]
     if zi is None:
         return sigtools._linear_filter(b, a, x, axis)
     else:
@@ -459,7 +497,7 @@ def lfiltic(b,a,y,x=None):
     which is used by lfilter to generate the output given the input.
 
     If M=len(b)-1 and N=len(a)-1.  Then, the initial conditions are given
-    in the vectors x and y as 
+    in the vectors x and y as
 
     x = {x[-1],x[-2],...,x[-M]}
     y = {y[-1],y[-2],...,y[-N]}
@@ -472,53 +510,70 @@ def lfiltic(b,a,y,x=None):
 
     zi = {z_0[-1], z_1[-1], ..., z_K-1[-1]}  where K=max(M,N).
     """
-    N = Numeric.size(a)-1
-    M = Numeric.size(b)-1
+    N = size(a)-1
+    M = size(b)-1
     K = max(M,N)
     y = asarray(y)
-    zi = zeros(K,y.typecode())
+    zi = zeros(K,y.dtype.char)
     if x is None:
-        x = zeros(M,y.typecode())
+        x = zeros(M,y.dtype.char)
     else:
         x = asarray(x)
-        L = Numeric.size(x)
+        L = size(x)
         if L < M:
             x = r_[x,zeros(M-L)]
-    L = Numeric.size(y)
+    L = size(y)
     if L < N:
         y = r_[y,zeros(N-L)]
 
     for m in range(M):
-        zi[m] = Numeric.sum(b[m+1:]*x[:M-m])
+        zi[m] = sum(b[m+1:]*x[:M-m],axis=0)
 
     for m in range(N):
-        zi[m] -= Numeric.sum(a[m+1:]*y[:N-m])
+        zi[m] -= sum(a[m+1:]*y[:N-m],axis=0)
 
     return zi
-    
+
+def deconvolve(signal, divisor):
+    """Deconvolves divisor out of signal.
+    """
+    num = atleast_1d(signal)
+    den = atleast_1d(divisor)
+    N = len(num)
+    D = len(den)
+    if D > N:
+        quot = [];
+        rem = num;
+    else:
+        input = ones(N-D+1, float)
+        input[1:] = 0
+        quot = lfilter(num, den, input)
+        rem = num - convolve(den, quot, mode='full')
+    return quot, rem
+
 
 def boxcar(M,sym=1):
     """The M-point boxcar window.
     """
-    return Numeric.ones(M,Numeric.Float)
+    return ones(M, float)
 
 def triang(M,sym=1):
     """The M-point triangular window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M + 1        
+        M = M + 1
     n = arange(1,int((M+1)/2)+1)
     if M % 2 == 0:
         w = (2*n-1.0)/M
-        w = scipy.r_[w, w[::-1]]
+        w = r_[w, w[::-1]]
     else:
         w = 2*n/(M+1.0)
-        w = scipy.r_[w, w[-2::-1]]
+        w = r_[w, w[-2::-1]]
 
     if not sym and not odd:
         w = w[:-1]
@@ -528,18 +583,18 @@ def parzen(M,sym=1):
     """The M-point Parzen window
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1    
-    n = Numeric.arange(-(M-1)/2.0,(M-1)/2.0+0.5,1.0)
+        M = M+1
+    n = arange(-(M-1)/2.0,(M-1)/2.0+0.5,1.0)
     na = extract(n < -(M-1)/4.0, n)
     nb = extract(abs(n) <= (M-1)/4.0, n)
     wa = 2*(1-abs(na)/(M/2.0))**3.0
     wb = 1-6*(abs(nb)/(M/2.0))**2.0 + 6*(abs(nb)/(M/2.0))**3.0
-    w = scipy.r_[wa,wb,wa[::-1]]
+    w = r_[wa,wb,wa[::-1]]
     if not sym and not odd:
         w = w[:-1]
     return w
@@ -548,15 +603,15 @@ def bohman(M,sym=1):
     """The M-point Bohman window
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1    
+        M = M+1
     fac = abs(linspace(-1,1,M)[1:-1])
     w = (1 - fac)* cos(pi*fac) + 1.0/pi*sin(pi*fac)
-    w = scipy.r_[0,w,0]    
+    w = r_[0,w,0]
     if not sym and not odd:
         w = w[:-1]
     return w
@@ -565,9 +620,9 @@ def blackman(M,sym=1):
     """The M-point Blackman window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
         M = M+1
@@ -581,51 +636,51 @@ def nuttall(M,sym=1):
     """A minimum 4-term Blackman-Harris window according to Nuttall.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1    
+        M = M+1
     a = [0.3635819, 0.4891775, 0.1365995, 0.0106411]
     n = arange(0,M)
     fac = n*2*pi/(M-1.0)
     w = a[0] - a[1]*cos(fac) + a[2]*cos(2*fac) - a[3]*cos(3*fac)
     if not sym and not odd:
-        w = w[:-1]    
+        w = w[:-1]
     return w
 
 def blackmanharris(M,sym=1):
     """The M-point minimum 4-term Blackman-Harris window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1        
+        M = M+1
     a = [0.35875, 0.48829, 0.14128, 0.01168];
     n = arange(0,M)
     fac = n*2*pi/(M-1.0)
     w = a[0] - a[1]*cos(fac) + a[2]*cos(2*fac) - a[3]*cos(3*fac)
     if not sym and not odd:
-        w = w[:-1]    
+        w = w[:-1]
     return w
 
-    
+
 def bartlett(M,sym=1):
     """The M-point Bartlett window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1    
+        M = M+1
     n = arange(0,M)
-    w = where(Numeric.less_equal(n,(M-1)/2.0),2.0*n/(M-1),2.0-2.0*n/(M-1))
+    w = where(less_equal(n,(M-1)/2.0),2.0*n/(M-1),2.0-2.0*n/(M-1))
     if not sym and not odd:
         w = w[:-1]
     return w
@@ -634,12 +689,12 @@ def hanning(M,sym=1):
     """The M-point Hanning window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1        
+        M = M+1
     n = arange(0,M)
     w = 0.5-0.5*cos(2.0*pi*n/(M-1))
     if not sym and not odd:
@@ -650,30 +705,30 @@ def barthann(M,sym=1):
     """Return the M-point modified Bartlett-Hann window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1            
+        M = M+1
     n = arange(0,M)
     fac = abs(n/(M-1.0)-0.5)
     w = 0.62 - 0.48*fac + 0.38*cos(2*pi*fac)
     if not sym and not odd:
         w = w[:-1]
-    return w    
+    return w
 
 def hamming(M,sym=1):
     """The M-point Hamming window.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1        
-    n = arange(0,M)    
+        M = M+1
+    n = arange(0,M)
     w = 0.54-0.46*cos(2.0*pi*n/(M-1))
     if not sym and not odd:
         w = w[:-1]
@@ -683,12 +738,12 @@ def kaiser(M,beta,sym=1):
     """Returns a Kaiser window of length M with shape parameter beta.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1    
+        M = M+1
     n = arange(0,M)
     alpha = (M-1)/2.0
     w = special.i0(beta * sqrt(1-((n-alpha)/alpha)**2.0))/special.i0(beta)
@@ -700,10 +755,10 @@ def gaussian(M,std,sym=1):
     """Returns a Gaussian window of length M with standard-deviation std.
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
-    odd = M % 2        
+        return ones(1,'d')
+    odd = M % 2
     if not sym and not odd:
         M = M + 1
     n = arange(0,M)-(M-1.0)/2.0
@@ -721,12 +776,12 @@ def general_gaussian(M,p,sig,sym=1):
     half power point is at (2*log(2)))**(1/(2*p))*sig
     """
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
-        M = M+1        
+        M = M+1
     n = arange(0,M)-(M-1.0)/2.0
     w = exp(-0.5*(n/sig)**(2*p))
     if not sym and not odd:
@@ -739,9 +794,9 @@ def slepian(M,width,sym=1):
         raise ValueError, "Cannot reliably obtain slepian sequences for"\
               " M*width > 27.38."
     if M < 1:
-        return Numeric.array([])
+        return array([])
     if M == 1:
-        return Numeric.ones(1,'d')
+        return ones(1,'d')
     odd = M % 2
     if not sym and not odd:
         M = M+1
@@ -749,18 +804,18 @@ def slepian(M,width,sym=1):
     twoF = width/2.0
     alpha = (M-1)/2.0
     m = arange(0,M)-alpha
-    n = m[:,NewAxis]
-    k = m[NewAxis,:]
+    n = m[:,newaxis]
+    k = m[newaxis,:]
     AF = twoF*special.sinc(twoF*(n-k))
     [lam,vec] = linalg.eig(AF)
-    ind = argmax(abs(lam))
+    ind = argmax(abs(lam),axis=-1)
     w = abs(vec[:,ind])
     w = w / max(w)
-    
+
     if not sym and not odd:
         w = w[:-1]
     return w
-            
+
 
 def hilbert(x, N=None):
     """Return the hilbert transform of x of length N.
@@ -770,11 +825,11 @@ def hilbert(x, N=None):
         N = len(x)
     if N <=0:
         raise ValueError, "N must be positive."
-    if scipy.iscomplexobj(x):
+    if iscomplexobj(x):
         print "Warning: imaginary part of x ignored."
-        x = scipy.real(x)
+        x = real(x)
     Xf = fft(x,N,axis=0)
-    h = Numeric.zeros(N)
+    h = zeros(N)
     if N % 2 == 0:
         h[0] = h[N/2] = 1
         h[1:N/2] = 2
@@ -783,7 +838,7 @@ def hilbert(x, N=None):
         h[1:(N+1)/2] = 2
 
     if len(x.shape) > 1:
-        h = h[:,Numeric.NewAxis]
+        h = h[:, newaxis]
     x = ifft(Xf*h)
     return x
 
@@ -798,13 +853,13 @@ def hilbert2(x,N=None):
         if N <=0:
             raise ValueError, "N must be positive."
         N = (N,N)
-    if scipy.iscomplexobj(x):
+    if iscomplexobj(x):
         print "Warning: imaginary part of x ignored."
-        x = scipy.real(x)
+        x = real(x)
     print N
     Xf = fft2(x,N,axes=(0,1))
-    h1 = Numeric.zeros(N[0],'d')
-    h2 = Numeric.zeros(N[1],'d')
+    h1 = zeros(N[0],'d')
+    h2 = zeros(N[1],'d')
     for p in range(2):
         h = eval("h%d"%(p+1))
         N1 = N[p]
@@ -816,23 +871,23 @@ def hilbert2(x,N=None):
             h[1:(N1+1)/2] = 2
         exec("h%d = h" % (p+1), globals(), locals())
 
-    h = h1[:,NewAxis] * h2[NewAxis,:]
+    h = h1[:,newaxis] * h2[newaxis,:]
     k = len(x.shape)
     while k > 2:
-        h = h[:,Numeric.NewAxis]
+        h = h[:, newaxis]
         k -= 1
     x = ifft2(Xf*h,axes=(0,1))
     return x
-    
+
 
 def cmplx_sort(p):
     "sort roots based on magnitude."
     p = asarray(p)
-    if scipy.iscomplexobj(p):
-        indx = Numeric.argsort(abs(p))
+    if iscomplexobj(p):
+        indx = argsort(abs(p))
     else:
-        indx = Numeric.argsort(p)
-    return Numeric.take(p,indx), indx
+        indx = argsort(p)
+    return take(p,indx,0), indx
 
 def unique_roots(p,tol=1e-3,rtype='min'):
     """Determine the unique roots and their multiplicities in two lists
@@ -841,7 +896,7 @@ def unique_roots(p,tol=1e-3,rtype='min'):
 
       p -- The list of roots
       tol --- The tolerance for two roots to be considered equal.
-      rtype --- How to determine the returned root from the close 
+      rtype --- How to determine the returned root from the close
                   ones:  'max': pick the maximum
                          'min': pick the minimum
                          'avg': average roots
@@ -851,11 +906,11 @@ def unique_roots(p,tol=1e-3,rtype='min'):
       mult -- The multiplicity of each root
     """
     if rtype in ['max','maximum']:
-        comproot = scipy.max
+        comproot = numpy.maximum
     elif rtype in ['min','minimum']:
-        comproot = scipy.min
+        comproot = numpy.minimum
     elif rtype in ['avg','mean']:
-        comproot = scipy.mean
+        comproot = numpy.mean
     p = asarray(p)*1.0
     tol = abs(tol)
     p, indx = cmplx_sort(p)
@@ -874,20 +929,18 @@ def unique_roots(p,tol=1e-3,rtype='min'):
         else:
             pout.append(tr)
             curp = tr
-            sameroots = [tr]            
+            sameroots = [tr]
             indx += 1
             mult.append(1)
     return array(pout), array(mult)
 
-from scipy_base import real_if_close, atleast_1d
-
 
 def invres(r,p,k,tol=1e-3,rtype='avg'):
     """Compute b(s) and a(s) from partial fraction expansion: r,p,k
 
     If M = len(b) and N = len(a)
-    
-            b(s)     b[0] x**(M-1) + b[1] x**(M-2) + ... + b[M-1] 
+
+            b(s)     b[0] x**(M-1) + b[1] x**(M-2) + ... + b[M-1]
     H(s) = ------ = ----------------------------------------------
             a(s)     a[0] x**(N-1) + a[1] x**(N-2) + ... + a[N-1]
 
@@ -899,14 +952,14 @@ def invres(r,p,k,tol=1e-3,rtype='avg'):
     fraction expansion has terms like
 
             r[i]      r[i+1]              r[i+n-1]
-          -------- + ----------- + ... + ----------- 
+          -------- + ----------- + ... + -----------
           (s-p[i])  (s-p[i])**2          (s-p[i])**n
 
     See also:  residue, poly, polyval, unique_roots
     """
     extra = k
     p, indx = cmplx_sort(p)
-    r = Numeric.take(r,indx)
+    r = take(r,indx,0)
     pout, mult = unique_roots(p,tol=tol,rtype=rtype)
     p = []
     for k in range(len(pout)):
@@ -928,17 +981,16 @@ def invres(r,p,k,tol=1e-3,rtype='avg'):
             b = polyadd(b,r[indx]*poly(t2))
             indx += 1
     b = real_if_close(b)
-    while Numeric.allclose(b[0], 0, rtol=1e-14) and (b.shape[-1] > 1):
+    while allclose(b[0], 0, rtol=1e-14) and (b.shape[-1] > 1):
         b = b[1:]
     return b, a
 
-from scipy import factorial
 def residue(b,a,tol=1e-3,rtype='avg'):
     """Compute partial-fraction expansion of b(s) / a(s).
 
     If M = len(b) and N = len(a)
-    
-            b(s)     b[0] s**(M-1) + b[1] s**(M-2) + ... + b[M-1] 
+
+            b(s)     b[0] s**(M-1) + b[1] s**(M-2) + ... + b[M-1]
     H(s) = ------ = ----------------------------------------------
             a(s)     a[0] s**(N-1) + a[1] s**(N-2) + ... + a[N-1]
 
@@ -947,10 +999,10 @@ def residue(b,a,tol=1e-3,rtype='avg'):
            (s-p[0])   (s-p[1])         (s-p[-1])
 
     If there are any repeated roots (closer than tol), then the partial
-    fraction expansion has terms like 
+    fraction expansion has terms like
 
             r[i]      r[i+1]              r[i+n-1]
-          -------- + ----------- + ... + ----------- 
+          -------- + ----------- + ... + -----------
           (s-p[i])  (s-p[i])**2          (s-p[i])**n
 
     See also:  invres, poly, polyval, unique_roots
@@ -983,7 +1035,7 @@ def residue(b,a,tol=1e-3,rtype='avg'):
                 term1 = polymul(polyder(bn,1),an)
                 term2 = polymul(bn,polyder(an,1))
                 bn = polysub(term1,term2)
-                an = polymul(an,an)                
+                an = polymul(an,an)
             r[indx+m-1] = polyval(bn,pout[n]) / polyval(an,pout[n]) \
                           / factorial(sig-m)
         indx += sig
@@ -993,7 +1045,7 @@ def residuez(b,a,tol=1e-3,rtype='avg'):
     """Compute partial-fraction expansion of b(z) / a(z).
 
     If M = len(b) and N = len(a)
-    
+
             b(z)     b[0] + b[1] z**(-1) + ... + b[M-1] z**(-M+1)
     H(z) = ------ = ----------------------------------------------
             a(z)     a[0] + a[1] z**(-1) + ... + a[N-1] z**(-N+1)
@@ -1003,10 +1055,10 @@ def residuez(b,a,tol=1e-3,rtype='avg'):
            (1-p[0]z**(-1))         (1-p[-1]z**(-1))
 
     If there are any repeated roots (closer than tol), then the partial
-    fraction expansion has terms like 
+    fraction expansion has terms like
 
                r[i]              r[i+1]                    r[i+n-1]
-          -------------- + ------------------ + ... + ------------------ 
+          -------------- + ------------------ + ... + ------------------
           (1-p[i]z**(-1))  (1-p[i]z**(-1))**2         (1-p[i]z**(-1))**n
 
     See also:  invresz, poly, polyval, unique_roots
@@ -1018,7 +1070,7 @@ def residuez(b,a,tol=1e-3,rtype='avg'):
     if krev == []:
         k = []
     else:
-        k = krev[::-1]    
+        k = krev[::-1]
     b = brev[::-1]
     p = roots(a)
     r = p*0.0
@@ -1048,7 +1100,7 @@ def residuez(b,a,tol=1e-3,rtype='avg'):
                 term1 = polymul(polyder(bn,1),an)
                 term2 = polymul(bn,polyder(an,1))
                 bn = polysub(term1,term2)
-                an = polymul(an,an)                
+                an = polymul(an,an)
             r[indx+m-1] = polyval(bn,1.0/pout[n]) / polyval(an,1.0/pout[n]) \
                           / factorial(sig-m) / (-pout[n])**(sig-m)
         indx += sig
@@ -1058,7 +1110,7 @@ def invresz(r,p,k,tol=1e-3,rtype='avg'):
     """Compute b(z) and a(z) from partial fraction expansion: r,p,k
 
     If M = len(b) and N = len(a)
-    
+
             b(z)     b[0] + b[1] z**(-1) + ... + b[M-1] z**(-M+1)
     H(z) = ------ = ----------------------------------------------
             a(z)     a[0] + a[1] z**(-1) + ... + a[N-1] z**(-N+1)
@@ -1068,17 +1120,17 @@ def invresz(r,p,k,tol=1e-3,rtype='avg'):
            (1-p[0]z**(-1))         (1-p[-1]z**(-1))
 
     If there are any repeated roots (closer than tol), then the partial
-    fraction expansion has terms like 
+    fraction expansion has terms like
 
                r[i]              r[i+1]                    r[i+n-1]
-          -------------- + ------------------ + ... + ------------------ 
+          -------------- + ------------------ + ... + ------------------
           (1-p[i]z**(-1))  (1-p[i]z**(-1))**2         (1-p[i]z**(-1))**n
 
     See also:  residuez, poly, polyval, unique_roots
     """
     extra = asarray(k)
     p, indx = cmplx_sort(p)
-    r = Numeric.take(r,indx)
+    r = take(r,indx,0)
     pout, mult = unique_roots(p,tol=tol,rtype=rtype)
     p = []
     for k in range(len(pout)):
@@ -1090,7 +1142,7 @@ def invresz(r,p,k,tol=1e-3,rtype='avg'):
         b = [0]
     indx = 0
     brev = asarray(b)[::-1]
-    for k in range(len(pout)):  
+    for k in range(len(pout)):
         temp = []
         # Construct polynomial which does not include any of this root
         for l in range(len(pout)):
@@ -1109,7 +1161,7 @@ def get_window(window,Nx,fftbins=1):
     """Return a window of length Nx and type window.
 
     If fftbins is 1, create a "periodic" window ready to use with ifftshift
-    and be multiplied by the result of an fft (SEE ALSO fftfreq). 
+    and be multiplied by the result of an fft (SEE ALSO fftfreq).
 
     Window types:  boxcar, triang, blackman, hamming, hanning, bartlett,
                    parzen, bohman, blackmanharris, nuttall, barthann,
@@ -1141,7 +1193,7 @@ def get_window(window,Nx,fftbins=1):
                 raise ValueError, "That window needs a parameter -- pass a tuple"
             else:
                 winstr = window
-                
+
         if winstr in ['blackman', 'black', 'blk']:
             winfunc = blackman
         elif winstr in ['triangle', 'triang', 'tri']:
@@ -1153,16 +1205,16 @@ def get_window(window,Nx,fftbins=1):
         elif winstr in ['hanning', 'hann', 'han']:
             winfunc = hanning
         elif winstr in ['blackmanharris', 'blackharr','bkh']:
-            winfun = blackmanharris
+            winfunc = blackmanharris
         elif winstr in ['parzen', 'parz', 'par']:
-            winfun = parzen
+            winfunc = parzen
         elif winstr in ['bohman', 'bman', 'bmn']:
             winfunc = bohman
         elif winstr in ['nuttall', 'nutl', 'nut']:
             winfunc = nuttall
         elif winstr in ['barthann', 'brthan', 'bth']:
             winfunc = barthann
-            
+
         elif winstr in ['kaiser', 'ksr']:
             winfunc = kaiser
         elif winstr in ['gaussian', 'gauss', 'gss']:
@@ -1183,7 +1235,7 @@ def get_window(window,Nx,fftbins=1):
         params = (Nx,beta,sym)
 
     return winfunc(*params)
-        
+
 
 def resample(x,num,t=None,axis=0,window=None):
     """Resample to num samples using Fourier method along the given axis.
@@ -1225,20 +1277,20 @@ def resample(x,num,t=None,axis=0,window=None):
         W = ifftshift(get_window(window,Nx))
         newshape = ones(len(x.shape))
         newshape[axis] = len(W)
-        W.shape = newshape
+        W=W.reshape(newshape)
         X = X*W
     sl = [slice(None)]*len(x.shape)
     newshape = list(x.shape)
     newshape[axis] = num
-    N = int(Numeric.minimum(num,Nx))
-    Y = Numeric.zeros(newshape,'D')
+    N = int(numpy.minimum(num,Nx))
+    Y = zeros(newshape,'D')
     sl[axis] = slice(0,(N+1)/2)
     Y[sl] = X[sl]
     sl[axis] = slice(-(N-1)/2,None)
     Y[sl] = X[sl]
     y = ifft(Y,axis=axis)*(float(num)/float(Nx))
 
-    if x.typecode() not in ['F','D']:
+    if x.dtype.char not in ['F','D']:
         y = y.real
 
     if t is None:
@@ -1247,10 +1299,6 @@ def resample(x,num,t=None,axis=0,window=None):
         new_t = arange(0,num)*(t[1]-t[0])* Nx / float(num) + t[0]
         return y, new_t
 
-from scipy_base import expand_dims, unique, prod, sort, zeros, ones, \
-     reshape, r_, any, c_, transpose, take, dot
-
-import scipy.linalg as linalg
 def detrend(data, axis=-1, type='linear', bp=0):
     """Remove linear trend along axis from data.
 
@@ -1262,7 +1310,7 @@ def detrend(data, axis=-1, type='linear', bp=0):
     if type not in ['linear','l','constant','c']:
         raise ValueError, "Trend type must be linear or constant"
     data = asarray(data)
-    dtype = data.typecode()
+    dtype = data.dtype.char
     if dtype not in 'dfDF':
         dtype = 'd'
     if type in ['constant','c']:
@@ -1280,9 +1328,9 @@ def detrend(data, axis=-1, type='linear', bp=0):
         rnk = len(dshape)
         if axis < 0: axis = axis + rnk
         newdims = r_[axis,0:axis,axis+1:rnk]
-        newdata = reshape(transpose(data,tuple(newdims)),(N,prod(dshape)/N))
+        newdata = reshape(transpose(data,tuple(newdims)),(N,prod(dshape,axis=0)/N))
         newdata = newdata.copy()  # make sure we have a copy
-        if newdata.typecode() not in 'dfDF':
+        if newdata.dtype.char not in 'dfDF':
             newdata = newdata.astype(dtype)
         # Find leastsq fit and remove it for each piece
         for m in range(Nreg):
@@ -1293,10 +1341,9 @@ def detrend(data, axis=-1, type='linear', bp=0):
             coef,resids,rank,s = linalg.lstsq(A,newdata[sl])
             newdata[sl] = newdata[sl] - dot(A,coef)
         # Put data back in original shape.
-        tdshape = take(dshape,newdims)
-        ret = reshape(newdata,tdshape)
+        tdshape = take(dshape,newdims,0)
+        ret = reshape(newdata,tuple(tdshape))
         vals = range(1,rnk)
         olddims = vals[:axis] + [0] + vals[axis:]
         ret = transpose(ret,tuple(olddims))
         return ret
-
