@@ -1,11 +1,12 @@
-# 
+## Automatically adapted for scipy Oct 05, 2005 by convertcode.py
+
+#
 # Author:  Travis Oliphant, 2002
 #
 
-from cephes import *
-from scipy_base import *
+from numpy import *
+from _cephes import *
 import types
-from scipy_base.fastumath import *
 import specfun
 
 def sinc(x):
@@ -24,25 +25,25 @@ def diric(x,n):
     x,n = asarray(x), asarray(n)
     n = asarray(n + (x-x))
     x = asarray(x + (n-n))
-    if x.typecode() in ['fFdD']:
-        ytype = x.typecode()
+    if x.dtype.char in ['fFdD']:
+        ytype = x.dtype.char
     else:
         ytype = 'd'
     y = zeros(x.shape,ytype)
 
     mask1 = (n <= 0) | (n <> floor(n))
-    insert(y,mask1,nan)
+    place(y,mask1,nan)
 
     z = asarray(x / 2.0 / pi)
     mask2 = (1-mask1) & (z == floor(z))
     zsub = extract(mask2,z)
     nsub = extract(mask2,n)
-    insert(y,mask2,pow(-1,zsub*(nsub-1)))
+    place(y,mask2,pow(-1,zsub*(nsub-1)))
 
     mask = (1-mask1) & (1-mask2)
     xsub = extract(mask,x)
     nsub = extract(mask,n)
-    insert(y,mask,sin(nsub*xsub/2.0)/(nsub*sin(xsub/2.0)))
+    place(y,mask,sin(nsub*xsub/2.0)/(nsub*sin(xsub/2.0)))
     return y
 
 
@@ -52,7 +53,7 @@ def jnjnp_zeros(nt):
     and arange them in order of their magnitudes.
 
     Outputs (all are arrays of length nt):
-    
+
        zo[l-1] -- Value of the lth zero of of Jn(x) and Jn'(x)
        n[l-1]  -- Order of the Jn(x) or Jn'(x) associated with lth zero
        m[l-1]  -- Serial number of the zeros of Jn(x) or Jn'(x) associated
@@ -62,11 +63,11 @@ def jnjnp_zeros(nt):
 
     See jn_zeros, jnp_zeros to get separated arrays of zeros.
     """
-    if not isscalar(nt) or (floor(nt)!=nt) or (nt>1400):
-        raise ValueError, "Number must be integer <=1400."
+    if not isscalar(nt) or (floor(nt)!=nt) or (nt>1200):
+        raise ValueError, "Number must be integer <= 1200."
     nt = int(nt)
-    zo,n,m,t = specfunc.jdzo(nt)
-    return zo,n,m,t
+    n,m,t,zo = specfun.jdzo(nt)
+    return zo[:nt],n[:nt],m[:nt],t[:nt]
 
 def jnyn_zeros(n,nt):
     """Compute nt zeros of the Bessel functions Jn(x), Jn'(x), Yn(x), and
@@ -128,7 +129,19 @@ def y1p_zeros(nt,complex=0):
     kf = 2
     kc = (complex != 1)
     return specfun.cyzo(nt,kf,kc)
-    
+
+def bessel_diff_formula(v, z, n, L, phase):
+    # from AMS55.
+    # L(v,z) = J(v,z), Y(v,z), H1(v,z), H2(v,z), phase = -1
+    # L(v,z) = I(v,z) or exp(v*pi*i)K(v,z), phase = 1
+    # For K, you can pull out the exp((v-k)*pi*i) into the caller
+    p = 1.0
+    s = L(v-n, z)
+    for i in xrange(1, n+1):
+        p = phase * (p * (n-i+1)) / i   # = choose(k, i)
+        s += p*L(v-n + i*2, z)
+    return s / (2.**n)
+
 def jvp(v,z,n=1):
     """Return the nth derivative of Jv(z) with respect to z.
     """
@@ -137,27 +150,29 @@ def jvp(v,z,n=1):
     if n == 0:
         return jv(v,z)
     else:
-        return (jvp(v-1,z,n-1) - jvp(v+1,z,n-1))/2.0
+        return bessel_diff_formula(v, z, n, jv, -1)
+#        return (jvp(v-1,z,n-1) - jvp(v+1,z,n-1))/2.0
 
 def yvp(v,z,n=1):
-    """Return the nth derivative of Yv(z) with respect to z. 
+    """Return the nth derivative of Yv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if n == 0:
         return yv(v,z)
     else:
-        return (yvp(v-1,z,n-1) - yvp(v+1,z,n-1))/2.0
+        return bessel_diff_formula(v, z, n, yv, -1)
+#        return (yvp(v-1,z,n-1) - yvp(v+1,z,n-1))/2.0
 
 def kvp(v,z,n=1):
-    """Return the nth derivative of Kv(z) with respect to z. 
+    """Return the nth derivative of Kv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if n == 0:
         return kv(v,z)
     else:
-        return (kvp(v-1,z,n-1) - kvp(v+1,z,n-1))/2.0
+        return (-1)**n * bessel_diff_formula(v, z, n, kv, 1)
 
 def ivp(v,z,n=1):
     """Return the nth derivative of Iv(z) with respect to z.
@@ -167,7 +182,7 @@ def ivp(v,z,n=1):
     if n == 0:
         return iv(v,z)
     else:
-        return (ivp(v-1,z,n-1) - ivp(v+1,z,n-1))/2.0
+        return bessel_diff_formula(v, z, n, iv, 1)
 
 def h1vp(v,z,n=1):
     """Return the nth derivative of H1v(z) with respect to z.
@@ -177,7 +192,8 @@ def h1vp(v,z,n=1):
     if n == 0:
         return hankel1(v,z)
     else:
-        return (h1vp(v-1,z,n-1) - h1vp(v+1,z,n-1))/2.0
+        return bessel_diff_formula(v, z, n, hankel1, -1)
+#        return (h1vp(v-1,z,n-1) - h1vp(v+1,z,n-1))/2.0
 
 def h2vp(v,z,n=1):
     """Return the nth derivative of H2v(z) with respect to z.
@@ -187,7 +203,8 @@ def h2vp(v,z,n=1):
     if n == 0:
         return hankel2(v,z)
     else:
-        return (h2vp(v-1,z,n-1) - h2vp(v+1,z,n-1))/2.0
+        return bessel_diff_formula(v, z, n, hankel2, -1)
+#        return (h2vp(v-1,z,n-1) - h2vp(v+1,z,n-1))/2.0
 
 def sph_jn(n,z):
     """Compute the spherical Bessel function jn(z) and its derivative for
@@ -198,11 +215,11 @@ def sph_jn(n,z):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
-    else: n1 = n    
-    if iscomplex(z):
+    else: n1 = n
+    if any(iscomplex(z)):
         nm,jn,jnp,yn,ynp = specfun.csphjy(n1,z)
     else:
-        nm,jn,jnp = specfun.sphj(n1,z)        
+        nm,jn,jnp = specfun.sphj(n1,z)
     return jn[:(n+1)], jnp[:(n+1)]
 
 def sph_yn(n,z):
@@ -214,8 +231,8 @@ def sph_yn(n,z):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
-    else: n1 = n 
-    if iscomplex(z) or (z<0):
+    else: n1 = n
+    if any(iscomplex(z)) or any(z<0):
         nm,jn,jnp,yn,ynp = specfun.csphjy(n1,z)
     else:
         nm,yn,ynp = specfun.sphy(n1,z)
@@ -230,8 +247,8 @@ def sph_jnyn(n,z):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
-    else: n1 = n    
-    if iscomplex(z) or (z<0):
+    else: n1 = n
+    if any(iscomplex(z)) or any(z<0):
         nm,jn,jnp,yn,ynp = specfun.csphjy(n1,z)
     else:
         nm,yn,ynp = specfun.sphy(n1,z)
@@ -248,10 +265,10 @@ def sph_in(n,z):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
     else: n1 = n
-    if iscomplex(z):
+    if any(iscomplex(z)):
         nm,In,Inp,kn,knp = specfun.csphik(n1,z)
     else:
-        nm,In,Inp = specfun.sphi(n1,z)        
+        nm,In,Inp = specfun.sphi(n1,z)
     return In[:(n+1)], Inp[:(n+1)]
 
 def sph_kn(n,z):
@@ -263,8 +280,8 @@ def sph_kn(n,z):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
-    else: n1 = n    
-    if iscomplex(z) or (z<0):
+    else: n1 = n
+    if any(iscomplex(z)) or any(z<0):
         nm,In,Inp,kn,knp = specfun.csphik(n1,z)
     else:
         nm,kn,knp = specfun.sphk(n1,z)
@@ -278,7 +295,7 @@ def sph_inkn(n,z):
         raise ValueError, "arguments must be scalars."
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
-    if iscomplex(z) or (z<0):
+    if any(iscomplex(z)) or any(z<0):
         nm,In,Inp,kn,knp = specfun.csphik(n,z)
     else:
         nm,In,Inp = specfun.sphi(n,z)
@@ -307,7 +324,7 @@ def riccati_yn(n,x):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n == 0): n1 = 1
-    else: n1 = n 
+    else: n1 = n
     nm,jn,jnp = specfun.rcty(n1,x)
     return jn[:(n+1)],jnp[:(n+1)]
 
@@ -372,7 +389,7 @@ def hyp0f1(v,z):
     Limit as q->infinity of 1F1(q;a;z/q)
     """
     z = asarray(z)
-    if z.typecode() in ['F', 'D']:
+    if z.dtype.char in ['F', 'D']:
         arg = 2*sqrt(abs(z))
         num = where(z>=0, iv(v-1,arg), jv(v-1,arg))
         den = abs(z)**((v-1.0)/2)
@@ -395,7 +412,7 @@ def polygamma(n, x):
     n, x = asarray(n), asarray(x)
     cond = (n==0)
     fac2 = (-1.0)**(n+1) * gamma(n+1.0) * zeta(n+1,x)
-    if sometrue(cond):
+    if sometrue(cond,axis=0):
         return where(cond, psi(x), fac2)
     return fac2
 
@@ -450,11 +467,11 @@ def mathieu_odd_coef(m,q):
         kd = 3
 
     b = mathieu_b(m,q)
-    fc = specfunc.fcoef(kd,m,q,b)
+    fc = specfun.fcoef(kd,m,q,b)
     return fc[:km]
 
 def lpmn(m,n,z):
-    """Associated Legendre functions of the second kind, Pmn(z) and its
+    """Associated Legendre functions of the first kind, Pmn(z) and its
     derivative, Pmn'(z) of order m and degree n.  Returns two
     arrays of size (m+1,n+1) containing Pmn(z) and Pmn'(z) for
     all orders from 0..m and degrees from 0..n.
@@ -475,9 +492,9 @@ def lpmn(m,n,z):
         sv = errprint(sv)
     else:
         mp = m
-    if iscomplex(z):
+    if any(iscomplex(z)):
         p,pd = specfun.clpmn(mp,n,real(z),imag(z))
-    else:        
+    else:
         p,pd = specfun.lpmn(mp,n,z)
     if (m < 0):
         p = p * fixarr
@@ -495,7 +512,7 @@ def lqmn(m,n,z):
     z can be complex.
     """
     if not isscalar(m) or (m<0):
-        raise ValueError, "m must be a non-negative integer."    
+        raise ValueError, "m must be a non-negative integer."
     if not isscalar(n) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if not isscalar(z):
@@ -505,9 +522,9 @@ def lqmn(m,n,z):
     if (m*n == 0):
         mm = max(1,m)
         nn = max(1,n)
-    if iscomplex(z):
+    if any(iscomplex(z)):
         q,qd = specfun.clqmn(mm,nn,z)
-    else:        
+    else:
         q,qd = specfun.lqmn(mm,nn,z)
     return q[:(m+1),:(n+1)],qd[:(m+1),:(n+1)]
 
@@ -531,7 +548,7 @@ def euler(n):
     if (n < 2): n1 = 2
     else:  n1 = n
     return specfun.eulerb(n1)[:(n+1)]
-    
+
 def lpn(n,z):
     """Compute sequence of Legendre functions of the first kind (polynomials),
     Pn(z) and derivatives for all degrees from 0 to n (inclusive).
@@ -544,14 +561,14 @@ def lpn(n,z):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
     else: n1 = n
-    if iscomplex(z):
+    if any(iscomplex(z)):
         pn,pd = specfun.clpn(n1,z)
     else:
         pn,pd = specfun.lpn(n1,z)
     return pn[:(n+1)],pd[:(n+1)]
 
 ## lpni
-    
+
 def lqn(n,z):
     """Compute sequence of Legendre functions of the second kind,
     Qn(z) and derivatives for all degrees from 0 to n (inclusive).
@@ -561,8 +578,8 @@ def lqn(n,z):
     if (n!= floor(n)) or (n<0):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
-    else: n1 = n    
-    if iscomplex(z):
+    else: n1 = n
+    if any(iscomplex(z)):
         qn,qd = specfun.clqn(n1,z)
     else:
         qn,qd = specfun.lqnb(n1,z)
@@ -583,7 +600,7 @@ def ai_zeros(nt):
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
         raise ValueError, "nt must be a positive integer scalar."
     return specfun.airyzo(nt,kf)
-    
+
 def bi_zeros(nt):
     """Compute the zeros of Airy Functions Bi(x) and Bi'(x), b and b'
     respectively, and the associated values of Ai(b') and Ai'(b).
@@ -629,7 +646,7 @@ def pbdv_seq(v,x):
     v0 = v-n
     if (n < 2): n1=2
     else: n1 = n
-    v1 = n1 + v0    
+    v1 = n1 + v0
     dv,dp,pdf,pdd = specfun.pbdv(v1,x)
     return dv[:(n1+1)],dp[:(n1+1)]
 
@@ -734,7 +751,7 @@ def kelvin_zeros(nt):
            specfun.klvnzo(nt,6), \
            specfun.klvnzo(nt,7), \
            specfun.klvnzo(nt,8)
-    
+
 def pro_cv_seq(m,n,c):
     """Compute a sequence of characteristic values for the prolate
     spheroidal wave functions for mode m and n'=m..n and spheroidal
@@ -748,7 +765,7 @@ def pro_cv_seq(m,n,c):
         raise ValueError, "Difference between n and m is too large."
     maxL = n-m+1
     return specfun.segv(m,n,c,1)[1][:maxL]
-    
+
 def obl_cv_seq(m,n,c):
     """Compute a sequence of characteristic values for the oblate
     spheroidal wave functions for mode m and n'=m..n and spheroidal
@@ -763,3 +780,21 @@ def obl_cv_seq(m,n,c):
     maxL = n-m+1
     return specfun.segv(m,n,c,-1)[1][:maxL]
 
+def agm(a,b):
+    """Arithmetic, Geometric Mean
+
+    Start with a_0=a and b_0=b and iteratively compute
+
+    a_{n+1} = (a_n+b_n)/2
+    b_{n+1} = sqrt(a_n*b_n)
+
+    until a_n=b_n.   The result is agm(a,b)
+
+    agm(a,b)=agm(b,a)
+    agm(a,a) = a
+    min(a,b) < agm(a,b) < max(a,b)
+    """
+    res1 = a+b+0.0
+    res2 = a-b
+    k = res2 / res1
+    return res1*pi/4/ellipk(k**2)
