@@ -1,3 +1,5 @@
+## Automatically adapted for scipy Oct 05, 2005 by convertcode.py
+
 #!/usr/bin/env python
 #
 # Author:  Travis Oliphant 2000
@@ -33,14 +35,14 @@ B_n = ( a4n / a3n sqrt(h_n-1 / h_n))**2
 assume:
 P_0(x) = 1
 P_-1(x) == 0
-             
+
 See Numerical Recipies in C, page 156 and
 Abramowitz and Stegun p. 774, 782
 
 Functions:
 
   gen_roots_and_weights  -- Generic roots and weights.
-  j_roots                -- Jacobi 
+  j_roots                -- Jacobi
   js_roots               -- Shifted Jacobi
   la_roots               -- Generalized Laguerre
   h_roots                -- Hermite
@@ -58,9 +60,11 @@ Functions:
 """
 
 from __future__ import nested_scopes
-from scipy_base import *
-import cephes
+from numpy import *
+from numpy.oldnumeric import take
+import _cephes as cephes
 _gam = cephes.gamma
+from scipy.linalg import eig
 
 def poch(z,m):
     """Pochhammer symbol (z)_m = (z)(z+1)....(z+m-1) = gamma(z+m)/gamma(z)"""
@@ -69,8 +73,8 @@ def poch(z,m):
 class orthopoly1d(poly1d):
     def __init__(self, roots, weights=None, hn=1.0, kn=1.0, wfunc=None, limits=None, monic=0):
         poly1d.__init__(self, roots, r=1)
-	equiv_weights = [weights[k] / wfunc(roots[k]) for k in range(len(roots))]
-	self.__dict__['weights'] = array(zip(roots,weights,equiv_weights)) 
+        equiv_weights = [weights[k] / wfunc(roots[k]) for k in range(len(roots))]
+        self.__dict__['weights'] = array(zip(roots,weights,equiv_weights))
         self.__dict__['weight_func'] = wfunc
         self.__dict__['limits'] = limits
         mu = sqrt(hn)
@@ -81,29 +85,6 @@ class orthopoly1d(poly1d):
         self.__dict__['coeffs'] *= kn
 
 
-_eigfunc_cache = None
-def get_eig_func():
-    global _eigfunc_cache
-    if _eigfunc_cache is not None:
-        return _eigfunc_cache
-    try:
-        import scipy.linalg
-        eig = scipy.linalg.eig
-    except ImportError:
-        try:
-            import linalg
-            eig = linalg.eig
-        except ImportError:
-            try:
-                import LinearAlgebra
-                eig = LinearAlgebra.eigenvectors
-            except:
-                raise ImportError, \
-                      "You must have scipy.linalg or LinearAlgebra to "\
-                      "use this function."
-    _eigfunc_cache = eig
-    return eig
-
 def gen_roots_and_weights(n,an_func,sqrt_bn_func,mu):
     """[x,w] = gen_roots_and_weights(n,an_func,sqrt_bn_func,mu)
 
@@ -118,16 +99,15 @@ def gen_roots_and_weights(n,an_func,sqrt_bn_func,mu):
     sqrt_bn_func(n)     should return sqrt(B_n)
     mu ( = h_0 )        is the integral of the weight over the orthogonal interval
     """
-    eig = get_eig_func()
     nn = arange(1.0,n)
     sqrt_bn = sqrt_bn_func(nn)
     an = an_func(concatenate(([0],nn)))
     [x,v] = eig((diag(an)+diag(sqrt_bn,1)+diag(sqrt_bn,-1)))
     answer = []
     sortind = argsort(real(x))
-    answer.append(take(x,sortind))
-    answer.append(take(mu*v[0]**2,sortind))
-    return answer    
+    answer.append(take(x,sortind,axis=0))
+    answer.append(take(mu*v[0]**2,sortind,axis=0))
+    return answer
 
 # Jacobi Polynomials 1               P^(alpha,beta)_n(x)
 def j_roots(n,alpha,beta,mu=0):
@@ -137,7 +117,7 @@ def j_roots(n,alpha,beta,mu=0):
     and weights (w) to use in Gaussian Quadrature over [-1,1] with weighting
     function (1-x)**alpha (1+x)**beta with alpha,beta > -1.
     """
-    if (alpha <= -1) or (beta <= -1):
+    if any(alpha <= -1) or any(beta <= -1):
         raise ValueError, "alpha and beta must be greater than -1."
     assert(n>0), "n must be positive."
 
@@ -145,7 +125,7 @@ def j_roots(n,alpha,beta,mu=0):
     # from recurrence relations
     sbn_J = lambda k: 2.0/(2.0*k+p+q)*sqrt((k+p)*(k+q)/(2*k+q+p+1)) * \
                 (where(k==1,1.0,sqrt(k*(k+p+q)/(2.0*k+p+q-1))))
-    if (p == q):
+    if any(p == q):  # XXX any or all???
         an_J = lambda k: 0.0*k
     else:
         an_J = lambda k: where(k==0,(q-p)/(p+q+2.0),
@@ -184,11 +164,11 @@ def js_roots(n,p1,q1,mu=0):
     function (1-x)**(p-q) x**(q-1) with p-q > -1 and q > 0.
     """
     # from recurrence relation
-    if not ( ( (p1 - q1) > -1 ) and ( q1 > 0 ) ):
+    if not ( any( (p1 - q1) > -1 ) and any( q1 > 0 ) ):
         raise ValueError, "(p - q) > -1 and q > 0 please."
     if (n <= 0):
         raise ValueError, "n must be positive."
-    
+
     p,q = p1,q1
 
     sbn_Js = lambda k: sqrt(where(k==1,q*(p-q+1.0)/(p+2.0), \
@@ -213,7 +193,7 @@ def js_roots(n,p1,q1,mu=0):
     #    return [(x+1)/2.0,w,mu0]
     #else:
     #    [x,w] = j_roots(n,p-q,q-1,mu=0)
-    #    return [(x+1)/2.0,w]    
+    #    return [(x+1)/2.0,w]
 
 def sh_jacobi(n, p, q, monic=0):
     """Returns the nth order Jacobi polynomial, G_n(p,q,x)
@@ -224,7 +204,7 @@ def sh_jacobi(n, p, q, monic=0):
         raise ValueError, "n must be nonnegative"
     wfunc = lambda x: (1.0-x)**(p-q) * (x)**(q-1.)
     if n==0: return orthopoly1d([],[],1.0,1.0,wfunc,(-1,1),monic)
-    n1 = n  
+    n1 = n
     x,w,mu0 = js_roots(n1,p,q,mu=1)
     hn = _gam(n+1)*_gam(n+q)*_gam(n+p)*_gam(n+p-q+1)
     hn /= (2*n+p)*(_gam(2*n+p)**2)
@@ -241,13 +221,13 @@ def la_roots(n,alpha,mu=0):
     polynomial, L^(alpha)_n(x), and weights (w) to use in Gaussian quadrature over
     [0,inf] with weighting function exp(-x) x**alpha with alpha > -1.
     """
-    if not (alpha > -1):
+    if not all(alpha > -1):
         raise ValueError, "alpha > -1"
     assert(n>0), "n must be positive."
     (p,q) = (alpha,0.0)
     sbn_La = lambda k: -sqrt(k*(k + p))  # from recurrence relation
-    an_La = lambda k: 2*k + p + 1                 
-    mu0 = cephes.gamma(alpha+1)           # integral of weight over interval 
+    an_La = lambda k: 2*k + p + 1
+    mu0 = cephes.gamma(alpha+1)           # integral of weight over interval
     val = gen_roots_and_weights(n,an_La,sbn_La,mu0)
     if mu:
         return val + [mu0]
@@ -259,7 +239,7 @@ def genlaguerre(n,alpha,monic=0):
     L^(alpha)_n(x), orthogonal over [0,inf) with weighting function
     exp(-x) x**alpha with alpha > -1
     """
-    if (alpha <= -1):
+    if any(alpha <= -1):
         raise ValueError, "alpha must be > -1"
     assert(n>=0), "n must be nonnegative"
     if n==0: n1 = n+1
@@ -294,7 +274,7 @@ def laguerre(n,monic=0):
     hn = 1.0
     kn = (-1)**n / _gam(n+1)
     p = orthopoly1d(x,w,hn,kn,lambda x: exp(-x),(0,inf),monic)
-    return p 
+    return p
 
 
 # Hermite  1                         H_n(x)
@@ -307,8 +287,8 @@ def h_roots(n,mu=0):
     """
     assert(n>0), "n must be positive."
     sbn_H = lambda k: sqrt(k/2)  # from recurrence relation
-    an_H = lambda k: 0*k                    
-    mu0 = sqrt(pi)               # integral of weight over interval 
+    an_H = lambda k: 0*k
+    mu0 = sqrt(pi)               # integral of weight over interval
     val = gen_roots_and_weights(n,an_H,sbn_H,mu0)
     if mu:
         return val + [mu0]
@@ -329,7 +309,7 @@ def hermite(n,monic=0):
     kn = 2**n
     p = orthopoly1d(x,w,hn,kn,wfunc,(-inf,inf),monic)
     return p
-    
+
 # Hermite  2                         He_n(x)
 def he_roots(n,mu=0):
     """[x,w] = he_roots(n)
@@ -340,8 +320,8 @@ def he_roots(n,mu=0):
     """
     assert(n>0), "n must be positive."
     sbn_He = lambda k: sqrt(k)   # from recurrence relation
-    an_He  = lambda k: 0*k                
-    mu0 = sqrt(2*pi)             # integral of weight over interval 
+    an_He  = lambda k: 0*k
+    mu0 = sqrt(2*pi)             # integral of weight over interval
     val = gen_roots_and_weights(n,an_He,sbn_He,mu0)
     if mu:
         return val + [mu0]
@@ -385,7 +365,7 @@ def gegenbauer(n,alpha,monic=0):
         return base
     #  Abrahmowitz and Stegan 22.5.20
     factor = _gam(2*alpha+n)*_gam(alpha+0.5) / _gam(2*alpha) / _gam(alpha+0.5+n)
-    return factor * base
+    return base * factor
 
 # Chebyshev of the first kind: T_n(x)  = n! sqrt(pi) / _gam(n+1./2)* P^(-1/2,-1/2)_n(x)
 #  Computed anew.
@@ -421,7 +401,7 @@ def chebyt(n,monic=0):
     kn = 2**(n-1)
     p = orthopoly1d(x,w,hn,kn,wfunc,(-1,1),monic)
     return p
-    
+
     return jacobi(n,-0.5,-0.5,monic=monic)
 
 # Chebyshev of the second kind
@@ -443,7 +423,7 @@ def chebyu(n,monic=0):
     if monic:
         return base
     factor = sqrt(pi)/2.0*_gam(n+2) / _gam(n+1.5)
-    return factor * base
+    return base * factor
 
 # Chebyshev of the first kind        C_n(x)
 def c_roots(n,mu=0):
@@ -527,8 +507,8 @@ def sh_chebyt(n,monic=0):
         factor = 4**n / 2.0
     else:
         factor = 1.0
-    return factor * base
-    
+    return base * factor
+
 
 # Shifted Chebyshev of the second kind    U^*_n(x)
 def us_roots(n,mu=0):
@@ -547,9 +527,9 @@ def sh_chebyu(n,monic=0):
     base = sh_jacobi(n,2.0,1.5,monic=monic)
     if monic: return base
     factor = 4**n
-    return factor * base
+    return base * factor
 
-# Legendre 
+# Legendre
 def p_roots(n,mu=0):
     """[x,w] = p_roots(n)
 
@@ -594,4 +574,4 @@ def sh_legendre(n,monic=0):
     hn = 1.0/(2*n+1.0)
     kn = _gam(2*n+1)/_gam(n+1)**2
     p = orthopoly1d(x,w,hn,kn,wfunc,limits=(0,1),monic=monic)
-    return p 
+    return p
