@@ -8,20 +8,29 @@ import statlib
 import stats
 import distributions
 import inspect
-from scipy_base import isscalar, r_, log, sum, around, unique, asarray
-from scipy_base import zeros, arange, sort, amin, amax, any, where, \
+from numpy import isscalar, r_, log, sum, around, unique, asarray
+from numpy import zeros, arange, sort, amin, amax, any, where, \
      array, atleast_1d, sqrt, ceil, floor, array, poly1d, compress, not_equal, \
-     pi, exp, ravel
+     pi, exp, ravel, angle
 import scipy
+import numpy
 import types
 import scipy.optimize as optimize
 import futil
-import scipy_base as sb
+import numpy as sb
 
+__all__ = ['find_repeats',
+           'bayes_mvs', 'kstat', 'kstatvar', 'probplot', 'ppcc_max', 'ppcc_plot',
+           'boxcox_llf', 'boxcox', 'boxcox_normmax', 'boxcox_normplot',
+           'shapiro', 'anderson', 'ansari', 'bartlett', 'levene', 'binom_test',
+           'fligner', 'mood', 'oneway', 'wilcoxon',
+           'pdf_moments', 'pdf_fromgamma', 'pdfapprox',
+           'circmean', 'circvar', 'circstd',
+          ]
 
 def find_repeats(arr):
     """Find repeats in arr and return (repeats, repeat_count)
-    """    
+    """
     v1,v2, n = futil.dfreps(arr)
     return v1[:n],v2[:n]
 
@@ -30,7 +39,7 @@ def find_repeats(arr):
 ###  Bayesian confidence intervals for mean, variance, std
 ##########################################################
 
-##  Assumes all is known is that mean, and std (variance) exist
+##  Assumes all is known is that mean, and std (variance,axis=0) exist
 ##   and are the same for all the data.  Uses Jeffrey's prior
 ##
 ##  Returns alpha confidence interval for the mean, variance,
@@ -65,7 +74,7 @@ def bayes_mvs(data,alpha=0.90):
     #
     fac = n*C/2.0
     peak = 2/(n+1.)
-    a = (n-1)/2.0    
+    a = (n-1)/2.0
     F_peak = distributions.invgamma.cdf(peak,a)
     q1 = F_peak - alpha/2.0
     q2 = F_peak + alpha/2.0
@@ -89,9 +98,9 @@ def bayes_mvs(data,alpha=0.90):
         sta = fac*distributions.gengamma.ppf(q1,a,-2)
     stb = fac*distributions.gengamma.ppf(q2,a,-2)
     stp = peak*fac
-        
+
     return (mp,(ma,mb)),(vp,(va,vb)),(stp,(sta,stb))
-    
+
 
 
 ################################
@@ -133,7 +142,7 @@ def kstat(data,n=2):
     data = ravel(data)
     N = len(data)
     for k in range(1,n+1):
-        S[k] = sum(data**k)
+        S[k] = sum(data**k,axis=0)
     if n==1:
         return S[1]*1.0/N
     elif n==2:
@@ -172,7 +181,7 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
     slope (scale), intercept (loc), and correlation coefficient (r), of the
     best straight line through the points.  If fit==0, only (osm, osr) is
     returned.
-    
+
     sparams is a tuple of shape parameter arguments for the distribution.
     """
     N = len(x)
@@ -183,7 +192,7 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
     Ui[1:-1] = (i-0.3175)/(N+0.365)
     try:
         ppf_func = eval('distributions.%s.ppf'%dist)
-    except AttributError:
+    except AttributeError:
         raise dist, "is not a valid distribution with a ppf."
     if sparams is None:
         sparams = ()
@@ -191,6 +200,7 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
         sparams = (sparams,)
     if not isinstance(sparams,types.TupleType):
         sparams = tuple(sparams)
+    """
     res = inspect.getargspec(ppf_func)
     if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
             0.0==res[-1][-2] and 1.0==res[-1][-1]):
@@ -199,6 +209,7 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
     if (len(sparams) < len(res[0])-len(res[-1])-1) or \
        (len(sparams) > len(res[0])-3):
         raise ValueError, "Incorrect number of shape parameters."
+    """
     osm = ppf_func(Ui,*sparams)
     osr = sort(x)
     if fit or (plot is not None):
@@ -234,8 +245,9 @@ def ppcc_max(x, brack=(0.0,1.0), dist='tukeylambda'):
     """
     try:
         ppf_func = eval('distributions.%s.ppf'%dist)
-    except AttributError:
+    except AttributeError:
         raise dist, "is not a valid distribution with a ppf."
+    """
     res = inspect.getargspec(ppf_func)
     if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
             0.0==res[-1][-2] and 1.0==res[-1][-1]):
@@ -244,6 +256,7 @@ def ppcc_max(x, brack=(0.0,1.0), dist='tukeylambda'):
     if (1 < len(res[0])-len(res[-1])-1) or \
        (1 > len(res[0])-3):
         raise ValueError, "Must be a one-parameter family."
+    """
     N = len(x)
     # compute uniform median statistics
     Ui = zeros(N)*1.0
@@ -296,8 +309,8 @@ def boxcox_llf(lmb, data):
     N = len(data)
     y = boxcox(data,lmb)
     my = stats.mean(y)
-    f = (lmb-1)*sum(log(data))
-    f -= N/2.0*log(sum((y-my)**2.0/N))
+    f = (lmb-1)*sum(log(data),axis=0)
+    f -= N/2.0*log(sum((y-my)**2.0/N,axis=0))
     return f
 
 def _boxcox_conf_interval(x, lmax, alpha):
@@ -325,7 +338,7 @@ def _boxcox_conf_interval(x, lmax, alpha):
         raise RuntimeError, "Could not find endpoint."
     lmminus = optimize.brentq(rootfunc,newlm,lmax,args=(x,target))
     return lmminus,lmplus
-       
+
 def boxcox(x,lmbda=None,alpha=None):
     """Return a positive dataset tranformed by a Box-Cox power transformation.
 
@@ -335,7 +348,7 @@ def boxcox(x,lmbda=None,alpha=None):
     function and return it as the second output argument.
 
     If alpha is not None, return the 100(1-alpha)% confidence interval for
-    lambda as the third output argument. 
+    lambda as the third output argument.
     """
     if any(x < 0):
         raise ValueError, "Data must be positive."
@@ -354,7 +367,7 @@ def boxcox(x,lmbda=None,alpha=None):
     interval = _boxcox_conf_interval(x, lmax, alpha)
     return y, lmax, interval
 
-    
+
 def boxcox_normmax(x,brack=(-1.0,1.0)):
     N = len(x)
     # compute uniform median statistics
@@ -374,7 +387,7 @@ def boxcox_normmax(x,brack=(-1.0,1.0)):
         r, prob  = stats.pearsonr(xvals, yvals)
         return 1-r
     return optimize.brent(tempfunc, brack=brack, args=(xvals, x))
-        
+
 
 def boxcox_normplot(x,la,lb,plot=None,N=80):
     svals = r_[la:lb:complex(N)]
@@ -411,8 +424,8 @@ def shapiro(x,a=None,reta=0):
 
               if reta is nonzero then also return the computed "a" values
                  as the third output.  If these are known for a given size
-                 they can be given as input instead of computed internally. 
-    
+                 they can be given as input instead of computed internally.
+
     """
     N = len(x)
     if N < 3:
@@ -431,7 +444,7 @@ def shapiro(x,a=None,reta=0):
         print "p-value may not be accurate for N > 5000."
     if reta:
         return w, pw, a
-    else:        
+    else:
         return w, pw
 
 # Values from Stephens, M A, "EDF Statistics for Goodness of Fit and
@@ -443,7 +456,7 @@ _Avals_expon  = array([0.922, 1.078, 1.341, 1.606, 1.957])
 #             Biometrika, Vol. 64, Issue 3, Dec. 1977, pp 583-588.
 _Avals_gumbel = array([0.474, 0.637, 0.757, 0.877, 1.038])
 # From Stephens, M A, "Tests of Fit for the Logistic Distribution Based
-#             on the Empirical Distribution Function.", Biometrika, 
+#             on the Empirical Distribution Function.", Biometrika,
 #             Vol. 66, Issue 3, Dec. 1979, pp 591-595.
 _Avals_logistic = array([0.426, 0.563, 0.660, 0.769, 0.906, 1.010])
 def anderson(x,dist='norm'):
@@ -484,8 +497,8 @@ def anderson(x,dist='norm'):
             a,b = ab
             tmp = (xj-a)/b
             tmp2 = exp(tmp)
-            val = [sum(1.0/(1+tmp2))-0.5*N,
-                   sum(tmp*(1.0-tmp2)/(1+tmp2))+N]
+            val = [sum(1.0/(1+tmp2),axis=0)-0.5*N,
+                   sum(tmp*(1.0-tmp2)/(1+tmp2),axis=0)+N]
             return array(val)
         sol0=array([xbar,stats.std(x)])
         sol = optimize.fsolve(rootfunc,sol0,args=(x,N),xtol=1e-5)
@@ -497,17 +510,17 @@ def anderson(x,dist='norm'):
         def fixedsolve(th,xj,N):
             val = stats.sum(xj)*1.0/N
             tmp = exp(-xj/th)
-            term = sum(xj*tmp)
-            term /= sum(tmp)
+            term = sum(xj*tmp,axis=0)
+            term /= sum(tmp,axis=0)
             return val - term
         s = optimize.fixed_point(fixedsolve, 1.0, args=(x,N),xtol=1e-5)
-        xbar = -s*log(sum(exp(-x/s))*1.0/N)
+        xbar = -s*log(sum(exp(-x/s),axis=0)*1.0/N)
         w = (y-xbar)/s
         z = distributions.gumbel_l.cdf(w)
         sig = array([25,10,5,2.5,1])
         critical = around(_Avals_gumbel / (1.0 + 0.2/sqrt(N)),3)
     i = arange(1,N+1)
-    S = sum((2*i-1.0)/N*(log(z)+log(1-z[::-1])))
+    S = sum((2*i-1.0)/N*(log(z)+log(1-z[::-1])),axis=0)
     A2 = -N-S
     return A2, critical, sig
 
@@ -562,7 +575,7 @@ def ansari(x,y):
     xy = r_[x,y]  # combine
     rank = stats.rankdata(xy)
     symrank = amin(array((rank,N-rank+1)),0)
-    AB = sum(symrank[:n])
+    AB = sum(symrank[:n],axis=0)
     uxy = unique(xy)
     repeats = (len(uxy) != len(xy))
     exact = ((m<55) and (n<55) and not repeats)
@@ -571,21 +584,21 @@ def ansari(x,y):
     if exact:
         astart, a1, ifault = statlib.gscale(n,m)
         ind = AB-astart
-        total = sum(a1)
+        total = sum(a1,axis=0)
         if ind < len(a1)/2.0:
             cind = int(ceil(ind))
             if (ind == cind):
-                pval = 2.0*sum(a1[:cind+1])/total
+                pval = 2.0*sum(a1[:cind+1],axis=0)/total
             else:
-                pval = 2.0*sum(a1[:cind])/total
+                pval = 2.0*sum(a1[:cind],axis=0)/total
         else:
             find = int(floor(ind))
             if (ind == floor(ind)):
-                pval = 2.0*sum(a1[find:])/total
+                pval = 2.0*sum(a1[find:],axis=0)/total
             else:
-                pval = 2.0*sum(a1[find+1:])/total
+                pval = 2.0*sum(a1[find+1:],axis=0)/total
         return AB, min(1.0,pval)
-    
+
     # otherwise compute normal approximation
     if N % 2:  # N odd
         mnAB = n*(N+1.0)**2 / 4.0 / N
@@ -594,8 +607,8 @@ def ansari(x,y):
         mnAB = n*(N+2.0)/4.0
         varAB = m*n*(N+2)*(N-2.0)/48/(N-1.0)
     if repeats:   # adjust variance estimates
-        # compute sum(tj * rj**2)
-        fac = sum(symrank**2)
+        # compute sum(tj * rj**2,axis=0)
+        fac = sum(symrank**2,axis=0)
         if N % 2: # N odd
             varAB = m*n*(16*N*fac-(N+1)**4)/(16.0 * N**2 * (N-1))
         else:  # N even
@@ -608,13 +621,13 @@ def bartlett(*args):
     """Perform Bartlett test with the null hypothesis that all input samples
     have equal variances.
 
-    Inputs are sample vectors:  bartlett(x,y,z,...) 
+    Inputs are sample vectors:  bartlett(x,y,z,...)
 
     Outputs: (T, pval)
 
          T    -- the Test statistic
          pval -- significance level if null is rejected with this value of T
-                 (prob. that null is true but rejected with this p-value.)  
+                 (prob. that null is true but rejected with this p-value.)
 
     Sensitive to departures from normality.  The Levene test is
     an alternative that is less sensitive to departures from
@@ -625,7 +638,7 @@ def bartlett(*args):
       http://www.itl.nist.gov/div898/handbook/eda/section3/eda357.htm
 
       Snedecor, George W. and Cochran, William G. (1989), Statistical
-        Methods, Eighth Edition, Iowa State University Press.     
+        Methods, Eighth Edition, Iowa State University Press.
     """
     k = len(args)
     if k < 2:
@@ -635,10 +648,10 @@ def bartlett(*args):
     for j in range(k):
         Ni[j] = len(args[j])
         ssq[j] = stats.var(args[j])
-    Ntot = sum(Ni)
-    spsq = sum((Ni-1)*ssq)/(1.0*(Ntot-k))
-    numer = (Ntot*1.0-k)*log(spsq) - sum((Ni-1.0)*log(ssq))
-    denom = 1.0 + (1.0/(3*(k-1)))*((sum(1.0/(Ni-1.0)))-1.0/(Ntot-k))
+    Ntot = sum(Ni,axis=0)
+    spsq = sum((Ni-1)*ssq,axis=0)/(1.0*(Ntot-k))
+    numer = (Ntot*1.0-k)*log(spsq) - sum((Ni-1.0)*log(ssq),axis=0)
+    denom = 1.0 + (1.0/(3*(k-1)))*((sum(1.0/(Ni-1.0),axis=0))-1.0/(Ntot-k))
     T = numer / denom
     pval = distributions.chi2.sf(T,k-1) # 1 - cdf
     return T, pval
@@ -661,7 +674,7 @@ def levene(*args,**kwds):
 
          W    -- the Test statistic
          pval -- significance level if null is rejected with this value of W
-                 (prob. that null is true but rejected with this p-value.)  
+                 (prob. that null is true but rejected with this p-value.)
 
     References:
 
@@ -669,9 +682,9 @@ def levene(*args,**kwds):
 
        Levene, H. (1960). In Contributions to Probability and Statistics:
          Essays in Honor of Harold Hotelling, I. Olkin et al. eds.,
-         Stanford University Press, pp. 278-292. 
+         Stanford University Press, pp. 278-292.
        Brown, M. B. and Forsythe, A. B. (1974), Journal of the American
-         Statistical Association, 69, 364-367         
+         Statistical Association, 69, 364-367
     """
     k = len(args)
     if k < 2:
@@ -693,8 +706,8 @@ def levene(*args,**kwds):
         func = stats.trim_mean
     for j in range(k):
         Ni[j] = len(args[j])
-        Yci[j] = func(args[j])    
-    Ntot = sum(Ni)
+        Yci[j] = func(args[j])
+    Ntot = sum(Ni,axis=0)
 
     # compute Zij's
     Zij = [None]*k
@@ -708,12 +721,12 @@ def levene(*args,**kwds):
         Zbar += Zbari[i]*Ni[i]
     Zbar /= Ntot
 
-    numer = (Ntot-k)*sum(Ni*(Zbari-Zbar)**2)
+    numer = (Ntot-k)*sum(Ni*(Zbari-Zbar)**2,axis=0)
 
     # compute denom_variance
     dvar = 0.0
     for i in range(k):
-        dvar += sum((Zij[i]-Zbari[i])**2)
+        dvar += sum((Zij[i]-Zbari[i])**2,axis=0)
 
     denom = (k-1.0)*dvar
 
@@ -735,7 +748,7 @@ def binom_test(x,n=None,p=0.5):
     Returns pval -- Probability that null test is rejected for this set
                     of x and n even though it is true.
     """
-    x = atleast_1d(x)
+    x = atleast_1d(x).astype(numpy.integer)
     if len(x) == 2:
         n = x[1]+x[0]
         x = x[0]
@@ -743,6 +756,7 @@ def binom_test(x,n=None,p=0.5):
         x = x[0]
         if n is None or n < x:
             raise ValueError, "n must be >= x"
+        n = numpy.int_(n)
     else:
         raise ValueError, "Incorrect length for x."
 
@@ -751,13 +765,13 @@ def binom_test(x,n=None,p=0.5):
 
     d = distributions.binom.pmf(x,n,p)
     rerr = 1+1e-7
-    if (x*1.0/n < p):
+    if (x < p*n):
         i = arange(x+1,n+1)
-        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr)
+        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr,axis=0)
         pval = distributions.binom.cdf(x,n,p) + distributions.binom.sf(n-y,n,p)
     else:
         i = arange(0,x)
-        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr)
+        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr,axis=0)
         pval = distributions.binom.cdf(y-1,n,p) + distributions.binom.sf(x-1,n,p)
 
     return min(1.0,pval)
@@ -785,7 +799,7 @@ def fligner(*args,**kwds):
 
          Xsq  -- the Test statistic
          pval -- significance level if null is rejected with this value of X
-                 (prob. that null is true but rejected with this p-value.)  
+                 (prob. that null is true but rejected with this p-value.)
 
     References:
 
@@ -814,7 +828,7 @@ def fligner(*args,**kwds):
 
     Ni = asarray([len(args[j]) for j in range(k)])
     Yci = asarray([func(args[j]) for j in range(k)])
-    Ntot = sum(Ni)
+    Ntot = sum(Ni,axis=0)
     # compute Zij's
     Zij = [abs(asarray(args[i])-Yci[i]) for i in range(k)]
     allZij = []
@@ -830,7 +844,7 @@ def fligner(*args,**kwds):
     anbar = stats.mean(a)
     varsq = stats.var(a)
 
-    Xsq = sum(Ni*(asarray(Aibar)-anbar)**2.0)/varsq
+    Xsq = sum(Ni*(asarray(Aibar)-anbar)**2.0,axis=0)/varsq
 
     pval = distributions.chi2.sf(Xsq,k-1) # 1 - cdf
     return Xsq, pval
@@ -855,7 +869,7 @@ def mood(x,y):
         raise ValueError, "Not enough observations."
     ranks = stats.rankdata(xy)
     Ri = ranks[:n]
-    M = sum((Ri - (N+1.0)/2)**2)
+    M = sum((Ri - (N+1.0)/2)**2,axis=0)
     # Approx stat.
     mnM = n*(N*N-1.0)/12
     varM = m*n*(N+1.0)*(N+2)*(N-2)/180
@@ -889,16 +903,16 @@ def oneway(*args,**kwds):
     Mi = array([stats.mean(args[i]) for i in range(k)])
     Vi = array([stats.var(args[i]) for i in range(k)])
     Wi = Ni / Vi
-    swi = sum(Wi)
-    N = sum(Ni)
-    my = sum(Mi*Ni)*1.0/N
-    tmp = sum((1-Wi/swi)**2 / (Ni-1.0))/(k*k-1.0)
+    swi = sum(Wi,axis=0)
+    N = sum(Ni,axis=0)
+    my = sum(Mi*Ni,axis=0)*1.0/N
+    tmp = sum((1-Wi/swi)**2 / (Ni-1.0),axis=0)/(k*k-1.0)
     if evar:
-        F = ((sum(Ni*(Mi-my)**2) / (k-1.0)) / (sum((Ni-1.0)*Vi) / (N-k)))
+        F = ((sum(Ni*(Mi-my)**2,axis=0) / (k-1.0)) / (sum((Ni-1.0)*Vi,axis=0) / (N-k)))
         pval = distributions.f.sf(F,k-1,N-k)  # 1-cdf
     else:
-        m = sum(Wi*Mi)*1.0/swi
-        F = sum(Wi*(Mi-m)**2) / ((k-1.0)*(1+2*(k-2)*tmp))
+        m = sum(Wi*Mi,axis=0)*1.0/swi
+        F = sum(Wi*(Mi-m)**2,axis=0) / ((k-1.0)*(1+2*(k-2)*tmp))
         pval = distributions.f.sf(F,k-1.0,1.0/(3*tmp))
 
     return F, pval
@@ -918,13 +932,13 @@ Returns: t-statistic, two-tailed p-value
         if len(x) <> len(y):
             raise ValueError, 'Unequal N in wilcoxon.  Aborting.'
         d = x-y
-    d = compress(not_equal(d,0),d) # Keep all non-zero differences
+    d = compress(not_equal(d,0),d,axis=-1) # Keep all non-zero differences
     count = len(d)
     if (count < 10):
         print "Warning: sample size too small for normal approximation."
     r = stats.rankdata(abs(d))
-    r_plus = sum((d > 0)*r)
-    r_minus = sum((d < 0)*r)
+    r_plus = sum((d > 0)*r,axis=0)
+    r_minus = sum((d < 0)*r,axis=0)
     T = min(r_plus, r_minus)
     mn = count*(count+1.0)*0.25
     se = math.sqrt(count*(count+1)*(2*count+1.0)/24)
@@ -975,14 +989,14 @@ def pdf_moments(cnt):
             else:
                 momdiff = cnt[m-1] - sig*sig*scipy.factorial2(m-1)
             Ck += Dvals[k][m] / sig**m * momdiff
-        # Add to totp 
-        totp = totp +  Ck*Dvals[k]        
-        
+        # Add to totp
+        totp = totp +  Ck*Dvals[k]
+
     def thisfunc(x):
         xn = (x-mu)/sig
         return totp(xn)*exp(-xn*xn/2.0)/sqrt(2*pi)/sig
     return thisfunc
-           
+
 
 def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
     if g4 is None:
@@ -1051,9 +1065,9 @@ def circstd(samples, high=2*pi, low=0):
     res = stats.mean(exp(1j*ang))
     V = 1-abs(res)
     return ((high-low)/2.0/pi) * sqrt(V)
-    
-    
-        
+
+
+
 #Tests to include (from R) -- some of these already in stats.
 ########
 #X Ansari-Bradley
