@@ -6,18 +6,15 @@
 
 """
 
-import unittest
 import sys
-from scipy_test.testing import *
-set_package_path()
-#XXX: fix scipy.stats to stats
+from numpy.testing import *
+from numpy import *
+import numpy
 import scipy
-import scipy_base.fastumath as math
-del sys.path[0]
-
-import Numeric
-N = Numeric
 
+set_package_path()
+import stats
+restore_path()
 
 """ Numbers in docstrings begining with 'W' refer to the section numbers
     and headings found in the STATISTICS QUIZ of Leland Wilkinson.  These are
@@ -34,14 +31,14 @@ N = Numeric
 ##  For completeness, I should write the relavant tests and count them as failures
 ##  Somewhat acceptable, since this is still beta software.  It would count as a
 ##  good target for 1.0 status
-X = N.array([1,2,3,4,5,6,7,8,9],N.Float)
-ZERO= N.array([0,0,0,0,0,0,0,0,0], N.Float)
-#MISS=N.array([.,.,.,.,.,.,.,.,.], N.Float)
-BIG=N.array([99999991,99999992,99999993,99999994,99999995,99999996,99999997,99999998,99999999],N.Float)
-LITTLE=N.array([0.99999991,0.99999992,0.99999993,0.99999994,0.99999995,0.99999996,0.99999997,0.99999998,0.99999999],N.Float)
-HUGE=N.array([1e+12,2e+12,3e+12,4e+12,5e+12,6e+12,7e+12,8e+12,9e+12],N.Float)
-TINY=N.array([1e-12,2e-12,3e-12,4e-12,5e-12,6e-12,7e-12,8e-12,9e-12],N.Float)
-ROUND=N.array([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5],N.Float)
+X = array([1,2,3,4,5,6,7,8,9],float)
+ZERO= array([0,0,0,0,0,0,0,0,0], float)
+#MISS=array([.,.,.,.,.,.,.,.,.], float)
+BIG=array([99999991,99999992,99999993,99999994,99999995,99999996,99999997,99999998,99999999],float)
+LITTLE=array([0.99999991,0.99999992,0.99999993,0.99999994,0.99999995,0.99999996,0.99999997,0.99999998,0.99999999],float)
+HUGE=array([1e+12,2e+12,3e+12,4e+12,5e+12,6e+12,7e+12,8e+12,9e+12],float)
+TINY=array([1e-12,2e-12,3e-12,4e-12,5e-12,6e-12,7e-12,8e-12,9e-12],float)
+ROUND=array([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5],float)
 X2 = X * X
 X3 = X2 * X
 X4 = X3 * X
@@ -51,7 +48,7 @@ X7 = X6 * X
 X8 = X7 * X
 X9 = X8 * X
 
-class test_round(unittest.TestCase):
+class test_round(ScipyTestCase):
     """ W.II. ROUND
 
         You should get the numbers 1 to 9.  Many language compilers,
@@ -69,40 +66,39 @@ class test_round(unittest.TestCase):
         packages.  Since the square of a square root should return the same
         number, and the exponential of a log should return the same number,
         we should get back a 2 from this function of functions.  By taking
-        the integer result and subtracting from 2, we are exposing the 
-        roundoff errors.  These simple functions are at the heart of  
+        the integer result and subtracting from 2, we are exposing the
+        roundoff errors.  These simple functions are at the heart of
         statistical calculations.
     """
 
     def check_rounding0(self):
         """ W.II.A.0. Print ROUND with only one digit.
-        
+
             You should get the numbers 1 to 9.  Many language compilers,
             such as Turbo Pascal and Lattice C, fail this test (they round
             numbers inconsistently). Needless to say, statical packages
             written in these languages may fail the test as well.
         """
- 
         for i in range(0,9):
-            y = scipy.round(ROUND[i])
+            y = round(ROUND[i])
             assert_equal(y,i+1)
-        
+
     def check_rounding1(self):
         """ W.II.A.1. Y = INT(2.6*7 -0.2) (Y should be 18)"""
         y = int(2.6*7 -0.2)
         assert_equal(y, 18)
-        
+
     def check_rounding2(self):
         """ W.II.A.2. Y = 2-INT(EXP(LOG(SQR(2)*SQR(2))))   (Y should be 0)"""
-        y=2-int(scipy.exp(scipy.log(scipy.sqrt(2.)*scipy.sqrt(2.))))
+        y=2-int(numpy.exp(numpy.log(numpy.sqrt(2.)*numpy.sqrt(2.))))
         assert_equal(y,0)
-        
+
     def check_rounding3(self):
         """ W.II.A.3. Y = INT(3-EXP(LOG(SQR(2)*SQR(2))))    (Y should be 1)"""
-        y=(int(round((3-scipy.exp(scipy.log(scipy.sqrt(2.0)*scipy.sqrt(2.0)))))))
+        y=(int(round((3-numpy.exp(numpy.log(numpy.sqrt(2.0)*numpy.sqrt(2.0)))))))
         assert_equal(y,1)
 
-class test_basicstats(unittest.TestCase):
+class test_basicstats(ScipyTestCase):
     """ W.II.C. Compute basic statistic on all the variables.
 
         The means should be the fifth value of all the variables (case FIVE).
@@ -118,6 +114,18 @@ class test_basicstats(unittest.TestCase):
         y = scipy.stats.std(X)
         assert_almost_equal(y,2.738612788)
 
+    def check_tmeanX(self):
+        y = scipy.stats.tmean(X, (2, 8), (True, True))
+        assert_almost_equal(y, 5.0)
+
+    def check_tvarX(self):
+        y = scipy.stats.tvar(X, (2, 8), (True, True))
+        assert_almost_equal(y, 4.6666666666666661)
+
+    def check_tstdX(self):
+        y = scipy.stats.tstd(X, (2, 8), (True, True))
+        assert_almost_equal(y, 2.1602468994692865)
+
     def check_meanZERO(self):
         y = scipy.stats.mean(ZERO)
         assert_almost_equal(y,0.0)
@@ -174,8 +182,8 @@ class test_basicstats(unittest.TestCase):
     def check_stdROUND(self):
         y = scipy.stats.std(ROUND)
         assert_approx_equal(y,2.738612788)
-            
-class test_corr(unittest.TestCase):
+
+class test_corr(ScipyTestCase):
     """ W.II.D. Compute a correlation matrix on all the variables.
 
         All the correlations, except for ZERO and MISS, shoud be exactly 1.
@@ -183,186 +191,186 @@ class test_corr(unittest.TestCase):
         other variables.  The same should go for SPEARMAN corelations, if
         your program has them.
     """
-    def check_pXX(self):    
+    def check_pXX(self):
         y = scipy.stats.pearsonr(X,X)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pXBIG(self):    
+    def check_pXBIG(self):
         y = scipy.stats.pearsonr(X,BIG)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pXLITTLE(self):    
+    def check_pXLITTLE(self):
         y = scipy.stats.pearsonr(X,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pXHUGE(self):    
+    def check_pXHUGE(self):
         y = scipy.stats.pearsonr(X,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pXTINY(self):    
+    def check_pXTINY(self):
         y = scipy.stats.pearsonr(X,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pXROUND(self):    
+    def check_pXROUND(self):
         y = scipy.stats.pearsonr(X,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pBIGBIG(self):    
+    def check_pBIGBIG(self):
         y = scipy.stats.pearsonr(BIG,BIG)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pBIGLITTLE(self):    
+    def check_pBIGLITTLE(self):
         y = scipy.stats.pearsonr(BIG,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pBIGHUGE(self):    
+    def check_pBIGHUGE(self):
         y = scipy.stats.pearsonr(BIG,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pBIGTINY(self):    
+    def check_pBIGTINY(self):
         y = scipy.stats.pearsonr(BIG,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pBIGROUND(self):    
+    def check_pBIGROUND(self):
         y = scipy.stats.pearsonr(BIG,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pLITTLELITTLE(self):    
+    def check_pLITTLELITTLE(self):
         y = scipy.stats.pearsonr(LITTLE,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pLITTLEHUGE(self):    
+    def check_pLITTLEHUGE(self):
         y = scipy.stats.pearsonr(LITTLE,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pLITTLETINY(self):    
+    def check_pLITTLETINY(self):
         y = scipy.stats.pearsonr(LITTLE,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pLITTLEROUND(self):    
+    def check_pLITTLEROUND(self):
         y = scipy.stats.pearsonr(LITTLE,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pHUGEHUGE(self):    
+    def check_pHUGEHUGE(self):
         y = scipy.stats.pearsonr(HUGE,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pHUGETINY(self):    
+    def check_pHUGETINY(self):
         y = scipy.stats.pearsonr(HUGE,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pHUGEROUND(self):    
+    def check_pHUGEROUND(self):
         y = scipy.stats.pearsonr(HUGE,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pTINYTINY(self):    
+    def check_pTINYTINY(self):
         y = scipy.stats.pearsonr(TINY,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pTINYROUND(self):    
+    def check_pTINYROUND(self):
         y = scipy.stats.pearsonr(TINY,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_pROUNDROUND(self):    
+    def check_pROUNDROUND(self):
         y = scipy.stats.pearsonr(ROUND,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXX(self):    
+    def check_sXX(self):
         y = scipy.stats.spearmanr(X,X)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXBIG(self):    
+    def check_sXBIG(self):
         y = scipy.stats.spearmanr(X,BIG)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXLITTLE(self):    
+    def check_sXLITTLE(self):
         y = scipy.stats.spearmanr(X,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXHUGE(self):    
+    def check_sXHUGE(self):
         y = scipy.stats.spearmanr(X,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXTINY(self):    
+    def check_sXTINY(self):
         y = scipy.stats.spearmanr(X,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sXROUND(self):    
+    def check_sXROUND(self):
         y = scipy.stats.spearmanr(X,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sBIGBIG(self):    
+    def check_sBIGBIG(self):
         y = scipy.stats.spearmanr(BIG,BIG)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sBIGLITTLE(self):    
+    def check_sBIGLITTLE(self):
         y = scipy.stats.spearmanr(BIG,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sBIGHUGE(self):    
+    def check_sBIGHUGE(self):
         y = scipy.stats.spearmanr(BIG,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sBIGTINY(self):    
+    def check_sBIGTINY(self):
         y = scipy.stats.spearmanr(BIG,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sBIGROUND(self):    
+    def check_sBIGROUND(self):
         y = scipy.stats.spearmanr(BIG,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sLITTLELITTLE(self):    
+    def check_sLITTLELITTLE(self):
         y = scipy.stats.spearmanr(LITTLE,LITTLE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sLITTLEHUGE(self):    
+    def check_sLITTLEHUGE(self):
         y = scipy.stats.spearmanr(LITTLE,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sLITTLETINY(self):    
+    def check_sLITTLETINY(self):
         y = scipy.stats.spearmanr(LITTLE,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sLITTLEROUND(self):    
+    def check_sLITTLEROUND(self):
         y = scipy.stats.spearmanr(LITTLE,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sHUGEHUGE(self):    
+    def check_sHUGEHUGE(self):
         y = scipy.stats.spearmanr(HUGE,HUGE)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sHUGETINY(self):    
+    def check_sHUGETINY(self):
         y = scipy.stats.spearmanr(HUGE,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sHUGEROUND(self):    
+    def check_sHUGEROUND(self):
         y = scipy.stats.spearmanr(HUGE,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sTINYTINY(self):    
+    def check_sTINYTINY(self):
         y = scipy.stats.spearmanr(TINY,TINY)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sTINYROUND(self):    
+    def check_sTINYROUND(self):
         y = scipy.stats.spearmanr(TINY,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
-    def check_sROUNDROUND(self):    
+    def check_sROUNDROUND(self):
         y = scipy.stats.spearmanr(ROUND,ROUND)
         r = y[0]
         assert_approx_equal(r,1.0)
 
-##    W.II.E.  Tabulate X against X, using BIG as a case weight.  The values 
+##    W.II.E.  Tabulate X against X, using BIG as a case weight.  The values
 ##    should appear on the diagonal and the total should be 899999955.
-##    If the table cannot hold these values, forget about working with 
+##    If the table cannot hold these values, forget about working with
 ##    census data.  You can also tabulate HUGE against TINY.  There is no
-##    reason a tabulation program should not be able to digtinguish 
+##    reason a tabulation program should not be able to digtinguish
 ##    different values regardless of their magnitude.
 
 ### I need to figure out how to do this one.
 
 
-class test_regression(unittest.TestCase):
+class test_regression(ScipyTestCase):
     def check_linregressBIGX(self):
         """ W.II.F.  Regress BIG on X.
 
@@ -373,7 +381,7 @@ class test_regression(unittest.TestCase):
         r=y[2]
         assert_almost_equal(intercept,99999990)
         assert_almost_equal(r,1.0)
-        
+
 ##     W.IV.A. Take the NASTY dataset above.  Use the variable X as a
 ##     basis for computing polynomials.  Namely, compute X1=X, X2=X*X,
 ##     X3=X*X*X, and so on up to 9 products.  Use the algebraic
@@ -431,7 +439,7 @@ anovadata = ([[1,'A1','B1',2],
               [12,'A1','B2',4],
               [13,'A1','B2',5]])
 
-##class test_anova(unittest.TestCase):
+##class test_anova(ScipyTestCase):
 ##    """ W.V.A.  Simple contrasts.
 ##
 ##        The following data contain an unbalanced design with a significant
@@ -439,7 +447,7 @@ anovadata = ([[1,'A1','B1',2],
 ##        is not significant, but this test is not particularlrly meaningful
 ##        because of the interaction.  Test, therefore, the simple constrast
 ##        between A1 and A2 within B1.  Then test A1 vs. A2 within B2.  Both
-##        tests should use the same residual error term (separate t-tests are 
+##        tests should use the same residual error term (separate t-tests are
 ##        unacceptable).  Several widely used mainframe programs fail this
 ##        test.  Unless the program can constrast any terms in a model (not
 ##        just main effects), it cannot handle this frequently encountered
@@ -495,22 +503,10 @@ def compare_results(res,desired):
         assert_array_equal(res[i],desired[i])
 
 
-from Numeric import *
-import Numeric
-from scipy_base.fastumath import *
-import unittest
-from scipy_test.testing import assert_array_equal, assert_equal
-from scipy_test.testing import assert_almost_equal
-from scipy_test.testing import assert_array_almost_equal, rand
-from scipy_test.testing import ScipyTestCase as TestCase
-import scipy
-from scipy import stats
-
-
 ##################################################
 ### Test for sum
 
-class test_gmean(TestCase):
+class test_gmean(ScipyTestCase):
 
     def check_1D_list(self):
         a = (1,2,3,4)
@@ -518,82 +514,78 @@ class test_gmean(TestCase):
         desired = power(1*2*3*4,1./4.)
         assert_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.gmean(a,axis=-1)
+        desired1 = stats.gmean(a,axis=-1)
         assert_almost_equal(desired1,actual,decimal=14)
     def check_1D_array(self):
-        a = array((1,2,3,4),Float32)
+        a = array((1,2,3,4), float32)
         actual= stats.gmean(a)
         desired = power(1*2*3*4,1./4.)
-        assert_almost_equal(desired,actual,decimal=14)
+        assert_almost_equal(desired,actual,decimal=7)
 
-        desired1 = scipy.stats.stats.gmean(a,axis=-1)
-        assert_almost_equal(desired1,actual,decimal=14)
+        desired1 = stats.gmean(a,axis=-1)
+        assert_almost_equal(desired1,actual,decimal=7)
 
     def check_2D_array_default(self):
         a = array(((1,2,3,4),
                    (1,2,3,4),
                    (1,2,3,4)))
         actual= stats.gmean(a)
+        desired = array((1,2,3,4))
+        assert_array_almost_equal(desired,actual,decimal=14)
+
+        desired1 = stats.gmean(a,axis=0)
+        assert_array_almost_equal(desired1,actual,decimal=14)
+
+    def check_2D_array_dim1(self):
+        a = array(((1,2,3,4),
+                   (1,2,3,4),
+                   (1,2,3,4)))
+        actual= stats.gmean(a, axis=1)
         v = power(1*2*3*4,1./4.)
         desired = array((v,v,v))
         assert_array_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.gmean(a,axis=-1)
-        assert_array_almost_equal(desired1,actual,decimal=14)
-
-    def check_2D_array_dim0(self):
-        a = array(((1,2,3,4),
-                   (1,2,3,4),
-                   (1,2,3,4)))
-        actual= stats.gmean(a,axis=0)
-        desired = array((1,2,3,4))
-        assert_array_almost_equal(desired,actual,decimal=14)
-
-        desired1 = scipy.stats.stats.gmean(a,axis=0)
-        assert_array_almost_equal(desired1,actual,decimal=14)
-
-class test_hmean(TestCase):
+class test_hmean(ScipyTestCase):
     def check_1D_list(self):
         a = (1,2,3,4)
         actual= stats.hmean(a)
         desired =  4. / (1./1 + 1./2 + 1./3 + 1./4)
         assert_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.hmean(array(a),axis=-1)
+        desired1 = stats.hmean(array(a),axis=-1)
         assert_almost_equal(desired1,actual,decimal=14)
     def check_1D_array(self):
-        a = array((1,2,3,4),Float64)
+        a = array((1,2,3,4), float64)
         actual= stats.hmean(a)
         desired =  4. / (1./1 + 1./2 + 1./3 + 1./4)
         assert_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.hmean(a,axis=-1)
+        desired1 = stats.hmean(a,axis=-1)
         assert_almost_equal(desired1,actual,decimal=14)
 
     def check_2D_array_default(self):
         a = array(((1,2,3,4),
                    (1,2,3,4),
                    (1,2,3,4)))
-        actual= stats.hmean(a)
-        v = 4. / (1./1 + 1./2 + 1./3 + 1./4)
-        desired = array((v,v,v))      
+        actual = stats.hmean(a)
+        desired = array((1.,2.,3.,4.))
         assert_array_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.hmean(a,axis=-1)
-        assert_array_almost_equal(desired1,actual,decimal=14)
+        actual1 = stats.hmean(a,axis=0)
+        assert_array_almost_equal(desired,actual1,decimal=14)
 
-    def check_2D_array_dim0(self):
+    def check_2D_array_dim1(self):
         a = array(((1,2,3,4),
                    (1,2,3,4),
                    (1,2,3,4)))
-        actual= stats.hmean(a,axis=0)
-        desired = array((1.,2.,3.,4.))        
-        assert_array_almost_equal(desired,actual,decimal=14)
 
-        desired1 = scipy.stats.stats.hmean(a,axis=0)
-        assert_array_almost_equal(desired1,actual,decimal=14)
+        v = 4. / (1./1 + 1./2 + 1./3 + 1./4)
+        desired1 = array((v,v,v))
+        actual1 = stats.hmean(a, axis=1)
+        assert_array_almost_equal(desired1,actual1,decimal=14)
 
-class test_mean(TestCase):
+
+class test_mean(ScipyTestCase):
     def check_basic(self):
         a = [3,4,5,10,-3,-5,6]
         af = [3.,4,5,10,-3,-5,-6]
@@ -602,7 +594,7 @@ class test_mean(TestCase):
         mn1 = 0.0
         for el in a:
             mn1 += el / float(Na)
-        assert_almost_equal(stats.mean(a),mn1,11) 
+        assert_almost_equal(stats.mean(a),mn1,11)
         mn2 = 0.0
         for el in af:
             mn2 += el / float(Naf)
@@ -617,11 +609,11 @@ class test_mean(TestCase):
         mn1 = zeros(N2,'d')
         for k in range(N1):
             mn1 += A[k,:] / N1
-        Numeric.allclose(stats.mean(a),mn1,rtol=1e-13,atol=1e-13)
+        allclose(stats.mean(a),mn1,rtol=1e-13,atol=1e-13)
         mn2 = zeros(N1,'d')
         for k in range(N2):
             mn2 += A[:,k] / N2
-        Numeric.allclose(stats.mean(a,axis=0),mn2,rtol=1e-13,atol=1e-13)
+        allclose(stats.mean(a,axis=0),mn2,rtol=1e-13,atol=1e-13)
 
     def check_ravel(self):
         a = rand(5,3,5)
@@ -630,23 +622,38 @@ class test_mean(TestCase):
             A += val
         assert_almost_equal(stats.mean(a,axis=None),A/(5*3.0*5))
 
-class test_median(TestCase):
+class test_median(ScipyTestCase):
     def check_basic(self):
         a1 = [3,4,5,10,-3,-5,6]
         a2 = [3,-6,-2,8,7,4,2,1]
         a3 = [3.,4,5,10,-3,-5,-6,7.0]
         assert_equal(stats.median(a1),4)
         assert_equal(stats.median(a2),2.5)
-        assert_equal(stats.median(a3),3.5)        
+        assert_equal(stats.median(a3),3.5)
 
-class test_std(TestCase):
+class test_percentile(ScipyTestCase):
+    def setUp(self):
+        self.a1 = [3,4,5,10,-3,-5,6]
+        self.a2 = [3,-6,-2,8,7,4,2,1]
+        self.a3 = [3.,4,5,10,-3,-5,-6,7.0]
+        
+    def check_median(self):
+        assert_equal(stats.median(self.a1), 4)
+        assert_equal(stats.median(self.a2), 2.5)
+        assert_equal(stats.median(self.a3), 3.5)
+    
+    def check_percentile(self):
+        x = arange(8) * 0.5
+        assert_equal(stats.scoreatpercentile(x, 0), 0.)
+        assert_equal(stats.scoreatpercentile(x, 100), 3.5)
+        assert_equal(stats.scoreatpercentile(x, 50), 1.75)
+        
+class test_std(ScipyTestCase):
     def check_basic(self):
         a = [3,4,5,10,-3,-5,6]
-        af = [3.,4,5,10,-3,-5,-6]
-        Na = len(a)
-        Naf = len(af)
+        b = [3,4,5,10,-3,-5,-6]
         assert_almost_equal(stats.std(a),5.2098807225172772,11)
-        assert_almost_equal(stats.std(af),5.9281411203561225,11)
+        assert_almost_equal(stats.std(b),5.9281411203561225,11)
 
     def check_2d(self):
         a = [[1.0, 2.0, 3.0],
@@ -655,25 +662,26 @@ class test_std(TestCase):
         b1 = array((3.7859388972001824, 5.2915026221291814,
                     2.0816659994661335))
         b2 = array((1.0,2.0,2.64575131106))
-        assert_array_almost_equal(stats.std(a),b2,11)
+        assert_array_almost_equal(stats.std(a),b1,11)
         assert_array_almost_equal(stats.std(a,axis=0),b1,11)
+        assert_array_almost_equal(stats.std(a,axis=1),b2,11)
 
 
-class test_cmedian(TestCase):
+class test_cmedian(ScipyTestCase):
     def check_basic(self):
         data = [1,2,3,1,5,3,6,4,3,2,4,3,5,2.0]
-        assert_almost_equal(stats.cmedian(data,5),2.7)
-        assert_almost_equal(stats.cmedian(data,3),3.1296296296296298)
-        assert_almost_equal(stats.cmedian(data),2.999665)
+        assert_almost_equal(stats.cmedian(data,5),3.2916666666666665)
+        assert_almost_equal(stats.cmedian(data,3),3.083333333333333)
+        assert_almost_equal(stats.cmedian(data),3.0020020020020022)
 
-class test_median(TestCase):
+class test_median(ScipyTestCase):
     def check_basic(self):
         data1 = [1,3,5,2,3,1,19,-10,2,4.0]
         data2 = [3,5,1,10,23,-10,3,-2,6,8,15]
         assert_almost_equal(stats.median(data1),2.5)
         assert_almost_equal(stats.median(data2),5)
 
-class test_mode(TestCase):
+class test_mode(ScipyTestCase):
     def check_basic(self):
         data1 = [3,5,1,10,23,3,2,6,8,6,10,6]
         vals = stats.mode(data1)
@@ -681,7 +689,7 @@ class test_mode(TestCase):
         assert_almost_equal(vals[1][0],3)
 
 
-class test_variability(TestCase):
+class test_variability(ScipyTestCase):
     """  Comparison numbers are found using R v.1.5.1
          note that length(testcase) = 4
     """
@@ -707,7 +715,7 @@ class test_variability(TestCase):
         #assert_approx_equal(y,5.4081)
         y = scipy.stats.samplevar(self.testcase)
         assert_approx_equal(y,1.25)
-        
+
     def check_samplestd(self):
         #y = scipy.stats.samplestd(self.shoes[0])
         #assert_approx_equal(y,2.325532197)
@@ -717,7 +725,7 @@ class test_variability(TestCase):
     def check_signaltonoise(self):
         """
         this is not in R, so used
-        mean(testcase)/(sqrt(var(testcase)*3/4)) """
+        mean(testcase,axis=0)/(sqrt(var(testcase)*3/4)) """
         #y = scipy.stats.signaltonoise(self.shoes[0])
         #assert_approx_equal(y,4.5709967)
         y = scipy.stats.signaltonoise(self.testcase)
@@ -741,11 +749,11 @@ class test_variability(TestCase):
         #assert_approx_equal(y,0.775177399)
         y = scipy.stats.sem(self.testcase)
         assert_approx_equal(y,0.6454972244)
-        
+
     def check_z(self):
-        """ 
+        """
         not in R, so used
-        (10-mean(testcase))/sqrt(var(testcase)*3/4)
+        (10-mean(testcase,axis=0))/sqrt(var(testcase)*3/4)
         """
         y = scipy.stats.z(self.testcase,scipy.stats.mean(self.testcase))
         assert_almost_equal(y,0.0)
@@ -753,15 +761,15 @@ class test_variability(TestCase):
     def check_zs(self):
         """
         not in R, so tested by using
-        (testcase[i]-mean(testcase))/sqrt(var(testcase)*3/4)
+        (testcase[i]-mean(testcase,axis=0))/sqrt(var(testcase)*3/4)
         """
         y = scipy.stats.zs(self.testcase)
         desired = ([-1.3416407864999, -0.44721359549996 , 0.44721359549996 , 1.3416407864999])
         assert_array_almost_equal(desired,y,decimal=12)
-        
 
 
-class test_moments(TestCase):
+
+class test_moments(ScipyTestCase):
     """
         Comparison numbers are found using R v.1.5.1
         note that length(testcase) = 4
@@ -775,7 +783,7 @@ class test_moments(TestCase):
     testmathworks = [1.165 , 0.6268, 0.0751, 0.3516, -0.6965]
     def check_moment(self):
         """
-        mean((testcase-mean(testcase))**power))"""
+        mean((testcase-mean(testcase))**power,axis=0),axis=0))**power))"""
         y = scipy.stats.moment(self.testcase,1)
         assert_approx_equal(y,0.0,10)
         y = scipy.stats.moment(self.testcase,2)
@@ -794,7 +802,7 @@ class test_moments(TestCase):
 
     def check_skewness(self):
         """
-            sum((testmathworks-mean(testmathworks))**3)/((sqrt(var(testmathworks)*4/5))**3)/5
+            sum((testmathworks-mean(testmathworks,axis=0))**3,axis=0)/((sqrt(var(testmathworks)*4/5))**3)/5
         """
         y = scipy.stats.skew(self.testmathworks)
         assert_approx_equal(y,-0.29322304336607,10)
@@ -804,8 +812,8 @@ class test_moments(TestCase):
         assert_approx_equal(y,0.0,10)
     def check_kurtosis(self):
         """
-            sum((testcase-mean(testcase))**4)/((sqrt(var(testcase)*3/4))**4)/4
-            sum((test2-mean(testmathworks))**4)/((sqrt(var(testmathworks)*4/5))**4)/5
+            sum((testcase-mean(testcase,axis=0))**4,axis=0)/((sqrt(var(testcase)*3/4))**4)/4
+            sum((test2-mean(testmathworks,axis=0))**4,axis=0)/((sqrt(var(testmathworks)*4/5))**4)/5
             Set flags for axis = 0 and
             fisher=0 (Pearson's defn of kurtosis for compatiability with Matlab)
         """
@@ -822,6 +830,4 @@ class test_moments(TestCase):
         assert_approx_equal(y,1.64)
 
 if __name__ == "__main__":
-    ScipyTest('stats.stats').run()
-
-
+    ScipyTest().run()
